module CG_mkl_mod
!Algebraically solves Ax=b where A is a symmetric matrix

	use sep
	

	implicit none
	include 'mkl_rci.fi'
	integer, private							  :: nmodel
  	character*8, parameter,dimension(8),private	  :: n_name = (/'n1','n2','n3','n4','n5','n6','n7','n8'/)
  	character*1024,private					  	  :: model_iter=" ", residual_iter=" ",cost_iter =" "
  	logical, private							  :: write_model=.false.
	
	
	contains
	
	
  
  		subroutine cg_mkl_init(nmodel_in,n_model_in,model_iter_in,residual_iter_in,cost_iter_in)
  			integer, intent(in)		:: nmodel_in
  			integer, intent(in)		:: n_model_in(7)
  			character*1024			:: model_iter_in,cost_iter_in,residual_iter_in
  			integer					:: idim
  	
  			model_iter = model_iter_in
  			cost_iter = cost_iter_in
  			residual_iter = residual_iter_in
  			nmodel = nmodel_in
  			do idim = 1,nmodel
				call to_history(n_name(idim),n_model_in(idim),model_iter)
			enddo
	
			write_model=.true.
			
		end subroutine cg_mkl_init
				
		subroutine cg_mkl(Fop,model_in,known_vector,niter)
		
		
			interface  
      			integer function Fop(adj,add,m,d)
        			real,dimension(:) :: m,d
        			logical,intent(in):: adj,add
      			end function 
   			 end interface 
   			 
			!input variables
			real,allocatable,dimension(:)		:: model_in, known_vector
			integer								:: niter
			
			!Internal variables
			integer								:: n 			! model size
			double precision, allocatable		:: x(:) 		! approximate solution
			double precision, allocatable		:: b(:)			! right-hand side of equation 
			integer								:: RCI_request 	! variable for handling RCs
			integer								:: ipar(128) 	! Integer parameters
			double precision					:: dpar(128) 	! Double precision parameters
			double precision, allocatable		:: tmp(:,:)		! temporary array for computation
			integer								:: itercount, stat
			real,allocatable,dimension(:)		:: model_out, gradient
			integer								:: wc, srite
			double precision					:: normalize
			real								:: rnorm
			
			
			
			
			n = size(model_in)
			!allocating
			allocate(x(n),b(n)); x = 0.; b = 0.
  			allocate(tmp(n,3)); tmp = 0.
  			allocate(model_out(n),gradient(n)); model_out = 0.; gradient = 0.
			
			!putting known_vector to internal array
			gradient = known_vector
			b = gradient
			
			
			call dcg_init(n, x, b, RCI_request, ipar, dpar, tmp)
  			!Change parameters to your inversion flavor
  	
  			!Check variables for inversion
  			call dcg_check(n, x, b, RCI_request, ipar, dpar, tmp)
  			if (RCI_request .ne. 0 ) call erexit("Not passing dcg_check")
			!If the starting model is equal zero
			itercount = 0

  	
  			!Running the solver
  			do 
  				call dcg(n, x, b, RCI_request, ipar, dpar, tmp)
  				if (RCI_request.eq.0) then !Apply the operator
  					write(0,*)"Solution found"
  					exit	
  				elseif(RCI_request.eq.1)then
  					write(0,*)"Applying the symmetric forward"
  					model_in = tmp(1:n,1)
  					if (any(model_in .ne. 0.0)) then
  						stat = Fop(.false., .false., model_in, model_out)
  					endif
  					tmp(1:n,2) = model_out
  				elseif(RCI_request.eq.2) then !Stop test
  					call dcg_get(n, x, b, RCI_request, ipar, dpar, tmp, itercount)
  					!writing residual and norm of the residual
  					if(itercount .ne. 0) then
  						call to_history(n_name(nmodel+1), itercount,residual_iter)
						call to_history(n_name(nmodel+1), itercount,model_iter)
  						call to_history("n1", itercount,cost_iter)
  						gradient = tmp(:,3) !converting to single precision
  						if (write_model) then
  							if(itercount .eq. 1) normalize = sum(tmp(:,3)*tmp(:,3))
  							rnorm = sum(tmp(:,3)*tmp(:,3))/normalize*100.0
  							write(0,*)"Residual norm percentage:",rnorm
  							wc = srite(residual_iter,gradient,4*n)
  							wc = srite(cost_iter,rnorm,4)
							!writing approximate solution
							gradient = x; !converting to single precision
							wc = srite(model_iter,gradient,4*n)
						endif
  					endif
  					write(0,*)"Checking for stop, at iter:", itercount
  					if(itercount .ge. niter) then
  						write(0,*)"Max number of iterations reached: Stopping solver"
  						exit
  					endif
  				elseif(RCI_request.eq.3) then ! preconditioner
  					write(0,*)"Applying preconditioner (if present)"
  					cycle
  				elseif(RCI_request.lt.0) then
  					write(0,*)"Error in running the inversion: negative flag from solver",RCI_request
  					exit
  				else
  					write(0,*)"Unknown flag: stopping"
  					exit
  				endif
  			enddo
  			
  			!Placing solution to input-output array
  			model_in = x
  			
  			!Deallocating temporary arrays
  			deallocate(gradient,model_out)
  			deallocate(tmp)
  			deallocate(b,x)
  			
  			
		end subroutine cg_mkl


end module CG_mkl_mod