!!$=head1 NAME
!!$
!!$Deriv1ord_1th - function to compute first-order derivative on wavelet (acoustic-elastic compatibility)
!!$
!!$=head1 SYNOPSIS
!!$
!!$Deriv1ord_1th.x < input.H > output.H
!!$
!!$=head1 DESCRIPTION
!!$
!!$Program to compute first-order derivative on wavelet (central difference 1th order)
!!$
!!$=head1 PARAMETERS
!!$
!!$=over 3
!!$
!!$=item from par
!!$
!!$
!!$
!!$=back
!!$
!!$
!!$=cut
!%
program Deriv1ord_1th

	!Used modules
	use sep

	!Variable declaration
	implicit none
	!Input variables
	integer  		    											:: n1,pad=1
	real									        				:: o1, d1
	real										        			:: C1t
	!Internal variables
	real, dimension(:), allocatable	                                                               			:: input,tmp
	integer											        	        :: wc, srite, rc, sreed, it

	call sep_init (SOURCE)
	!reading from input file and writing header to output file
	call from_history("n1",n1)
	call from_history("o1",o1)
	call from_history("d1",d1)
	call to_history("n1",n1)
	call to_history("o1",o1)
	call to_history("d1",d1)
	!Finite difference stencil
	C1t=(1.0)/(2.0*d1)


	!allocate memory for random number
	allocate(input(n1),tmp(1-pad:n1+pad))
	tmp = 0.0
	!Reading input wavelet
	rc=sreed("in", input, 4*n1)

	do it = 1,n1
		tmp(it) = C1t*(input(it+1)-input(it-1)) 
	enddo


	wc=srite("out", tmp(1:n1), 4*n1)

	!closing statements
	deallocate(tmp,input)
	call sep_close()
	call exit (0)

end program Deriv1ord_1th
