!!$=head1 NAME
!!$
!!$DotProd - program that computes dot product of files for Dot Product test purpose
!!$
!!$=head1 SYNOPSIS
!!$
!!$DotProd.x m_dim= d_dim= model1= model2= data1= data2= > /dev/null 
!!$
!!$=head1 DESCRIPTION
!!$
!!$Program for creating an output file containing random numbers
!!$
!!$=head1 PARAMETERS
!!$
!!$=over 3
!!$
!!$=item from par
!!$
!!$B<integer> I<m_dim> - number of dimension of the model files (max 5 dimensional file)
!!$
!!$B<integer> I<d_dim> - number of dimension of the data files (max 5 dimensional file)
!!$
!!$B<filename> I<model1> - random model
!!$
!!$B<filename> I<model2> - output model of adjoint operator
!!$
!!$B<filename> I<data1> - output data of forward operator
!!$
!!$B<filename> I<data2> - random data
!!$
!!$
!!$=back
!!$
!!$
!!$=cut
!%
program DotProd

	!Used modules 
  	use sep
  
  	!Variable declaration
  	implicit none
  	!Input variables
  	integer  		    				:: m_dim
  	integer  		    				:: d_dim
  	character*256						:: file_tag
  	!Internal variables
  	real, dimension(:), allocatable		:: mm1, mm2, dd1, dd2
  	real*8, dimension(:), allocatable	:: mm1d, mm2d, dd1d, dd2d
  	real*8								:: dotm, dotd
  	integer								:: n1, n2, n3, n4, n5

  	call sep_init (SOURCE)
  	!reading from parameters
	call from_par ("m_dim",m_dim)
	call from_par ("d_dim",d_dim)

	!Allocating and reading model files
	file_tag = "model1"
	
	select case(m_dim)
		case(1)
			call from_aux (file_tag,"n1",n1)
			allocate(mm1(n1))
		case(2)
			call from_aux (file_tag,"n1",n1)
			call from_aux (file_tag,"n2",n2)
			allocate(mm1(n1*n2))
		case(3)
			call from_aux (file_tag,"n1",n1)
			call from_aux (file_tag,"n2",n2)
			call from_aux (file_tag,"n3",n3)
			allocate(mm1(n1*n2*n3))
		case(4)
			call from_aux (file_tag,"n1",n1)
			call from_aux (file_tag,"n2",n2)
			call from_aux (file_tag,"n3",n3)
			call from_aux (file_tag,"n4",n4)
			allocate(mm1(n1*n2*n3*n4))
		case(5)
			call from_aux (file_tag,"n1",n1)
			call from_aux (file_tag,"n2",n2)
			call from_aux (file_tag,"n3",n3)
			call from_aux (file_tag,"n4",n4)
			call from_aux (file_tag,"n5",n5)
			allocate(mm1(n1*n2*n3*n5))
	end select
	mm1 = 0.0
	call sep_read(mm1,"model1")
	
	file_tag = "model2"
	
	select case(m_dim)
		case(1)
			call from_aux (file_tag,"n1",n1)
			allocate(mm2(n1))
		case(2)
			call from_aux (file_tag,"n1",n1)
			call from_aux (file_tag,"n2",n2)
			allocate(mm2(n1*n2))
		case(3)
			call from_aux (file_tag,"n1",n1)
			call from_aux (file_tag,"n2",n2)
			call from_aux (file_tag,"n3",n3)
			allocate(mm2(n1*n2*n3))
		case(4)
			call from_aux (file_tag,"n1",n1)
			call from_aux (file_tag,"n2",n2)
			call from_aux (file_tag,"n3",n3)
			call from_aux (file_tag,"n4",n4)
			allocate(mm2(n1*n2*n3*n4))
		case(5)
			call from_aux (file_tag,"n1",n1)
			call from_aux (file_tag,"n2",n2)
			call from_aux (file_tag,"n3",n3)
			call from_aux (file_tag,"n4",n4)
			call from_aux (file_tag,"n5",n5)
			allocate(mm2(n1*n2*n3*n5))
	end select
	mm2 = 0.0
	call sep_read(mm2,"model2")
	
	!Allocating and reading data files
	file_tag = "data1"
	select case(d_dim)
		case(1)
			call from_aux (file_tag,"n1",n1)
			allocate(dd1(n1))
		case(2)
			call from_aux (file_tag,"n1",n1)
			call from_aux (file_tag,"n2",n2)
			allocate(dd1(n1*n2))
		case(3)
			call from_aux (file_tag,"n1",n1)
			call from_aux (file_tag,"n2",n2)
			call from_aux (file_tag,"n3",n3)
			allocate(dd1(n1*n2*n3))
		case(4)
			call from_aux (file_tag,"n1",n1)
			call from_aux (file_tag,"n2",n2)
			call from_aux (file_tag,"n3",n3)
			call from_aux (file_tag,"n4",n4)
			allocate(dd1(n1*n2*n3*n4))
		case(5)
			call from_aux (file_tag,"n1",n1)
			call from_aux (file_tag,"n2",n2)
			call from_aux (file_tag,"n3",n3)
			call from_aux (file_tag,"n4",n4)
			call from_aux (file_tag,"n5",n5)
			allocate(dd1(n1*n2*n3*n5))
	end select
	dd1 = 0.0
	call sep_read(dd1,"data1")
	file_tag = "data2"
	
	select case(d_dim)
		case(1)
			call from_aux (file_tag,"n1",n1)
			allocate(dd2(n1))
		case(2)
			call from_aux (file_tag,"n1",n1)
			call from_aux (file_tag,"n2",n2)
			allocate(dd2(n1*n2))
		case(3)
			call from_aux (file_tag,"n1",n1)
			call from_aux (file_tag,"n2",n2)
			call from_aux (file_tag,"n3",n3)
			allocate(dd2(n1*n2*n3))
		case(4)
			call from_aux (file_tag,"n1",n1)
			call from_aux (file_tag,"n2",n2)
			call from_aux (file_tag,"n3",n3)
			call from_aux (file_tag,"n4",n4)
			allocate(dd2(n1*n2*n3*n4))
		case(5)
			call from_aux (file_tag,"n1",n1)
			call from_aux (file_tag,"n2",n2)
			call from_aux (file_tag,"n3",n3)
			call from_aux (file_tag,"n4",n4)
			call from_aux (file_tag,"n5",n5)
			allocate(dd2(n1*n2*n3*n5))
	end select
	dd2 = 0.0
	call sep_read(dd2,"data2")
	allocate(mm1d(size(mm1)),mm2d(size(mm1)),dd1d(size(dd1)),dd2d(size(dd1)))
	mm1d = mm1
	mm2d = mm2
	dd1d = dd1
	dd2d = dd2
	dotm = sum(mm1d*mm2d)	
	dotd = sum(dd1d*dd2d)
	write(0,*)"Dot product model side = ", dotm
	write(0,*)"Dot product  data side = ", dotd
	write(0,*)"Abs Error =", dotm-dotd
	write(0,*)"Rel Error =", (dotm-dotd)/dotm
	
	
	!closing statements
	deallocate(dd2d,dd1d,mm2d,mm1d)
	deallocate(dd2,dd1,mm2,mm1)
	call sep_close()
	call exit (0)

end program DotProd