!Module that perfoms the 1D linear interpolation for propagation steps
!Model dimension (2) (irregular grid) (two steps)
!Data dimension (1)  (regular grid)	(single step)
module Lint_mod


  implicit none
  !Module variables
  real*8, allocatable, private	:: interp_filter(:,:)	
  integer, private				:: nz, nx, ny, it2
  
  
  contains

	!Initialization routine
	subroutine Lint_init(nz_in, nx_in, ny_in, nt_in, nw_in, desample)
		
		!Input variables	
		integer						:: nz_in, nx_in, ny_in, nt_in, nw_in, desample

		!Assigning the internal variables
		nz = nz_in
		nx = nx_in
		ny = ny_in
		allocate(interp_filter(desample+1,2))
 		do it2=1,desample+1
			interp_filter(it2,1)=1.-float((it2-1))/float(desample)
			interp_filter(it2,2)=1.-interp_filter(it2,1)
  		end do
  		interp_filter = interp_filter * sqrt(float(nw_in)/float(nt_in))

		return
	end subroutine Lint_init
	
	subroutine Lint_init2(it2_in)
		
		integer		:: it2_in
		
		it2 = it2_in
		return
	end subroutine Lint_init2

!  	subroutine Lint_call(adj,add,mm,dd)
!    	logical,intent(in)   	 :: adj,add
!   		real,dimension(:) 		 :: mm,dd
!    		call Lint_op(adj,add,mm,dd)
!  	end subroutine Lint_call
  	
  	
  	!Operator routine
	subroutine Lint_op(adj, add, mm1, mm2, dd)
	
		!Input-Output variables
		logical, intent(in)				:: adj, add
		real, dimension(nz,nx,ny)		:: mm1,mm2 !Irregular grid data (model)
		real, dimension(nz,nx,ny)		:: dd !Regular grid data (data)
		!Internal variables

    	if(adj) then !Adjoint
        	if (.not. add) then 
        		mm1 = 0. !Zero-out the model1
        		mm2 = 0. !Zero-out the model2
        	endif
		 	mm1(:,:,:) = mm1(:,:,:) + dd(:,:,:)*interp_filter(it2,1)
		 	mm2(:,:,:) = mm2(:,:,:) + dd(:,:,:)*interp_filter(it2,2)

		else	     !Forward
			if(.not. add) dd = 0. !Zero-out the data
			dd(:,:,:) = dd(:,:,:) + mm1(:,:,:)*interp_filter(it2,1) + mm2(:,:,:)*interp_filter(it2,2)
		
		endif
		
		return
	end subroutine Lint_op
	
	
	subroutine Lint_close()
		if(allocated(interp_filter)) deallocate(interp_filter)
		return
	end subroutine Lint_close


end module Lint_mod