!Module for the padding of the source and 3D nearest-neighbor interpolation with the adjoint such that truncating and 3D interpolating
!Model dimension (z,x,y,t)
!Data dimension (t)
!D(t) = Pad_near3D(Model(z,x,y,t))
!Model(z,x,y,t) = Pad_near3D*(D(t))
module Pad_near3D

  implicit none
  !Module variables
  real, private					:: x, y, z, x0, y0, z0, dx, dy, dz !position of the point and origin of the axes
  integer, private				:: nz,nx, ny, nt !number of time samples

  contains
	
	!Initialization routine
	subroutine Pad_near3D_init(nz_in, nx_in, ny_in, nt_in, z_in, x_in, y_in, z0_in, x0_in, y0_in, dz_in, dx_in, dy_in)
		
		!Input variables	
		real, intent(in)				:: z_in, x_in, y_in, z0_in, x0_in, y0_in, dz_in, dx_in, dy_in
		integer, intent(in)				:: nz_in, nx_in, ny_in, nt_in

		!Assigning the internal variables
		x = x_in
		y = y_in
		z = z_in
		x0 = x0_in
		y0 = y0_in
		z0 = z0_in
		dx = dx_in
		dy = dy_in
		dz = dz_in
		nx = nx_in
		ny = ny_in
		nz = nz_in
		nt = nt_in

		return
	end subroutine Pad_near3D_init

  	subroutine Pad_near3D_call(adj,add,mm,dd)
    	logical,intent(in)   		:: adj,add
   		real,dimension(:) 		 	:: mm,dd
    		call Pad_near3D_op(adj,add,mm,dd)
  	end subroutine Pad_near3D_call

	
	!Operator routine
	subroutine Pad_near3D_op(adj, add, mm, dd)
	
		!Input-Output variables
		logical, intent(in)								:: adj, add
		real, dimension(nz,nx,ny,nt), intent(inout)		:: mm !Padded data (z,x,y,t)
		real, dimension(nt), intent(inout)				:: dd !Data(t)
		!Internal variables
		integer											:: zi, xi, yi

		xi = 1. + (x-x0)/dx
		yi = 1. + (y-y0)/dy
		zi = 1. + (z-z0)/dz
    		if(adj) then !Adjoint
        		if (.not. add) mm = 0. !Zero-out the model
				mm(zi,xi,yi,:)       = mm(zi,xi,yi,:)       + dd(:)
		else	     !Forward
			if(.not. add) dd = 0. !Zero-out the data
				dd(:) = dd(:) + mm(zi,xi,yi,:)
		endif
		
		return
	end subroutine Pad_near3D_op
	

end module Pad_near3D




