!Module for padding the velocity model for the absorbing boundaries

module Pad_pert

  implicit none

  interface Pad_perturbation
    module procedure Pad_perturbation_next,Pad_perturbation_ext
  end interface

  contains

	subroutine Pad_perturbation_next(vel,vel_pad,nv_pad,free_in)
		!Input-output variables
	  	real, dimension(:,:,:), allocatable, intent(inout)		:: vel   !velocity model (single precision)
	  	real, dimension(:,:,:), intent(inout), pointer			:: vel_pad !Padded perturbation
  		integer, intent(in)										      :: nv_pad !Padding for the perturbation
      logical, intent(in), optional                         :: free_in !Logical defining whether a free surface is necessary
		!Internal variables
		integer													         :: nz, nx, ny, iz, ix, iy, buffer
      logical                                               :: free=.false.

      parameter( buffer=5) !Top padding for free surface if requested
      !Checking if free surface is requested
      if (present(free_in)) then
         free=free_in
      endif

		!Reading the model dimensions
		nz = size(vel,1)
		nx = size(vel,2)
		ny = size(vel,3)

		!Checking the model dimensions and decide if 2D or 3D
		if ((nz .eq. 1) .or. (nx .eq. 1) .or. (ny .eq. 1)) then

         !Allocating the wider velocity model (2D)
         if (free) then
            allocate(vel_pad(nz+buffer+nv_pad,nx+2*nv_pad,ny))
         else
            allocate(vel_pad(nz+2*nv_pad,nx+2*nv_pad,ny))
         endif
			vel_pad = 0.

         !Putting the original velocity in the wider model
         if (free) then
            !$OMP PARALLEL DO private(ix,iz)
		  	   do ix = 1,nx
               do iz = 1,nz
                  vel_pad(iz+buffer,ix+nv_pad,:) = vel(iz,ix,:)
		 	      enddo
			   enddo
			   !$OMP END PARALLEL DO
         else
            !$OMP PARALLEL DO private(ix,iz)
		  	   do ix = 1,nx
               do iz = 1,nz
                  vel_pad(iz+nv_pad,ix+nv_pad,:) = vel(iz,ix,:)
		 	      enddo
			   enddo
			   !$OMP END PARALLEL DO
         endif

         !Extending in the x
         if (free) then
            !$OMP PARALLEL DO private(ix,iz)
   			do ix = 1,nv_pad
   			   do iz = 1,nz
                  vel_pad(iz+buffer,ix,:)=vel(iz,1,:)
   				   vel_pad(iz+buffer,ix+nv_pad+nx,:)=vel(iz,nx,:)
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         else
   			!$OMP PARALLEL DO private(ix,iz)
   			do ix = 1,nv_pad
   			   do iz = 1,nz
                  vel_pad(iz+nv_pad,ix,:)=vel(iz,1,:)
   				   vel_pad(iz+nv_pad,ix+nv_pad+nx,:)=vel(iz,nx,:)
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         endif

			!Extending in the z
         if (free) then
            !$OMP PARALLEL DO private(ix,iz)
   			do ix = 1,nx+2*nv_pad
   			   do iz = 1,nv_pad
                  vel_pad(iz+nz+buffer,ix,:)=vel_pad(nz+buffer,ix,:)
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
            !$OMP PARALLEL DO private(ix,iz)
   			do ix = 1,nx+2*nv_pad
               do iz = 1,buffer
                  vel_pad(iz,ix,:)=vel_pad(buffer+1,ix,:)
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         else
            !$OMP PARALLEL DO private(ix,iz)
   			do ix = 1,nx+2*nv_pad
   			   do iz = 1,nv_pad
                  vel_pad(iz,ix,:)=vel_pad(nv_pad+1,ix,:)
                  vel_pad(iz+nz+nv_pad,ix,:)=vel_pad(nz+nv_pad,ix,:)
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         endif

		else

         !Allocating the wider velocity model
         if (free) then
            allocate(vel_pad(nz+buffer+nv_pad,nx+2*nv_pad,ny+2*nv_pad))
         else
   			allocate(vel_pad(nz+2*nv_pad,nx+2*nv_pad,ny+2*nv_pad))
         endif
			vel_pad = 0.

         !Putting the original velocity in the wider model
         if (free) then
            !$OMP PARALLEL DO private(iy,ix,iz)
   			do iy = 1,ny
   			   do ix = 1,nx
   			      do iz = 1,nz
                     vel_pad(iz+buffer,ix+nv_pad,iy+nv_pad) = vel(iz,ix,iy)
   			      enddo
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         else
   			!$OMP PARALLEL DO private(iy,ix,iz)
   			do iy = 1,ny
   			   do ix = 1,nx
   			      do iz = 1,nz
                     vel_pad(iz+nv_pad,ix+nv_pad,iy+nv_pad) = vel(iz,ix,iy)
   			      enddo
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         endif

			!Extending in the x
         if (free) then
            !$OMP PARALLEL DO private(iy,ix,iz)
   			do iy = 1,ny
   			   do ix = 1,nv_pad
   			      do iz = 1,nz
         				vel_pad(iz+buffer,ix,iy+nv_pad)=vel(iz,1,iy)
         				vel_pad(iz+buffer,ix+nv_pad+nx,iy+nv_pad)=vel(iz,nx,iy)
   			      enddo
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         else
   			!$OMP PARALLEL DO private(iy,ix,iz)
   			do iy = 1,ny
   			   do ix = 1,nv_pad
   			      do iz = 1,nz
         				vel_pad(iz+nv_pad,ix,iy+nv_pad)=vel(iz,1,iy)
         				vel_pad(iz+nv_pad,ix+nv_pad+nx,iy+nv_pad)=vel(iz,nx,iy)
   			      enddo
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         endif

			!Extending in the y
         if (free) then
            !$OMP PARALLEL DO private(iy,ix,iz)
   			do iy = 1,nv_pad
   			   do ix = 1,nx+2*nv_pad
   			      do iz = 1,nz
         				vel_pad(iz+buffer,ix,iy)=vel_pad(iz+buffer,ix,1+nv_pad)
         				vel_pad(iz+buffer,ix,iy+nv_pad+ny)=vel_pad(iz+buffer,ix,ny+nv_pad)
   			      enddo
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         else
   			!$OMP PARALLEL DO private(iy,ix,iz)
   			do iy = 1,nv_pad
   			   do ix = 1,nx+2*nv_pad
   			      do iz = 1,nz
         				vel_pad(iz+nv_pad,ix,iy)=vel_pad(iz+nv_pad,ix,1+nv_pad)
         				vel_pad(iz+nv_pad,ix,iy+nv_pad+ny)=vel_pad(iz+nv_pad,ix,ny+nv_pad)
   			      enddo
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         endif

			!Extending in the z
         if (free) then
            !$OMP PARALLEL DO private(iy,ix,iz)
   			do iy = 1,ny+2*nv_pad
   			   do ix = 1,nx+2*nv_pad
   			      do iz = 1,nv_pad
         				vel_pad(iz+buffer+nz,ix,iy)=vel_pad(nz+buffer,ix,iy)
   			      enddo
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
            !$OMP PARALLEL DO private(iy,ix,iz)
   			do iy = 1,ny+2*nv_pad
   			   do ix = 1,nx+2*nv_pad
   			      do iz = 1,buffer
         				vel_pad(iz,ix,iy)=vel_pad(1+nv_pad,ix,iy)
   			      enddo
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         else
   			!$OMP PARALLEL DO private(iy,ix,iz)
   			do iy = 1,ny+2*nv_pad
   			   do ix = 1,nx+2*nv_pad
   			      do iz = 1,nv_pad
         				vel_pad(iz,ix,iy)=vel_pad(1+nv_pad,ix,iy)
         				vel_pad(iz+nv_pad+nz,ix,iy)=vel_pad(nz+nv_pad,ix,iy)
   			      enddo
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         endif

		endif


		deallocate(vel)
		return
	end subroutine Pad_perturbation_next

  subroutine Pad_perturbation_ext(vel_in,vel_in_pad,nv_pad,hx_max,free_in)
		!Input-output variables
	  	real, dimension(:,:,:,:), allocatable, intent(inout)		:: vel_in   !velocity model (single precision)
	  	real, dimension(:,:,:,:), intent(inout), pointer			  :: vel_in_pad !Padded perturbation
  		integer, intent(in)										                  :: nv_pad,hx_max !Padding for the perturbation and number of samples on one side of the subsurface offset extension
      logical, intent(in), optional                           :: free_in !Logical defining whether a free surface is necessary
		!Internal variables
		integer													                          :: nz, nx, ny, nhx, iz, ix, iy, buffer
      logical                                                 :: free=.false.

      parameter( buffer=5) !Top padding for free surface if requested
      !Checking if free surface is requested
      if (present(free_in)) then
         free=free_in
      endif

		!Reading the model dimensions
		nz = size(vel_in,1)
		nx = size(vel_in,2)
		ny = size(vel_in,3)

		!Checking the model dimensions and decide if 2D or 3D
		if ((nz .eq. 1) .or. (nx .eq. 1) .or. (ny .eq. 1)) then

         !Allocating the wider velocity model (2D)
         if (free) then
            allocate(vel_in_pad(nz+buffer+nv_pad,nx+2*nv_pad,ny,-hx_max:hx_max))
         else
            allocate(vel_in_pad(nz+2*nv_pad,nx+2*nv_pad,ny,-hx_max:hx_max))
         endif
			vel_in_pad = 0.

         !Putting the original velocity in the wider model
         if (free) then
            !$OMP PARALLEL DO private(ix,iz)
		  	   do ix = 1,nx
               do iz = 1,nz
                  vel_in_pad(iz+buffer,ix+nv_pad,:,:) = vel_in(iz,ix,:,:)
		 	      enddo
			   enddo
			   !$OMP END PARALLEL DO
         else
            !$OMP PARALLEL DO private(ix,iz)
		  	   do ix = 1,nx
               do iz = 1,nz
                  vel_in_pad(iz+nv_pad,ix+nv_pad,:,:) = vel_in(iz,ix,:,:)
		 	      enddo
			   enddo
			   !$OMP END PARALLEL DO
         endif

         !Extending in the x
         if (free) then
            !$OMP PARALLEL DO private(ix,iz)
   			do ix = 1,nv_pad
   			   do iz = 1,nz
             vel_in_pad(iz+buffer,ix,:,:)=vel_in(iz,1,:,:)
   				   vel_in_pad(iz+buffer,ix+nv_pad+nx,:,:)=vel_in(iz,nx,:,:)
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         else
   			!$OMP PARALLEL DO private(ix,iz)
   			do ix = 1,nv_pad
   			   do iz = 1,nz
             vel_in_pad(iz+nv_pad,ix,:,:)=vel_in(iz,1,:,:)
   				   vel_in_pad(iz+nv_pad,ix+nv_pad+nx,:,:)=vel_in(iz,nx,:,:)
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         endif

			!Extending in the z
         if (free) then
            !$OMP PARALLEL DO private(ix,iz)
   			do ix = 1,nx+2*nv_pad
   			   do iz = 1,nv_pad
                  vel_in_pad(iz+nz+buffer,ix,:,:)=vel_in_pad(nz+buffer,ix,:,:)
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
            !$OMP PARALLEL DO private(ix,iz)
   			do ix = 1,nx+2*nv_pad
               do iz = 1,buffer
                  vel_in_pad(iz,ix,:,:)=vel_in_pad(buffer+1,ix,:,:)
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         else
            !$OMP PARALLEL DO private(ix,iz)
   			do ix = 1,nx+2*nv_pad
   			   do iz = 1,nv_pad
                  vel_in_pad(iz,ix,:,:)=vel_in_pad(nv_pad+1,ix,:,:)
                  vel_in_pad(iz+nz+nv_pad,ix,:,:)=vel_in_pad(nz+nv_pad,ix,:,:)
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         endif

		else

         !Allocating the wider velocity model
         if (free) then
            allocate(vel_in_pad(nz+buffer+nv_pad,nx+2*nv_pad,ny+2*nv_pad,-hx_max:hx_max))
         else
   			    allocate(vel_in_pad(nz+2*nv_pad,nx+2*nv_pad,ny+2*nv_pad,-hx_max:hx_max))
         endif
			   vel_in_pad = 0.

         !Putting the original velocity in the wider model
         if (free) then
            !$OMP PARALLEL DO private(iy,ix,iz)
   			do iy = 1,ny
   			   do ix = 1,nx
   			      do iz = 1,nz
                     vel_in_pad(iz+buffer,ix+nv_pad,iy+nv_pad,:) = vel_in(iz,ix,iy,:)
   			      enddo
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         else
   			!$OMP PARALLEL DO private(iy,ix,iz)
   			do iy = 1,ny
   			   do ix = 1,nx
   			      do iz = 1,nz
                     vel_in_pad(iz+nv_pad,ix+nv_pad,iy+nv_pad,:) = vel_in(iz,ix,iy,:)
   			      enddo
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         endif

			!Extending in the x
         if (free) then
            !$OMP PARALLEL DO private(iy,ix,iz)
   			do iy = 1,ny
   			   do ix = 1,nv_pad
   			      do iz = 1,nz
         				vel_in_pad(iz+buffer,ix,iy+nv_pad,:)=vel_in(iz,1,iy,:)
         				vel_in_pad(iz+buffer,ix+nv_pad+nx,iy+nv_pad,:)=vel_in(iz,nx,iy,:)
   			      enddo
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         else
   			!$OMP PARALLEL DO private(iy,ix,iz)
   			do iy = 1,ny
   			   do ix = 1,nv_pad
   			      do iz = 1,nz
         				vel_in_pad(iz+nv_pad,ix,iy+nv_pad,:)=vel_in(iz,1,iy,:)
         				vel_in_pad(iz+nv_pad,ix+nv_pad+nx,iy+nv_pad,:)=vel_in(iz,nx,iy,:)
   			      enddo
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         endif

			!Extending in the y
         if (free) then
            !$OMP PARALLEL DO private(iy,ix,iz)
   			do iy = 1,nv_pad
   			   do ix = 1,nx+2*nv_pad
   			      do iz = 1,nz
         				vel_in_pad(iz+buffer,ix,iy,:)=vel_in_pad(iz+buffer,ix,1+nv_pad,:)
         				vel_in_pad(iz+buffer,ix,iy+nv_pad+ny,:)=vel_in_pad(iz+buffer,ix,ny+nv_pad,:)
   			      enddo
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         else
   			!$OMP PARALLEL DO private(iy,ix,iz)
   			do iy = 1,nv_pad
   			   do ix = 1,nx+2*nv_pad
   			      do iz = 1,nz
         				vel_in_pad(iz+nv_pad,ix,iy,:)=vel_in_pad(iz+nv_pad,ix,1+nv_pad,:)
         				vel_in_pad(iz+nv_pad,ix,iy+nv_pad+ny,:)=vel_in_pad(iz+nv_pad,ix,ny+nv_pad,:)
   			      enddo
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         endif

			!Extending in the z
         if (free) then
            !$OMP PARALLEL DO private(iy,ix,iz)
   			do iy = 1,ny+2*nv_pad
   			   do ix = 1,nx+2*nv_pad
   			      do iz = 1,nv_pad
         				vel_in_pad(iz+buffer+nz,ix,iy,:)=vel_in_pad(nz+buffer,ix,iy,:)
   			      enddo
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
            !$OMP PARALLEL DO private(iy,ix,iz)
   			do iy = 1,ny+2*nv_pad
   			   do ix = 1,nx+2*nv_pad
   			      do iz = 1,buffer
         				vel_in_pad(iz,ix,iy,:)=vel_in_pad(1+nv_pad,ix,iy,:)
   			      enddo
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         else
   			!$OMP PARALLEL DO private(iy,ix,iz)
   			do iy = 1,ny+2*nv_pad
   			   do ix = 1,nx+2*nv_pad
   			      do iz = 1,nv_pad
         				vel_in_pad(iz,ix,iy,:)=vel_in_pad(1+nv_pad,ix,iy,:)
         				vel_in_pad(iz+nv_pad+nz,ix,iy,:)=vel_in_pad(nz+nv_pad,ix,iy,:)
   			      enddo
   			   enddo
   			enddo
   			!$OMP END PARALLEL DO
         endif

		endif

		deallocate(vel_in)
		return
	end subroutine Pad_perturbation_ext



end module Pad_pert
