!Module for creating a propagator operator for dot product purposes
!Input fine grid wavefield model(nz,nx,ny,nt)
!Output fine grid propagated wavefield data(nz,nx,ny,nt)
module Prop_test_mod

	use Prop_step

	implicit none
  	!Module variables
  	integer, private									:: nz, nx, ny, nt
  	real, allocatable, private							:: dummy(:,:,:), tmp(:,:,:,:)
  	
  	
	contains
	
	
		subroutine Prop_test_init(nz_in, nx_in, ny_in, nt_in, nv_pad_in, vel_in, dt_in, dz_in, dx_in, dy_in,twoD_in)
		
			!Input variables	
			integer, intent(in)						:: nz_in, nx_in, ny_in, nt_in, nv_pad_in
			real, dimension(:,:,:), pointer			:: vel_in
			real									:: dt_in, dz_in, dx_in, dy_in
			logical, intent(in)						:: twoD_in
		
			!Initialize stepper and module's variables
			call Prop_step_init(nz_in, nx_in, ny_in, nt_in, nv_pad_in, vel_in, dt_in, dz_in, dx_in, dy_in,twoD_in)
			nz = nz_in
			nx = nx_in
			ny = ny_in
			nt = nt_in
			
			if(.not. allocated(dummy)) allocate(dummy(nz,nx,ny),tmp(nz,nx,ny,nt))
			dummy = 0.; tmp = 0.
				
	
			return
		end subroutine Prop_test_init
	
	
		subroutine Prop_test_call(adj,add,mm,dd)
    		logical,intent(in)  :: adj,add
   			real,dimension(:)  	:: mm,dd
    			call Prop_test_op(adj,add,mm,dd)
  		end subroutine Prop_test_call
  	
	  	subroutine Prop_test_op(adj,add,mm,dd)
			!Input-Output variables
			logical,intent(in) 				:: adj,add
   			real,dimension(nz,nx,ny,nt)  	:: mm,dd
   			integer							:: it
   			
   			tmp = 0.
   			
   			if(adj) then !Adjoint
        		if (.not. add) then
        			mm = 0. !Zero-out the model
        		else
        			tmp = mm
        			mm = 0.
        		endif
        		
        		mm(:,:,:,nt) =  0. !Initial condition
				call Prop_step_op(adj, dummy, mm(:,:,:,nt), mm(:,:,:,nt-1))
				mm(:,:,:,nt-1) = mm(:,:,:,nt-1) + dd(:,:,:,nt)
				do it =nt-2,1,-1
					call Prop_step_op(adj, mm(:,:,:,it+2), mm(:,:,:,it+1), mm(:,:,:,it))
					mm(:,:,:,it) = mm(:,:,:,it) + dd(:,:,:,it+1)
				enddo
				
				mm = mm + tmp
				


			else	     !Forward
				if (.not. add) then
        			dd = 0. !Zero-out the data
        		else
        			tmp = dd
        			dd = 0.
        		endif 
				dd(:,:,:,1) = 0. !Initial condition
				call Prop_step_op(adj, dummy, dd(:,:,:,1), dd(:,:,:,2))
				dd(:,:,:,2) = dd(:,:,:,2) + mm(:,:,:,1)
				do it =3,nt
					call Prop_step_op(adj, dd(:,:,:,it-2), dd(:,:,:,it-1), dd(:,:,:,it))
					dd(:,:,:,it) = dd(:,:,:,it)  + mm(:,:,:,it-1)
				enddo
				dd = dd + tmp
			endif
			
			return	
		end subroutine Prop_test_op
	
	
	
		subroutine Prop_test_close()
			call Prop_step_close()
			if(allocated(dummy))deallocate(tmp,dummy)
			return
		end subroutine Prop_test_close




end module Prop_test_mod