!!$=head1 NAME
!!$
!!$Random - function for generating SEP format random files
!!$
!!$=head1 SYNOPSIS
!!$
!!$Random.x dim=[1] n*=[] o*=[] d*= scheme=[1] > out.H 
!!$
!!$=head1 DESCRIPTION
!!$
!!$Program for creating an output file containing random numbers
!!$
!!$=head1 PARAMETERS
!!$
!!$=over 3
!!$
!!$=item from par
!!$
!!$B<integer> I<dim> - [1] number of dimension of the output file (max 5 dimensional file)
!!$
!!$B<integer> I<n*> - number of elements per dimension (standard sep paramenters)
!!$
!!$B<real> I<d*> -  sampling for each dimension (standard sep paramenters)
!!$
!!$B<real> I<o*> -  origin for each dimension (standard sep paramenters)
!!$
!!$B<integer> I<scheme> - [1] 0-numbers between -1 to 1, 1- numbers between 0 and 1
!!$
!!$
!!$=back
!!$
!!$
!!$=cut
!%
program Random

	!Used modules 
  	use sep
  
  	!Variable declaration
  	implicit none
  	!Input variables
  	integer  		    				:: dim, n1, n2, n3, n4, n5, scheme
  	real								:: o1, o2, o3, o4, o5, d1, d2, d3, d4, d5
  	!Internal variables
  	real, dimension(:), allocatable		:: dd

  	call sep_init (SOURCE)
  	!reading from parameters
	call from_par ("dim",dim,1)
	call from_par ("scheme",scheme,1)
	
	select case(dim)
		case(1)
			call from_par ("n1",n1)
			call from_par ("o1",o1)
			call from_par ("d1",d1)
			call to_history("n1",n1)
			call to_history("o1",o1)
			call to_history("d1",d1)
			!allocate memory for random number
			allocate(dd(n1))
			dd = 0.0
		case(2)
			call from_par ("n1",n1)
			call from_par ("o1",o1)
			call from_par ("d1",d1)
			call to_history("n1",n1)
			call to_history("o1",o1)
			call to_history("d1",d1)
			call from_par ("n2",n2)
			call from_par ("o2",o2)
			call from_par ("d2",d2)
			call to_history("n2",n2)
			call to_history("o2",o2)
			call to_history("d2",d2)
			!allocate memory for random number
			allocate(dd(n1*n2))
			dd = 0.0
		case(3)
			call from_par ("n1",n1)
			call from_par ("o1",o1)
			call from_par ("d1",d1)
			call to_history("n1",n1)
			call to_history("o1",o1)
			call to_history("d1",d1)
			call from_par ("n2",n2)
			call from_par ("o2",o2)
			call from_par ("d2",d2)
			call to_history("n2",n2)
			call to_history("o2",o2)
			call to_history("d2",d2)
			call from_par ("n3",n3)
			call from_par ("o3",o3)
			call from_par ("d3",d3)
			call to_history("n3",n3)
			call to_history("o3",o3)
			call to_history("d3",d3)
			!allocate memory for random number
			allocate(dd(n1*n2*n3))
			dd = 0.0
		case(4)
			call from_par ("n1",n1)
			call from_par ("o1",o1)
			call from_par ("d1",d1)
			call to_history("n1",n1)
			call to_history("o1",o1)
			call to_history("d1",d1)
			call from_par ("n2",n2)
			call from_par ("o2",o2)
			call from_par ("d2",d2)
			call to_history("n2",n2)
			call to_history("o2",o2)
			call to_history("d2",d2)
			call from_par ("n3",n3)
			call from_par ("o3",o3)
			call from_par ("d3",d3)
			call to_history("n3",n3)
			call to_history("o3",o3)
			call to_history("d3",d3)
			call from_par ("n4",n4)
			call from_par ("o4",o4)
			call from_par ("d4",d4)
			call to_history("n4",n4)
			call to_history("o4",o4)
			call to_history("d4",d4)
			!allocate memory for random number
			allocate(dd(n1*n2*n3*n4))
			dd = 0.0
		case(5)
			call from_par ("n1",n1)
			call from_par ("o1",o1)
			call from_par ("d1",d1)
			call to_history("n1",n1)
			call to_history("o1",o1)
			call to_history("d1",d1)
			call from_par ("n2",n2)
			call from_par ("o2",o2)
			call from_par ("d2",d2)
			call to_history("n2",n2)
			call to_history("o2",o2)
			call to_history("d2",d2)
			call from_par ("n3",n3)
			call from_par ("o3",o3)
			call from_par ("d3",d3)
			call to_history("n3",n3)
			call to_history("o3",o3)
			call to_history("d3",d3)
			call from_par ("n4",n4)
			call from_par ("o4",o4)
			call from_par ("d4",d4)
			call to_history("n4",n4)
			call to_history("o4",o4)
			call to_history("d4",d4)
			call from_par ("n5",n5)
			call from_par ("o5",o5)
			call from_par ("d5",d5)
			call to_history("n5",n5)
			call to_history("o5",o5)
			call to_history("d5",d5)
			!allocate memory for random number
			allocate(dd(n1*n2*n3*n4*n5))
			dd = 0.0
	end select
	
	!change random number generator seed
	call random_seed()
	!fill with random number from 0  to 1
	call random_number(dd)
	!if scheme == 0 fill with random number from -1 to 1
	if (scheme .eq. 0) dd = 2.0*dd - 1.0
	call sep_write(dd)
	
	
	!closing statements
	deallocate(dd)
	call sep_close()
	call exit (0)

end program Random