!Module to run commands given from parameter files when forward and adjoint operators are given as executable.

module command_run_mod


	use sep

	implicit none
	character*1024,allocatable, private				:: command_fw(:), command_adj(:)
	character*1024, private							:: forward_file=' ', adjoint_file=' ' 
	character*8, parameter,dimension(7), private	:: n_name = (/'n1','n2','n3','n4','n5','n6','n7'/)
	character*8, parameter,dimension(7), private	:: d_name = (/'d1','d2','d3','d4','d5','d6','d7'/)
	character*8, parameter,dimension(7), private	:: o_name = (/'o1','o2','o3','o4','o5','o6','o7'/)
	character*1024, private							:: line_buffer=' ', input_file_fw=' ', output_file_fw=' ', input_file_adj=' ', output_file_adj=' '
	integer, private								:: nmodel, ndata !number of dimensions for the model and data space
	integer, private								:: n_model(7)=0, n_data(7)=0
	real, private									:: d_model(7)=0.0, o_model(7)=0.0, d_data(7)=0.0, o_data(7)=0.0
	integer, private								:: n_lines_fw, n_lines_adj
	real, allocatable, private						:: model_temp(:), data_temp(:)
	
	
	contains 
	
	
		subroutine command_run_init(forward_file_in,adjoint_file_in, input_file_fw_in, output_file_fw_in, input_file_adj_in, output_file_adj_in, nmodel_in, ndata_in, n_model_in, d_model_in, o_model_in, n_data_in, d_data_in, o_data_in)
			character*1024, intent(in)					:: forward_file_in, adjoint_file_in
			character*1024, intent(in)					:: input_file_fw_in, output_file_fw_in, input_file_adj_in, output_file_adj_in
			integer, intent(in)							:: nmodel_in, ndata_in
			integer										:: iostat, read_stat, open_stat, icommand
			integer, intent(in)							:: n_model_in(7), n_data_in(7)
			real, intent(in)							:: d_model_in(7), o_model_in(7), d_data_in(7), o_data_in(7)
			integer							 			:: stat, auxinout
			
			forward_file = forward_file_in
			input_file_fw = input_file_fw_in
			output_file_fw = output_file_fw_in
			input_file_adj = input_file_adj_in
			output_file_adj = output_file_adj_in
			adjoint_file = adjoint_file_in
			nmodel = nmodel_in
			ndata = ndata_in
			n_model = n_model_in
			d_model = d_model_in
			o_model = o_model_in
			n_data = n_data_in
			d_data = d_data_in
			o_data = o_data_in
			
			!Allocating memory for internal model and data array
			if(.not. allocated(model_temp)) allocate(model_temp(product(n_model(1:nmodel))))
			if(.not. allocated(data_temp)) allocate(data_temp(product(n_data(1:ndata))))
			model_temp = 0.0; data_temp = 0.0
			
			!FORWARD COMMANDS	
   			!Open the file forward
			open(file=forward_file,unit=22,form='formatted', status='old', iostat=open_stat)
  			if (open_stat .ne. 0 ) then
  				write(0,*) "Cannot read the input file for forward"
  				stop
  			endif
  			read_stat=0
			n_lines_fw=0
  			do while (read_stat == 0)
  				read(unit=22,fmt='(A)',iostat=read_stat) line_buffer
  				if (read_stat == 0) then
  					if (line_buffer(1:1)=='#' .or. line_buffer(2:2)=='#') cycle
        			read(line_buffer,*) n_lines_fw
					if(n_lines_fw .gt. 0) exit
  				endif
  			enddo
  			!Allocate the memory for the positions
			allocate(command_fw(n_lines_fw))
			command_fw = ' '
			!Reading the positions
			icommand=0
  			do while (read_stat == 0)
  				read(unit=22,fmt='(A)',iostat=read_stat) line_buffer
  				if (read_stat == 0) then
  					if (line_buffer(1:1)=='#' .or. line_buffer(2:2)=='#') cycle
       				icommand = icommand + 1
        			read(line_buffer,'(A)') command_fw(icommand)
  				endif
 			enddo
			close(22)
			
			!stat = auxinout(input_file)
			
			
			!ADJOINT COMMANDS	
   			!Open the file adjoint
			open(file=adjoint_file,unit=22,form='formatted', status='old', iostat=open_stat)
  			if (open_stat .ne. 0 ) then
  				write(0,*) "Cannot read the input file for adjoint"
  				stop
  			endif
  			read_stat=0
			n_lines_adj=0
  			do while (read_stat == 0)
  				read(unit=22,fmt='(A)',iostat=read_stat) line_buffer
  				if (read_stat == 0) then
  					if (line_buffer(1:1)=='#' .or. line_buffer(2:2)=='#') cycle
        			read(line_buffer,*) n_lines_adj
					if(n_lines_adj .gt. 0) exit
  				endif
  			enddo
  			!Allocate the memory for the positions
			allocate(command_adj(n_lines_adj))
			command_adj = ' '
			!Reading the positions
			icommand=0
  			do while (read_stat == 0)
  				read(unit=22,fmt='(A)',iostat=read_stat) line_buffer
  				if (read_stat == 0) then
  					if (line_buffer(1:1)=='#' .or. line_buffer(2:2)=='#') cycle
       				icommand = icommand + 1
        			read(line_buffer,'(A)') command_adj(icommand)
  				endif
 			enddo
			close(22)
			
			
			!stat = auxinout(output_file)
		
		end subroutine command_run_init
		
		subroutine command_run_call(adj,add,mm,dd)
			logical,intent(in)   	 :: adj,add
   			real,dimension(:) 		 :: mm,dd
    		call command_run_op(adj,add,mm,dd)
		end subroutine command_run_call
		
		
		!Added for compatibility with GEE solvers
		function command_run_fun_call(adj,add,mm,dd) result(stat)
			logical,intent(in)   	 :: adj,add
   			real,dimension(:) 		 :: mm,dd
   			integer					 :: stat
    		call command_run_op(adj,add,mm,dd)
    		stat = 0
		end function command_run_fun_call
		
		
		subroutine command_run_op(adj,add,mm,dd)
			logical,intent(in)   		 					 :: adj,add
   			real,dimension(product(n_model(1:nmodel))) 		 :: mm
   			real,dimension(product(n_data(1:ndata))) 		 :: dd
   			!Internal variables
			integer						 					 :: wc, srite, rc, sreed, sk, sseek, icommand, idim
   			
   		
  			
  			if(adj) then !Adjoint
        		if (.not. add) mm = 0. !Zero-out the model
        		!Writing input data to file
				wc=srite(input_file_adj,dd,4*product(n_data(1:ndata)))
				sk = sseek(input_file_adj,0,0)
				do idim = 1,ndata
					call to_history(n_name(idim),n_data(idim),input_file_adj)
					call to_history(d_name(idim),d_data(idim),input_file_adj)
					call to_history(o_name(idim),o_data(idim),input_file_adj)
				enddo
				!Running the commands
				do icommand = 1, n_lines_adj
					call system(command_adj(icommand))
					call system('sync') !Make sure to flush system buffer
				enddo
		        
		        !Reading the output file and adding to the internal model array
		        rc = sreed(output_file_adj,model_temp,4*product(n_model(1:nmodel)))
				sk = sseek(output_file_adj,0,0)
				mm = mm + model_temp					

			else	     !Forward
				if(.not. add) dd = 0. !Zero-out the data
				!Writing input model to file
				wc=srite(input_file_fw,mm,4*product(n_model(1:nmodel)))
				sk = sseek(input_file_fw,0,0)
				do idim = 1,nmodel
					call to_history(n_name(idim),n_model(idim),input_file_fw)
					call to_history(d_name(idim),d_model(idim),input_file_fw)
					call to_history(o_name(idim),o_model(idim),input_file_fw)
				enddo
				
				!Running the commands
				do icommand = 1, n_lines_fw
					call system(command_fw(icommand))
					call system('sync') !Make sure to flush system buffer
				enddo
		        
		        !Reading the output file and adding to the internal data array
		        rc = sreed(output_file_fw,data_temp,4*product(n_data(1:ndata)))
				sk = sseek(output_file_fw,0,0)
				dd = dd + data_temp
			endif
		
		end subroutine command_run_op
		
		
		subroutine command_run_close()
			if(allocated(command_adj)) deallocate(command_adj)
			if(allocated(command_fw)) deallocate(command_fw)
			if(allocated(data_temp)) deallocate(data_temp)
			if(allocated(model_temp)) deallocate(model_temp)
		end subroutine command_run_close



end module command_run_mod