!Module that perfoms the 1D linear inteporlation (it considers that the irregular points are inside the regular grid)
!Model dimension (t1) (regular grid)
!Data dimension (t2)  (irregular grid)
!D(t2) = interp1D(Model(t1))
!Model(t2) = interp1D*(D(t1))
module interp1D

  implicit none
  !Module variables
  real, private					:: od, om, sd, sm !Origin and sampling of the data and model
  integer, private				:: nd, nm 	  !number of samples of the data and the model

  contains
	
	!Initialization routine
	subroutine interp1D_init(od_in, om_in, sd_in, sm_in,nd_in, nm_in)
		
		!Input variables	
		real						:: od_in, om_in, sd_in, sm_in
		integer						:: nd_in, nm_in

		!Assigning the internal variables
		od = od_in
		om = om_in
		sd = sd_in
		sm = sm_in
		od = od_in
		nd = nd_in
		nm = nm_in

		return
	end subroutine interp1D_init

  	subroutine interp1D_call(adj,add,mm,dd)
    	logical,intent(in)   	 :: adj,add
   		real,dimension(:) 		 :: mm,dd
    		call interp1D_op(adj,add,mm,dd)
  	end subroutine interp1D_call

	
	!Operator routine
	subroutine interp1D_op(adj, add, mm, dd)
	
		!Input-Output variables
		logical, intent(in)				:: adj, add
		real, dimension(nm)				:: mm !Regular grid data (model)
		real, dimension(nd)				:: dd !Irregular grid data (data)
		!Internal variables
		integer							:: id, im
		real							:: f, w

    		if(adj) then !Adjoint
        		if (.not. add) mm = 0. !Zero-out the model
		        !$OMP PARALLEL DO private(id,f,im,w)
			do id = 1, nd
				f = 1. + (od+(id-1)*sd-om)/sm; im = f; w = f - im
				mm(im)   = mm(im)   + dd(id)*(1.-w) 
				mm(im+1) = mm(im+1) + dd(id)*w
			enddo	
		        !$OMP END PARALLEL DO			

		else	     !Forward
			if(.not. add) dd = 0. !Zero-out the data
		        !$OMP PARALLEL DO private(id,f,im,w)
			do id = 1, nd
				f = 1. + (od+(id-1)*sd-om)/sm; im = f; w = f - im
				dd(id) = dd(id) + mm(im)*(1.-w) + mm(im+1)*w
			enddo				
		        !$OMP END PARALLEL DO
		endif
		
		return
	end subroutine interp1D_op
	

end module interp1D




