/*C version of propagation kernel*/
#include<omp.h>
#include <stdbool.h>
#include <math.h> 
#include <stdio.h>
#include <stdlib.h>

void propWaveC(float *pold, float *pcur, float *pnew, float *vsq, float *d1a, float *d2a, float *d3a, float d0, int *b,int *e,int b1, int b2){
int i1,i2,i3;
int i;
float d11,d12,d13,d14,d15;
float d21,d22,d23,d24,d25;
float d31,d32,d33,d34,d35;

d11=d1a[0]; d12=d1a[1]; d13=d1a[2]; d14=d1a[3]; d15=d1a[4];
d21=d2a[0]; d22=d2a[1]; d23=d2a[2]; d24=d2a[3]; d25=d2a[4];
d31=d3a[0]; d32=d3a[1]; d33=d3a[2]; d34=d3a[3]; d35=d3a[4];


//#pragma omp parallel for default(shared) private(i3,i2,i1,i)
for(i3=b[2]; i3 <= e[2]; i3++){
  for(i2=b[1]; i2 <= e[1]; i2++){
    for(i1=b[0],i=b[0]+i2*b1+i3*b2; i1 <= e[0]; i1++,i++){
      pnew[i]=vsq[i]*(
        d11*(pcur[i-1]+pcur[i+1])+
        d12*(pcur[i-2]+pcur[i+2])+
        d13*(pcur[i-3]+pcur[i+3])+
        d14*(pcur[i-4]+pcur[i+4])+
        d15*(pcur[i-5]+pcur[i+5])+
        d21*(pcur[i-1*b1]+pcur[i+1*b1])+
        d22*(pcur[i-2*b1]+pcur[i+2*b1])+
        d23*(pcur[i-3*b1]+pcur[i+3*b1])+
        d24*(pcur[i-4*b1]+pcur[i+4*b1])+
        d25*(pcur[i-5*b1]+pcur[i+5*b1])+
        d31*(pcur[i-1*b2]+pcur[i+1*b2])+
        d32*(pcur[i-2*b2]+pcur[i+2*b2])+
        d33*(pcur[i-3*b2]+pcur[i+3*b2])+
        d34*(pcur[i-4*b2]+pcur[i+4*b2])+
        d35*(pcur[i-5*b2]+pcur[i+5*b2])+
        d0*pcur[i])+
       2*pcur[i]-pold[i];
    }
  }
}

}


int buildWork(int *szs,int *block, int *sect){
	int np = omp_get_num_threads();
	for(int i=0; i < 3; i++){
		bool good=false;
		while(!good){
			sect[i]=floor(szs[i]/block[i]);
			int extra = szs[i]-block[i]*sect[i];
			//printf ("extra is %d for %i", extra, i);
			if (extra < block[i]*0.2 && extra !=0){
				block[i]-=1;
			}
			else{
				good=true;
				if (extra>0){
					sect[i]+=1;
				}
			}
		}
	}
	return 1;
}



int calcRange(int *beg, int *ends, int *block, int *sects, int *b1, int *b2, int *b3, int *e1, int *e2, int *e3){
//CHECK THE LOOPS
	for(int i3=0; i3 < sects[2]; i3++){
		for(int i2=0; i2 < sects[1]; i2++){
			for(int i1=0,i=i2*sects[0]+i3*sects[1]; i1 < sects[0]; i1++, i++){
				b1[i]=fmin(ends[0],(i1-1)*block[0]+beg[0]);
				b2[i]=fmin(ends[1],(i2-1)*block[1]+beg[1]);
				b3[i]=fmin(ends[2],(i3-1)*block[2]+beg[2]);
				e1[i]=fmin(ends[0],(i1)*block[0]+beg[0]);
				e2[i]=fmin(ends[1],(i2)*block[1]+beg[1]);
				e3[i]=fmin(ends[2],(i3)*block[2]+beg[2]);
			}
		}
	}
	return 1;
}



void propWaveMain(float *pold, float *pcur, float *pnew, float *vsq, float *d1a, float *d2a, float *d3a, float d0, int *b,int *e,int b1, int b2, int *block){
	int stat, iblock;
	int szs[3],sect[3],br[3],er[3];
	szs[0] = e[0] - b[0] + 1;
	szs[1] = e[1] - b[1] + 1;
	szs[2] = e[2] - b[2] + 1;
	int *b1s, *b2s, *b3s, *e1s, *e2s, *e3s;
	//construct the block size and # of sections
	stat = buildWork(szs,block,sect);
	//allocate memory
	int n_blocks=(sect[0])*(sect[1])*(sect[2]);
	b1s = (int*) malloc(n_blocks*sizeof(int));
	b2s = (int*) malloc(n_blocks*sizeof(int));
	b3s = (int*) malloc(n_blocks*sizeof(int));
	e1s = (int*) malloc(n_blocks*sizeof(int));
	e2s = (int*) malloc(n_blocks*sizeof(int));
	e3s = (int*) malloc(n_blocks*sizeof(int));
	stat = calcRange(b,e,block,sect,b1s,b2s,b3s,e1s,e2s,e3s);
	
	//loop over the blocks
	#pragma omp parallel for default(shared) private(iblock)
	for(iblock = 0; iblock < n_blocks; iblock++){
		br[0]=b1s[iblock]-1;br[1]=b1s[iblock]-1;br[2]=b1s[iblock]-1;
		er[0]=e1s[iblock]-1;er[1]=e1s[iblock]-1;er[2]=b1s[iblock]-1;
		propWaveC(pold,pcur,pnew,vsq,d1a,d2a,d3a,d0,br,er,b1,b2);
	}
	
	//free memory
	free(e3s);free(e2s);free(e1s);
	free(b3s);free(b2s);free(b1s);
}















