!Module that perfoms the 1D sinc inteporlation with 20 points (it considers that the irregular points are inside the regular grid)
!Model dimension (t1) (irregular grid)
!Data dimension (t2)  (regular grid)
!D(t2) = sinc1D(Model(t1))
!Model(t2) = sinc1D*(D(t1))
module sinc1D

  implicit none
  !Module variables
  real, private									:: od, om, sd, sm !Origin and sampling of the data and model
  integer, private								:: nd, nm 	  !number of samples of the data and the model
  integer, dimension(21),private				:: relind =[-10:10]
  real, dimension(:), allocatable, private		:: tmp
  real, private									:: pi
  integer, private								:: len_ind, mid

  parameter(pi=4.D0*DATAN(1.D0))

  contains
	
	!Initialization routine
	subroutine sinc1D_init(od_in, om_in, sd_in, sm_in,nd_in, nm_in)
		
		!Input variables	
		real						:: od_in, om_in, sd_in, sm_in
		integer						:: nd_in, nm_in
		
		!Assigning the internal variables
		od = od_in
		om = om_in
		sd = sd_in
		sm = sm_in
		od = od_in
		nd = nd_in
		nm = nm_in
		len_ind = size(relind)
		mid = len_ind/2 + 1
		if (.not. allocated(tmp)) allocate(tmp(1-len_ind/2:nd+len_ind/2))
		tmp = 0.

		return
	end subroutine sinc1D_init

  	subroutine sinc1D_call(adj,add,mm,dd)
    		logical,intent(in)  	:: adj,add
   			real,dimension(:)  		:: mm,dd
    		call sinc1D_op(adj,add,mm,dd)
  	end subroutine sinc1D_call

	
	!Operator routine
	subroutine sinc1D_op(adj, add, mm, dd)
	
		!Input-Output variables
		logical, intent(in)					:: adj, add
		real, dimension(nm)					:: mm !Irregular grid data (model)
		real, dimension(nd)					:: dd !Regular grid data (data)
		!Internal variables
		integer								:: id, im, first, last
		real								:: indf, w
		real, dimension(len_ind)			:: weight

    	if(adj) then !Adjoint
        	if (.not. add) mm = 0. !Zero-out the model
        	tmp = 0.
        	tmp(1:nd) = dd(:)
        	do im = 1, nm
				indf = 1. + (om+(im-1)*sm-od)/sd; id = indf; w = id - indf
				weight(:) = sin(pi*(real(relind(:))+w)*sd)/(pi*(real(relind(:))+w)*sd)
				if(w .eq. 0.) weight(mid)=1. ! avoid 0./0.
				mm(im) = mm(im) + sum(weight(:)*tmp(relind(:)+id))			
			enddo			
			
		else	     !Forward
			if(.not. add) dd = 0. !Zero-out the data
			do im = 1, nm
				indf = 1. + (om+(im-1)*sm-od)/sd; id = indf; w = id - indf
				weight(:) = sin(pi*(real(relind(:))+w)*sd)/(pi*(real(relind(:))+w)*sd)
				if(w .eq. 0.) weight(mid)=1. ! avoid 0./0.
				tmp(relind(:)+id) = weight(:)*mm(im)	
				first = max(1,minval(relind(:)+id))
				last = min(nd,maxval(relind(:)+id))
				dd(first:last) = dd(first:last) + tmp(first:last)
			enddo						
		endif
		
		return
	end subroutine sinc1D_op
	
	
	subroutine sinc1D_close()
		if (allocated(tmp)) deallocate(tmp)
	end subroutine sinc1D_close
	

end module sinc1D




