module solver_mod  
! 0 = F m - d
  use sep
  
  implicit none
  integer, private							  :: nmodel
  character*8, parameter,dimension(8),private :: n_name = (/'n1','n2','n3','n4','n5','n6','n7','n8'/)
  character*1024,private					  :: model_iter=" ", cost_iter =" "
  logical, private							  :: write_model=.false.
  contains
  
  subroutine solver_tiny_init(nmodel_in,n_model_in,model_iter_in,cost_iter_in)
  	integer, intent(in)		:: nmodel_in
  	integer, intent(in)		:: n_model_in(7)
  	character*1024			:: model_iter_in,cost_iter_in
  	integer					:: idim
  	
  	model_iter = model_iter_in
  	cost_iter = cost_iter_in
  	nmodel = nmodel_in
  	do idim = 1,nmodel
		call to_history(n_name(idim),n_model_in(idim),model_iter)
	enddo
	
	write_model=.true.
  	
  
  end subroutine
  
  subroutine solver_tiny( m,d, Fop, stepper, niter, m0,resd)
    optional ::                                     m0,resd
!--------------------begin definitions -------
    interface  
      integer function Fop(adj,add,m,d)
        real,dimension(:) :: m,d
        logical,intent(in):: adj,add
      end function 
      integer function stepper(forget,m,g,rr,gg)
        real,dimension(:) ::          m,g,rr,gg    
        logical           ::   forget
      end function
    end interface 
    real, dimension(:),    intent(in)  :: d       ! data
    real, dimension(:),    intent(out) :: m       ! model
    real, dimension(:),    intent(in)  :: m0      ! initial model
    integer,               intent(in)  :: niter   ! number of iterations
    integer                            :: iter    ! iteration number
    real, dimension(size( m))          :: g       ! gradient (dm)
    real, dimension(size( d)),target   :: rr      ! data residual ( vector)
    real, dimension(:),       pointer  :: rd      ! data residual (pointer)
    real, dimension(size( d)),target   :: gg      ! conj grad     ( vector)
    real, dimension(:),       pointer  :: gd      ! conj grad     (pointer)
    integer                            :: stat    ! status flag
    real, dimension(:),    intent(out) :: resd    ! residual
    logical                            :: forget
    integer							   :: wc, srite
    rd => rr(1:size( d))
    gd => gg(1:size( d))
!--------------------------begin initialization -----------
    rd = -d                               ! Rd =      - d
    m  = 0
    if (present( m0)) then
      m = m0                              !  m =        m0
      if(any(m .ne. 0.0))then
      	stat = Fop(.false.,.true.,m,rd)     ! Rd = Rd + F m0
      endif
    end if
    write(0,*) "       iter      rd*rd            g*g"
!--------- begin iterations -----------
    do iter = 1,niter  
      stat = Fop( .true.,.false.,g,rd)        !  g =  F' Rd
      stat = Fop(.false.,.false.,g,gd)        ! Gd =  F  g
      stat = stepper(forget, m,g, rr,gg)     ! m = m+dm; R = R + dR
      write(0,*) iter, sum(rd*rd), sum(g*g)
      if (write_model) then
      	!writing inverted model as iterating
      	wc = srite(model_iter,m,4*size(m))
      	call to_history(n_name(nmodel+1),iter,model_iter)
      	!writing cost function as iteratingb
      	wc = srite(cost_iter,sum(rd*rd),4)
      	call to_history("n1",iter,cost_iter)
      endif
    end do 
    if (present( resd)) then
      resd = rd
    end if
  end subroutine 
end module 
