#!/usr/bin/env python3.4

#Script to generate streamer geometry files
import sys,os
sys.path.append(os.environ.get('REPOSITORY')+"/python_solver/python_modules")
import sep_util as sep
from sep_python import from_cmd_line
import numpy as np


if __name__ == '__main__':
    if(len(sys.argv) == 1):
        print("NAME \n")
        print("	Geometry_streamer - Streamer geometry file generator for 2D models \n")
        print("SYNOPSIS")
        print("	Geometry_streamer.py model= dx_sources= z_sources= ox_source= dx_receivers= z_receivers= streamer_length= source_file= receiver_file_prefix=  \n")
        print("DESCRIPTION")
        print("	Generates parameter files contaning source and receiver positions for a streamer acquisition in the entire provided model.")
        print(" Note that it will not truncate the streamer length. Sampling and length units must be consistent with model file.\n")
        print("INPUT PARAMETERS")
        print("	model - char")
        print("		[none]: Seplib file containing sampling of the model considered for the propagation. Second axis should be x-axis.\n")
        print("	dx_sources - float")
        print("		[none]: Source interval. \n")
        print("	ox_source - float")
        print("		[none]: Source first x position. \n")
        print("	z_sources - float")
        print("		[none]: Source depth. \n")
        print("	dx_receivers - float")
        print("		[none]: Receiver interval. \n")
        print("	z_receivers - float")
        print("		[none]: Streamer depth. \n")
        print("	streamer_length - float")
        print("		[none]: Streamer length. \n")
        print("	source_file - char")
        print("		[none]: Source position file name. \n")
        print("	receiver_file_prefix - char")
        print("		[none]: Receiver position file prefix. The program will add the suffix _shotNUMBER.p, where NUMBER is the shot number for the given streamer position. \n")
    else:
        model_file=from_cmd_line("model")
        _,ax_model=sep.read_file(model_file)
        nx = ax_model[1][0]
        ox = ax_model[1][1]
        dx = ax_model[1][2]
        max_x=(nx-1)*dx+ox
        #Acqusition parameters
        dx_sources=from_cmd_line("dx_sources",conv="float")
        ox_source=from_cmd_line("ox_source",conv="float")
        z_sources=from_cmd_line("z_sources",conv="float")

        dx_receivers=from_cmd_line("dx_receivers",conv="float")
        z_receivers=from_cmd_line("z_receivers",conv="float")
        streamer_length=from_cmd_line("streamer_length",conv="float")

        #Source and receiver parameter files
        source_file=from_cmd_line("source_file")
        receiver_file_prefix=from_cmd_line("receiver_file_prefix")
        #Moving the source and checking if it is within model boundaries
        x_source = ox_source
        xs_tofile=np.empty(0) #Array containing source positions
        nrec=np.floor(streamer_length/abs(dx_receivers)+1)
        #Array containing relative positions of the receiver streamer
        rec_streamer_pos=np.linspace(0.,streamer_length,nrec)

        while (ox<=x_source<=max_x):
            xs_tofile=np.append(xs_tofile,x_source)
            #Shifting receiver position
            rec_pos=x_source+np.sign(dx_receivers)*rec_streamer_pos
            x_source+=dx_sources
            #Finding elements within model boundaries
            rec_pos=rec_pos[np.where(ox<=rec_pos)] #Left boundary
            rec_pos=rec_pos[np.where(rec_pos<=max_x)] #Right boundary
            if(rec_pos.shape[0]==0):
                #If not receivers are within the bounds then remove given source and try next source
                xs_tofile=xs_tofile[1:]
                continue
            #Writing given receiver file
            with open(receiver_file_prefix+"_shot%s.p"%(xs_tofile.shape[0]),"w") as f:
                f.write("#Number of receivers\n")
                f.write("%s \n"%rec_pos.shape[0])
                f.write("#Device positions (x,y,z)[m]\n")
                for ix_rec in rec_pos:
                    f.write("%s %s %s \n"%(ix_rec,0.0,z_receivers))

        #Writing sources' position file
        with open(source_file,"w") as f:
            f.write("#Number of sources\n")
            f.write("%s \n"%xs_tofile.shape[0])
            f.write("#Device positions (x,y,z)[m]\n")
            for ix_sou in xs_tofile:
                f.write("%s %s %s \n"%(ix_sou,0.0,z_sources))




















#
