#!/usr/bin/python
import commands,os,sys
import pbs_util
import sepbase
from os.path import abspath
import re
from math import ceil,floor
import datetime

def get_min_max_pos(par_file,index=0,ndev=None):
  """Get maximum and minimum position from a parameter file"""
  pos_x=[]
  pos_y=[]
  string_expression = re.compile("([-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?) ([-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?) ([-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?)")
  with open(par_file,"r") as f:
    for line in f:
  		find=string_expression.search(line)
      #Appending x and y position
  		if find:
  			pos_x.append(float(find.group(1))); pos_y.append(float(find.group(3)))
  if(ndev==None):
    max_pos=[max(pos_x[index:]),max(pos_y[index:])] #x and y
    min_pos=[min(pos_x[index:]),min(pos_y[index:])]
  else:
    max_pos=[max(pos_x[index:index+ndev+1]),max(pos_y[index:index+ndev+1])] #x and y
    min_pos=[min(pos_x[index:index+ndev+1]),min(pos_y[index:index+ndev+1])]
  return max_pos,min_pos

def find_walltime(delta_time=None):
  """Function to convert a time interval from seconds to wall time for PBS (i.e., HH:MM:SS)"""
  if(delta_time!=None):
    time_string=str(datetime.timedelta(seconds=delta_time))
    #Check if time is more than a day (because walltime can be more than 24 hours)
    if("day" in time_string):
      days_int=int(time_string.split(",")[0].split()[0])
      hh = int(time_string.split(",")[1].split(":")[0])
      mm = time_string.split(",")[1].split(":")[1]
      ss = time_string.split(",")[1].split(":")[2]
      hh += days_int*24
      wall_time = "%s:%s:%s"%(hh,mm,ss)
    else:
	  wall_time="0"+time_string
	  print("WARNING! Walltime of %s provided (hh:mm:ss)"%(wall_time))
  else:
    wall_time=None
  return wall_time

#Why Ginsu? Look on Wikipedia!
def ginsu(source_pos,rec_pos,model,padding,shot_index=0,nshots=None):
  """Function to return commands to window (x,y) portion of model or pad with zeros it based on source and receiver position files"""
  #Model related parameters
  assert(model!=None),"Error! Input model is None!"
  nx=int(sepbase.get_sep_his_par(model,"n2"))
  ox=float(sepbase.get_sep_his_par(model,"o2"))
  dx=float(sepbase.get_sep_his_par(model,"d2"))
  ny=int(sepbase.get_sep_his_par(model,"n3"))
  oy=float(sepbase.get_sep_his_par(model,"o3"))
  dy=float(sepbase.get_sep_his_par(model,"d3"))
  max_x_model=ox+(nx-1)*dx
  max_y_model=oy+(ny-1)*dy
  #Computing padding necessary
  shift_pad_x=padding*dx
  shift_pad_y=padding*dy
  #Getting maximum and minimum extend to be used during propagation
  max_pos_shot,min_pos_shot=get_min_max_pos(source_pos,shot_index,nshots)
  max_pos_rec,min_pos_rec=get_min_max_pos(rec_pos)
  #Finding min and max between receivers and shots (make the max and min falling onto a grid point)
  max_pos_x=ceil(max([max_pos_shot[0],max_pos_rec[0]])/dx+1)*dx #Because max in Window does not include if point falls onto the grid
  max_pos_y=ceil(max([max_pos_shot[1],max_pos_rec[1]])/dy+1)*dy
  min_pos_x=floor(min([min_pos_shot[0],min_pos_rec[0]])/dx)*dx
  min_pos_y=floor(min([min_pos_shot[1],min_pos_rec[1]])/dy)*dy
  #Verifying if shots and receivers are within the model
  assert(max_pos_x+shift_pad_x<=max_x_model),"Maximum x coordinate outsite of model in use: max_pos=%s, max_model=%s"%(max_pos_x,max_x_model)
  assert(min_pos_x-shift_pad_x>=ox),"Minimum x coordinate outsite of model in use: min_pos=%s, min_model=%s"%(min_pos_x,ox)
  if(not ny==1):
    assert(max_pos_y+shift_pad_y<=max_y_model),"Maximum y coordinate outsite of model in use: max_pos=%s, max_model=%s"%(max_pos_y,max_y_model)
    assert(min_pos_y-shift_pad_y>=oy),"Minimum y coordinate outsite of model in use: min_pos=%s, min_model=%s"%(min_pos_y,oy)
  #Creating commands to cutpad model to minimum size necessary for propagation
  #input and output models to be added outside
  if(not ny==1):
    cmd_window = "Window3d max2=%s min2=%s max3=%s min3=%s squeeze=0 "%(model,max_pos_x+shift_pad_x,min_pos_x-shift_pad_x,max_pos_y+shift_pad_y,min_pos_y-shift_pad_y)+"<%s >%s\n"
    nx_pad_beg=int((min_pos_x-shift_pad_x-ox)/dx)
    nx_pad_end=int((max_x_model-(max_pos_x+shift_pad_x))/dx)+1
    ny_pad_beg=int((min_pos_y-shift_pad_y-oy)/dy)
    ny_pad_end=int((max_y_model-(max_pos_y+shift_pad_y))/dy)+1
    cmd_pad = "Pad beg2=%s end2=%s beg3=%s end3=%s "%(nx_pad_beg,nx_pad_end,ny_pad_beg,ny_pad_end)+"<%s >%s\n"
  else:
    cmd_window = "Window3d max2=%s min2=%s squeeze=0 "%(max_pos_x+shift_pad_x,min_pos_x-shift_pad_x)+"<%s >%s\n"
    nx_pad_beg=int((min_pos_x-shift_pad_x-ox)/dx)
    nx_pad_end=int((max_x_model-(max_pos_x+shift_pad_x))/dx)+1
    cmd_pad = "Pad beg2=%s end2=%s "%(nx_pad_beg,nx_pad_end)+"<%s >%s\n"

  return cmd_window,cmd_pad


class BatchTaskExecutor:
  '''Run a batch job, with automatic fault recovery,
  Basically, it performs jobs that can be parallellized in the granularity of individual shots,
  like wave-equation migration and tomography operators.
  '''

  def __init__(self,job_param_reader,pbs_submitter=None,max_time=None):
    import re,os
    self.job_param_reader = job_param_reader
    self.pbs_submitter = pbs_submitter
    self.max_time=None
    #Setting maximum time if provided
    if(max_time!=None):
      string_expression = re.compile(r"([0-9]*\.?[0-9]+?)([a-z])")
      find=string_expression.match(max_time)
      assert find, "Error! Maximum time provided not correct format (time digits and unit [s,m,h,d]): currently %s"%(max_time)
      self.max_time=float(find.group(1))
      #Checking format and converting in seconds
      unit=find.group(2)
      if(unit=='s'):
        self.max_time*=1.0
      elif(unit=='m'):
        self.max_time*=60.0
      elif(unit=='h'):
        self.max_time*=3600.0
      elif(unit=='d'):
        self.max_time*=8640.0
      else:
        assert False, "Error! Provide correct unit [s,m,h,d]: currently %s"%(unit)
    if self.pbs_submitter is None:  # Provide a default job submitter
      #self.pbs_submitter = pbs_util.PbsSubmitter(zip(job_param_reader.queues, job_param_reader.queues_cap), job_param_reader.total_jobs_cap)
      self.pbs_submitter = pbs_util.SubmitterFromParamReader(self.job_param_reader)
    #Checking if Batching system will be run PBS or is a PBS job itself with a preallocated nodes
    if("PBS_NODEFILE" in os.environ):
      with open(os.environ['PBS_NODEFILE']) as f:
        self.nodelist = f.readlines()
      self.nodelist = [x.strip() for x in self.nodelist]
    else:
      self.nodelist=None
    if(self.nodelist!=None):
      assert("PBS_O_WORKDIR" in os.environ), "Error! Submited batching system but problem retriving variable PBS_O_WORKDIR"
      #Changing directory to current original directory
      os.chdir(os.environ["PBS_O_WORKDIR"])
    return

  def LaunchBatchTask(self, prefix, batch_task_composer):
    param_reader = self.job_param_reader
    pbs_submitter = self.pbs_submitter
    #Setting maximum time
    pbs_submitter.max_time = self.max_time
    #Finding corresponding wall time
    pbs_submitter.walltime = find_walltime(self.max_time)
    #Setting additional parameters for submitter_ssh
    pbs_submitter.nodelist = self.nodelist
    # Initialize script_creators and PbsSubmitter.
    pbs_script_creator = pbs_util.PbsScriptCreator(param_reader)
    # Main submission loop.
    AllFilesComputed = False
    rounds = 0
    while not AllFilesComputed:
      if rounds >= 5:
        sepbase.err('!The batch_task %s has been iterated over %d rounds, but not fully completed. Check your job description!' % (prefix, rounds))
      rounds += 1
      if(self.nodelist==None):
        #Submit multiple job approach
        pbs_submitter.WaitOnAllJobsFinish(prefix+'-')
      else:
        #Submit single job approach
        pbs_submitter.WaitOnAllJobsFinishSSH()
      AllFilesComputed = True
      subjobids, subjobfns_list = batch_task_composer.GetSubjobsInfo()
      assert len(subjobids) == len(subjobfns_list)
      for subjobid, subjobfns in zip(subjobids,subjobfns_list):  # For each job
        need_recompute = False
        # Here check if the designated files have already been precomputed, if so, we can skip this job.
        for fn in subjobfns:
          file_error = pbs_util.CheckSephFileError(fn,False)
          if file_error == 0:
            print "Target file is good, skip: %s" % fn
          else:
            if rounds > 1:
              if file_error == 1:
              	#A file is missing, increasing number of jobs to run
            	total_num_jobs += 1
                print "! target file is invalid (NaN): %s" % fn
              else:
                print "! target file is missing or invalid (check binary part): %s" % fn
            need_recompute = True
            break
        if need_recompute:
          AllFilesComputed = False
          scripts, subjobname = batch_task_composer.GetSubjobScripts(subjobid)
          assert scripts is not None
          if not subjobname:  # Provide a default subjobname
            subjobname = "%s" % subjobid
          pbs_script_creator.CreateScriptForNewJob(subjobname)
          scripts.append(pbs_script_creator.CmdFinalCleanUpTempDir())
          if(self.nodelist==None):
            #Submit multiple job approach
            #Check if last job to be submitted
            if(subjobid == subjobids[-1]):
            	last_job = True
            else:
            	last_job = False
            pbs_submitter.SubmitJob(pbs_script_creator.AppendScriptsContent(scripts),last_job)
          else:
            #Submit single job approach where ssh is used to submit jobs to different machines
            pbs_submitter.SubmitJobSSH(pbs_script_creator.AppendScriptsContent(scripts))
        # end if need_recompute
      # end for subjobsinfo
    # end for while
    print "Batch Task %s finished!" % prefix
    return


class BatchTaskComposer:
  '''NOTICE: This is the interface that you (as a user) needs to program, which implements the batch task/job description.
  The way the batch task is defined is explained as follows (a migration example will be used in the explaination):
  1): For each batch task, it will be divided into multiple (>=1) subjobs.
      How the batch task will be divided is up to the design choice of the user.
      Each subjob is considered to be the workload submitted to a single computer node as one PBS job.
      In the example of WE-migration of 50 shots data. One possible way to divide this task is to break the 50 shots into 10 groups, and we create 10 subjobs, with each subjob being responsible for migrating the 5 shots within the corresponding shots group.
  2): For each subjob, the user is responsible to provide the actual computation recipe that will perform the expected work, in the form of shell script lines.
      For the migration example, the computation recipe for the first subjob (subjobid=0) can be sth like:
      ```
        mybin/Migration.x vel=vel.H <dat-shot0.H >img0.H
        mybin/Migration.x vel=vel.H <dat-shot1.H >img1.H
        mybin/Migration.x vel=vel.H <dat-shot2.H >img2.H
        mybin/Migration.x vel=vel.H <dat-shot3.H >img3.H
        mybin/Migration.x vel=vel.H <dat-shot4.H >img4.H
      ```
      Note that here all the boiler-plates in the original PBS script has been taken care by the framework, and the user does not need to worry about it.
  '''
  def __init__(self):
    '''Dummy constructor.'''
    return

  def GetSubjobScripts(self, subjobid):
    '''Implement this interface to specify the computation recipe for each subjob.
    Returns:
      scripts: a list of strings. Each strings should corresponds to one line of cmdline execution.
      jobname: the designated jobname, if set=None, then the framework will generate a jobname automatically based on subjobid.
    '''
    assert False, "!Implement me in the derived class."
    return

  def GetSubjobsInfo(self):
    '''Implement this interface to specify the subjobs description.
    The function should return two lists: subjobids and subjobfns.
    Returns:
      subjobids: is a list of unique interger ids, each one refers to a subjob.
      subjobfns: is a list of lists (string-type). Each sublist contains multiple (>=1) filenames that are the expected outcome of executing the corresponding subjob.
    In the migration example, subjobids=[0,1,2,3,...,9], and subjobfns will be sth like:
    [ [img0.H,img1.H,...img4.H], [img5.H,img6.H,...,img9.H], ... ]
    Basically, the list of filenames in subjobfns are the designated files that should be in place after that subjob finishes.
    '''
    assert False, "!Implement me in the derived class."
    return


class CombineTaskComposer(BatchTaskComposer):
  '''.'''
  def __init__(self, param_reader, input_seph_list, output_fn, combine_pars, datapath, initial_seph_domain_file):
    '''Constructor.'''
    BatchTaskComposer.__init__(self)
    self.param_reader = param_reader
    self.input_seph_list = input_seph_list
    self.output_fn = output_fn
    self.combine_pars = combine_pars
    self.datapath = datapath
    self.initial_seph_domain_file = initial_seph_domain_file
    return

  def GetSubjobScripts(self, subjobid):
    '''Return the scripts content for that subjob.'''
    assert subjobid == 0
    scriptor = pbs_util.JobScriptor(self.param_reader)
    scripts = []
    scripts.append(scriptor.CmdCombineMultipleOutputSephFiles(self.input_seph_list, self.output_fn, self.combine_pars, self.datapath, self.initial_seph_domain_file))
    return scripts,None

  def GetSubjobsInfo(self):
    '''Should return two lists, subjobids and subjobfns. In this case only one
    job and one file.
    '''
    return [0], [[self.output_fn]]
