#!/usr/bin/python
import copy
import os
import sepbase
import tempfile
import unittest
import re
import math

from batch_task_executor import *
import pbs_util


class AcousticBatchTaskComposer(BatchTaskComposer):
	def __init__(self, param_reader, njobs, nfiles_perjob, vel_in):
		BatchTaskComposer.__init__(self)
		self.njobs = njobs
		self.nfiles_perjob = nfiles_perjob
		# Create all subjobs info
		self.subjobids = range(0, self.njobs)
		self.subjobfns_list = [None]*self.njobs
		path_out = param_reader.path_out
		#Necessary to Window the model
		self.vel_mod=vel_in
		# Create files for each subjob.
		for i in range(0,self.njobs):
			nfiles = self.nfiles_perjob
			self.subjobfns_list[i] = [None]*nfiles
			for j in range(0,nfiles):
				self.subjobfns_list[i][j] = '%s/%s-%s-%s.H' % (path_out,param_reader.prefix,i,j)
		return

	def GetSubjobScripts(self, subjobid):
		'''Return the scripts content for that subjob.'''
		subjobfns = self.subjobfns_list[subjobid]
		scripts=[]
		# Build scripts that create the files designated in subjobfns
		jobname = subjobfns[0]
		jobname_nopath=jobname.split("/")[-1]
		shots=[]
		shots=(re.findall('\d+', jobname ))
		shot_num=int(shots[-2])
		args_acoustic=eq_args_from_cmdline

		#Fixing padding in case of windowing
		npad=100

		nshots_perjob=shots_per_jobs[shot_num]
		#define shot position file
		cmd = 'mkdir -p Par \n'
		scripts.append(cmd)
		if(int(args_acoustic['fixed_rec'])):
			cmd = 'rm -f Par/%s-%s.p \n' %(param_reader.prefix,shot_num)
			scripts.append(cmd)
			cmd = 'touch Par/%s-%s.p \n' %(param_reader.prefix,shot_num)
			scripts.append(cmd)
			cmd = 'echo %s >> Par/%s-%s.p \n' %(nshots_perjob,param_reader.prefix,shot_num)
			scripts.append(cmd)
			cmd = 'sed -n %s,%sp %s >> Par/%s-%s.p \n' %(first_shot_number[shot_num]+4,first_shot_number[shot_num]+4+nshots_perjob-1,args_acoustic['par_sou'],param_reader.prefix,shot_num)
			scripts.append(cmd)
			#Creating data for the given shot
			data_name = 'data-%s-%s.H'%(param_reader.prefix,shot_num)
			cmd = 'Window3d <%s n3=%s f3=%s > %s \n' %(args_acoustic['data'],nshots_perjob,first_shot_number[shot_num],data_name)
			scripts.append(cmd)
			#Window the model if requested
			if(int(args_acoustic['window_mod'])):
				CWD = os.getcwd()
				os.chdir(args_acoustic['path_out']) #Relative path from path_out
				cmd_window,cmd_pad=ginsu(args_acoustic['par_sou'],args_acoustic['par_rec'],self.vel_mod,npad,first_shot_number[shot_num],nshots_perjob)
				os.chdir(CWD) #Go back to MAIN directory
				#Window lame parameter file
				tmp_model_name="vel_windowed_%s"%jobname_nopath
				cmd = cmd_window%(self.vel_mod,tmp_model_name)
				scripts.append(cmd)
				args_acoustic['vel']=tmp_model_name
			#Running the executable
			tmp_model_file="tmp_"+jobname_nopath
			cmd = '../bin/Bornfast_hess_adj.x < %s wavelet=%s sources=Par/%s-%s.p recs=%s >%s verbose=1 dt=%s deltadata=%s threads=%s sqslow=%s vel_padded=%s hx_max=%s \n' %(args_acoustic['vel'],args_acoustic['wavelet'],param_reader.prefix,shot_num,args_acoustic['par_rec'],tmp_model_file,args_acoustic['dt'],data_name,args_acoustic['threads'],args_acoustic['sqslow'],args_acoustic['mod_padded'],args_acoustic['hx_max'])
			scripts.append(cmd)
			#Removing data file
			cmd = 'Rm %s \n' %(data_name)
			scripts.append(cmd)
			#Removing windowed files
			if(int(args_acoustic['window_mod'])):
				#Padding adjoint model
				cmd=cmd_pad%(tmp_model_file,jobname)
				scripts.append(cmd)
				cmd = "Rm %s %s \n"%(args_acoustic['vel'],tmp_model_file)
				scripts.append(cmd)
			else:
				cmd = "mv %s %s \n"%(tmp_model_file,jobname)
				scripts.append(cmd)
		else:
			#Variable containing temporary data
			jobname_tmp=[]
			for ishot in range(nshots_perjob):
				shot_index=first_shot_number[shot_num]+ishot+1
				rec_file=args_acoustic['par_rec'].replace("NUMBER",str(shot_index))
				jobname_tmp.append(jobname+str(shot_index))
				#define shot position file
				cmd = 'rm -f Par/%s-%s-%s.p \n' %(param_reader.prefix,shot_num,ishot)
				scripts.append(cmd)
				cmd = 'touch Par/%s-%s-%s.p \n' %(param_reader.prefix,shot_num,ishot)
				scripts.append(cmd)
				cmd = 'echo %s >> Par/%s-%s-%s.p \n' %(1,param_reader.prefix,shot_num,ishot)
				scripts.append(cmd)
				cmd = 'sed -n %s,%sp %s >> Par/%s-%s-%s.p \n' %(first_shot_number[shot_num]+4+ishot,first_shot_number[shot_num]+4+ishot,args_acoustic['par_sou'],param_reader.prefix,shot_num,ishot)
				scripts.append(cmd)
				#Creating data for the given shot
				data_name = 'data-%s-%s.H'%(param_reader.prefix,shot_num)
				cmd = 'Window3d <%s n3=%s f3=%s > %s \n' %(args_acoustic['data'],1,shot_index-1,data_name)
				scripts.append(cmd)
				#Window the model if requested
				if(int(args_acoustic['window_mod'])):
					CWD = os.getcwd()
					os.chdir(args_acoustic['path_out']) #Relative path from path_out
					cmd_window,cmd_pad=ginsu(args_acoustic['par_sou'],rec_file,self.vel_mod,npad,(shot_index-1),1)
					os.chdir(CWD) #Go back to MAIN directory
					#Window lame parameter file
					tmp_model_name="vel_windowed_%s"%jobname_nopath
					cmd = cmd_window%(self.vel_mod,tmp_model_name)
					scripts.append(cmd)
					args_acoustic['vel']=tmp_model_name
				#Running the executable
				cmd = '../bin/Bornfast_hess_adj.x < %s wavelet=%s sources=Par/%s-%s-%s.p recs=%s >%s verbose=1 dt=%s deltadata=%s threads=%s sqslow=%s vel_padded=%s hx_max=%s \n' %(args_acoustic['vel'],args_acoustic['wavelet'],param_reader.prefix,shot_num,ishot,rec_file,jobname_tmp[ishot],args_acoustic['dt'],data_name,args_acoustic['threads'],args_acoustic['sqslow'],args_acoustic['mod_padded'],args_acoustic['hx_max'])
				scripts.append(cmd)
				#Removing windowed files
				if(int(args_acoustic['window_mod'])):
					#Padding adjoint model
					tmp_model_file="tmp_"+jobname_nopath+"%s"%(ishot)
					cmd=cmd_pad%(jobname_tmp[ishot],tmp_model_file)
					scripts.append(cmd)
					#Copying the padded model to the job-file name
					cmd = "Rm %s; mv %s %s \n"%(jobname_tmp[ishot],tmp_model_file,jobname_tmp[ishot])
					scripts.append(cmd)
					cmd = "Rm %s \n"%(args_acoustic['vel'])
					scripts.append(cmd)
				#Removing temporary data files
				cmd = 'Rm %s \n' %(data_name)
				scripts.append(cmd)
			#Concatenate all data files for current job
			cmd = 'Cat3d %s > %s axis=4\n' %(' '.join(jobname_tmp),jobname)
			scripts.append(cmd)
			#Removing temporary data files
			cmd = 'Rm %s\n' %(' '.join(jobname_tmp))
			scripts.append(cmd)
		return scripts, str(subjobid)

	def GetSubjobsInfo(self):
		'''Should return two lists, subjobids and subjobfns.'''
		return copy.deepcopy(self.subjobids), copy.deepcopy(self.subjobfns_list)


if __name__ == '__main__':
	if(len(sys.argv) == 1):
		print "NAME \n"
		print "	BatchTaskExecutor_adjoint_acoustic_born - Batch task executor to run multiple shots on cluster \n"
		print "SYNOPSIS"
		print "	batch_task_executor_acoustic_born_adj.py pbs_template=tmpl.sh queues=default,sep queues_cap=20,20 path_out= prefix= nshots= nshots_perjob=1 vel=vel.H par_rec=./Par/rec_unif.p par_sou=./Par/source_unif.p wavelet=wavelet.H dt=0.0 sqslow=0 data= total_jobs_cap=100,100\n"
		print "DESCRIPTION"
		print "	Run multiple shots on any clusters running PBS for acoustic Born adjoint"
		print "INPUT PARAMETERS"
		print "	max_time= - int/float and char"
		print "		[none]: Maximum running time to complete all the jobs when all running. If reached, the submitter kills the jobs and restart them."
		print "		        Format: time digits and unit [s=seconds,m=minutes,h=hours,d=days] (e.g., 20.0h means 20 hours)\n"
		print "	pbs_template= - char"
		print "		[no default]: File containing pbs script template (i.e., file in which commands are added)\n"
		print "	queues= - char"
		print "		[no default]: Queues to use during job submissions\n"
		print "	queues_cap= - char"
		print "		[no default]: Maximum jobs allowed in Q status\n"
		print "	path_out= - char"
		print "		[no default]: Path in which all files are going to be written\n"
		print "	prefix= - char"
		print "		[no default]: Prefix to identify job and file names\n"
		print "	nshots= - int"
		print "		[no default]: Total number of shots to run\n"
		print "	nshots_perjob= - int"
		print "		[1]: Number of shots per job \n"
		print "	vel= - char"
		print "		[no default]: Velocity model file\n"
		print "	par_rec= - char"
		print "		[no default]: Parameter file containing receiver positions"
		print "		              If fixed_rec=0, this parameter should be a file name containing NUMBER to identify the shot number for each position file"
		print "		              (e.g., recs_shot1.p,recs_shots2.p,...; then par_rec=recs_shotNUMBER.p)"
		print "		              Note that the order in par_sou must be the same as the NUMBER in the receiver position files\n"
		print "	par_sou= - char"
		print "		[no default]: Parameter file containing source positions\n"
		print "	wavelet= - char"
		print "		[no default]: Wavelet file\n"
		print "	dt= - float"
		print "		[0.0]: Fine dt to use during propagation\n"
		print "	sqslow= - int"
		print "		[0]: If 1 output model perturbation is slowness squared\n"
		print "	hx_max= - int"
		print "		[0]: Number of element on one side of the subsurface x-axis extension (default no extension hx_max=0)\n"
		print "	total_jobs_cap= - int"
		print "		[100]: Total number of jobs per queue\n"
		print "	data= - char"
		print "		[no default]: File containing dataset\n"
		print "	threads= - int"
		print "		[24]:  Number of threads to use during computation\n"
		print "	mod_padded= - int"
		print "		[0]:  Model was padded (100 elements), useful for FWI\n"
		print "	fixed_rec= - int"
		print "		[1]:  Receiver geometry is fixed for all shots (e.g., node acquisition)\n"
		print "	window_mod= - int"
		print "		[0]:  Window the model for propagation based on shot and receiver positions (Note: program sets mod_padded=1)\n"
		print ""
	else:
		print "Run BatchTaskExecutor_acoustic_imaging with params:", sys.argv
		# Check basic environment variable setup.
		assert 'SEP' in os.environ, '!Environment var SEP is not set yet, should have your seplib enviroment set up first! Check paths like /opt/SEP/'
		#Parsing command line
		eq_args_from_cmdline,args = sepbase.parse_args(sys.argv)
		#Parsing command line for PBS related parameters
		param_reader = pbs_util.JobParamReader(eq_args_from_cmdline)


		#Parameters for running the jobs
		nshots=int(eq_args_from_cmdline['nshots'])
		if eq_args_from_cmdline.has_key('nshots_perjob'):
			nshots_perjob=int(eq_args_from_cmdline['nshots_perjob'])
		else:
			nshots_perjob = 1

		#Parsing max_time
		if eq_args_from_cmdline.has_key('max_time'):
			max_time_in=eq_args_from_cmdline['max_time']
		else:
			max_time_in = None

		if(not eq_args_from_cmdline.has_key('dt')):
			eq_args_from_cmdline['dt']=0.0
		if(not eq_args_from_cmdline.has_key('threads')):
			eq_args_from_cmdline['threads']=24
		if(not eq_args_from_cmdline.has_key('sqslow')):
			eq_args_from_cmdline['sqslow']=0
		if(not eq_args_from_cmdline.has_key('mod_padded')):
			eq_args_from_cmdline['mod_padded']=0
		if(not eq_args_from_cmdline.has_key('fixed_rec')):
			eq_args_from_cmdline['fixed_rec']=1
		if(not eq_args_from_cmdline.has_key('window_mod')):
			eq_args_from_cmdline['window_mod']=0
		if(not eq_args_from_cmdline.has_key('hx_max')):
			eq_args_from_cmdline['hx_max']=0
		if(int(eq_args_from_cmdline['window_mod'])): eq_args_from_cmdline['mod_padded']=1

		first_shot_number=range(0,nshots,nshots_perjob)
		njobs=len(first_shot_number)
		shots_per_jobs=[nshots_perjob]*(njobs-1)
		shots_per_jobs.append(nshots-first_shot_number[-1])

		#Creating job obj to run
		nfiles_perjob=1
		acoustic_btc = AcousticBatchTaskComposer(param_reader,njobs,nfiles_perjob,eq_args_from_cmdline['vel'])

		#Creating executor and launching jobs
		bte = BatchTaskExecutor(param_reader,max_time=max_time_in)
		prefix = param_reader.prefix
		bte.LaunchBatchTask(prefix, acoustic_btc)
		print 'Run Successful!'
