#!/usr/bin/python
import sep_python as sep
import sys

if __name__ == '__main__':
	if(len(sys.argv) == 1):
		print "NAME \n"
		print "	dvpdvsdrho2dmodrho - Convert perturbations in Vp/Vs/density to Lame/Mu/density \n"
		print "SYNOPSIS"
		print "	dvpdvsdrho2dmodrho.py vp0= vs0= rho0= pert_in= pert_out= adj=0 \n"
		print "DESCRIPTION"
		print "	Convert perturbations in Vp/Vs/density(m/s,m/s,kg/m^3) to Lame/Mu/density(GPa,Gpa,kg/m^3) with Forward and Adjoint operator for 2D files"
		print "INPUT PARAMETERS"
		print "	vp0= - char"
		print "		[no default]: Background Vp model (m/s)\n"
		print "	vs0= - char"
		print "		[no default]: Background Vs model (m/s)\n"
		print "	rho0= - char"
		print "		[no default]: Background Density model (kg/m^3)\n"
		print "	pert_in= - char"
		print "		[no default]: Input model perturbations \n"
		print "	pert_out= - char"
		print "		[no default]: Output model perturbations\n"
		print "	adj= - int"
		print "		[0]: If 0 run the FW, dVp/dVs/drho -> dlame/dmu/drho"
		print "		     If 1 run the ADJ, dlame/dmu/drho -> dVp/dVs/drho"
		print ""
	else:
		
		#Parsing command line
		vp0_file=sep.from_cmd_line("vp0") 
		vs0_file=sep.from_cmd_line("vs0") 
		rho0_file=sep.from_cmd_line("rho0") 
		pert_in=sep.from_cmd_line("pert_in") 
		pert_out=sep.from_cmd_line("pert_out") 
		adj_in=sep.from_cmd_line("adj",default=0,conv="int") 
		
		#Temporary files
		tmp_dvp_file=sep.tmp_file("tmp_dvp.H")
		tmp_dvs_file=sep.tmp_file("tmp_dvs.H")
		tmp_drho_file=sep.tmp_file("tmp_drho.H")
		tmp_dlame_file=sep.tmp_file("tmp_dlame.H")
		tmp_dmu_file=sep.tmp_file("tmp_dmu.H")
		if (adj_in == 0):
			#FW OPERATOR
			#Separating perturbations
			#dVp
			cmd="Window3d <%s n3=1 f3=0 >%s"%(pert_in,tmp_dvp_file); sep.RunShellCmd(cmd)
			#dVs
			cmd="Window3d <%s n3=1 f3=1 >%s"%(pert_in,tmp_dvs_file); sep.RunShellCmd(cmd)
			#drho
			cmd="Window3d <%s n3=1 f3=2 >%s"%(pert_in,tmp_drho_file); sep.RunShellCmd(cmd)
			#Computing new model perturbations
			#dlame
			cmd="Math file1=%s file2=%s file3=%s file4=%s file5=%s file6=%s exp='(file6*(file1*file1-2.0*file2*file2)-4.0*file2*file3*file5+2.0*file1*file3*file4)/1e9' >%s "%(vp0_file,vs0_file,rho0_file,tmp_dvp_file,tmp_dvs_file,tmp_drho_file,tmp_dlame_file)
			sep.RunShellCmd(cmd)
			#dmu
			cmd="Math file1=%s file2=%s file3=%s file4=%s exp='(file1*file1*file4+2.0*file1*file2*file3)/1e9' >%s"%(vs0_file,rho0_file,tmp_dvs_file,tmp_drho_file,tmp_dmu_file)
			sep.RunShellCmd(cmd)
			#Cat the new perturbations
			cmd="Cat3d %s %s %s axis=3 >%s"%(tmp_dlame_file,tmp_dmu_file,tmp_drho_file,pert_out)
			sep.RunShellCmd(cmd)
		else:
			#ADJ OPERATOR
			#Separating perturbations
			#dVp
			cmd="Window3d <%s n3=1 f3=0 >%s"%(pert_in,tmp_dlame_file); sep.RunShellCmd(cmd)
			#dVs
			cmd="Window3d <%s n3=1 f3=1 >%s"%(pert_in,tmp_dmu_file); sep.RunShellCmd(cmd)
			#drho
			cmd="Window3d <%s n3=1 f3=2 >%s"%(pert_in,tmp_drho_file); sep.RunShellCmd(cmd)
			#Computing new model perturbations
			#dVp
			cmd="Math file1=%s file2=%s file3=%s exp='2.0*file1*file2*file3/1e9' >%s"%(vp0_file,rho0_file,tmp_dlame_file,tmp_dvp_file)
			sep.RunShellCmd(cmd)
			#dVs
			cmd="Math file1=%s file2=%s file3=%s file4=%s exp='(-4.0*file1*file2*file3+2.0*file1*file2*file4)/1e9' >%s"%(vs0_file,rho0_file,tmp_dlame_file,tmp_dmu_file,tmp_dvs_file)
			sep.RunShellCmd(cmd)
			#drho'
			tmp_drho_out_file=sep.tmp_file("tmp_drho_prime.H")
			cmd="Math file1=%s file2=%s file3=%s file4=%s file5=%s file6=%s exp='((file1*file1-2.0*file2*file2)*file4+file2*file2*file5)/1e9+file6' >%s"%(vp0_file,vs0_file,rho0_file,tmp_dlame_file,tmp_dmu_file,tmp_drho_file,tmp_drho_out_file)
			sep.RunShellCmd(cmd)
			#Cat the new perturbations
			cmd="Cat3d %s %s %s axis=3 >%s"%(tmp_dvp_file,tmp_dvs_file,tmp_drho_out_file,pert_out)
			sep.RunShellCmd(cmd)
			sep.Rm(tmp_drho_out_file)
		
		#Removing temp files
		sep.Rm(tmp_dvp_file)
		sep.Rm(tmp_dvs_file)
		sep.Rm(tmp_drho_file)
		sep.Rm(tmp_dlame_file)
		sep.Rm(tmp_dmu_file)
		
	
		
		
		
		