#!/usr/bin/python
import copy
import os as os_sys
import sepbase
import tempfile
import unittest
import re
import math
import subprocess

from batch_task_executor import *
import pbs_util

# get the code directory path for the extended Born adjoint executable (contains at the end)
code_path = os_sys.environ['CODE_PATH']

### An example to define a batch task through inheriting the BatchTaskComposer class.
### IMPORTANT: Read /home/zyang03/script/batch_task_executor.py to learn the detailed specifications/explanations of the interfaces defined in BatchTaskComposer.

class NonlinearBatchTaskComposer(BatchTaskComposer):
  def __init__(self, param_reader, njobs, nfiles_perjob):
    BatchTaskComposer.__init__(self)
    self.njobs = njobs
    self.nfiles_perjob = nfiles_perjob
    # Create all subjobs info
    self.subjobids = range(0, self.njobs)
    self.subjobfns_list = [None]*self.njobs
    path_out = param_reader.path_out
    # Create files for each subjob.
    for i in range(0,self.njobs):
       nfiles = self.nfiles_perjob
       self.subjobfns_list[i] = [None]*nfiles
       for j in range(0,nfiles):
         self.subjobfns_list[i][j] = '%s/%s-%s-%s.H' % (path_out,param_reader.prefix,i,j)
    return

  def GetSubjobScripts(self,subjobid):
    '''Return the scripts content for that subjob.'''
    subjobfns = self.subjobfns_list[subjobid]
    print 'subjob id',subjobid
    print 'subjobfns',subjobfns
    scripts=[]

	# print jobname
    jobname = subjobfns[0]   
    print 'jobname is:',jobname 
    
	# print 
    shots=[]
    shots=(re.findall('\d+', jobname ))
    shots = shots[-2:]      
    print 'shots',shots

	# print shot number
    shot_num=int(shots[0])
    print 'shot nb',shot_num

    sub_shot=int(shots[1])
    print 'sub shot:',sub_shot    
    print 'nb of shots per job',nshots_perjob    

    # read from command line
    args_nonlinear=eq_args_from_cmdline
    
    # position of the first source for this shot
    print "source position for this shot [samples]",os_jobs[shot_num]    
    
	# slice the dataset and extract the specific shots for each job
    fsource = (shot_num)*nshots_perjob
    cmd1 = 'Window3d n3=%s f3=%s < %s > %s-temp%s.H \n' %(nshots_perjob,fsource,data,prefix,shot_num)
    print "command to slice dataset:",cmd1

	# define the command to launch the Born adjoint for each job
    cmd2 = code_path+'/Bin/born_ext_adj_dt2_main.x par=%s os=%s ns=%s vel=%s hzmax=%s hxmax=%s htmax=%s < %s-temp%s.H > %s \n' %(parfile,os_jobs[shot_num],nshots_perjob,vel,hzmax,hxmax,htmax,prefix,shot_num,jobname) 
    print "command to launch the Born adjoint executable:",cmd2

	# append both commands into "cmd"
    cmd = '%s; %s;' % (cmd1,cmd2)
    scripts.append(cmd)
    cmd= 'Rm %s-temp%s.H' %(prefix,shot_num)
    scripts.append(cmd)
    
    return scripts, str(subjobid)

  def GetSubjobsInfo(self):
    '''Should return two lists, subjobids and subjobfns.'''
    return copy.deepcopy(self.subjobids), copy.deepcopy(self.subjobfns_list)

if __name__ == '__main__':

  print "Run BatchTaskExecutor_test with params:", sys.argv

  # Check basic environment variable setup.
  assert 'SEP' in os_sys.environ, '!Environment var SEP is not set yet, should have your seplib enviroment set up first! Check paths like /opt/SEP/'
  eq_args_from_cmdline,args = sepbase.parse_args(sys.argv)
  param_reader = pbs_util.JobParamReader(eq_args_from_cmdline)

  print "eq_args_from_cmdline",eq_args_from_cmdline
  print "args",args

  # read number of shots
  nshots=int(eq_args_from_cmdline['nshots'])
  print 'nb of shots:',nshots

  # read nb of shots per job
  nshots_perjob=int(eq_args_from_cmdline['nshotjob'])
  print 'nb of shots per job:',nshots_perjob

  # read data file name
  data=eq_args_from_cmdline['data']
  print 'read data file name: OK'	

  # read velocity file name
  vel=eq_args_from_cmdline['vel']
  print 'read velocity file name: OK'

  # read the position of the first source (in nb of samples)
  os=int(eq_args_from_cmdline['os'])
  print 'position of the first source for the first shot os:',os

  # read distance between two sources (in nb of samples)
  ds=int(eq_args_from_cmdline['ds'])
  print 'distance between two consecutive sources:',ds,'[sample]'

  # read hzmax
  hzmax=int(eq_args_from_cmdline['hzmax'])
  print 'maximum half-offset in z:',hzmax,'[sample]'

  # read hxmax
  hxmax=int(eq_args_from_cmdline['hxmax'])
  print 'maximum half-offset in x:',hxmax,'[sample]'

  # read htmax
  htmax=int(eq_args_from_cmdline['htmax'])
  print 'maximum half time-lag:',htmax,'[sample]'

  # read parameter file
  parfile=eq_args_from_cmdline['par']
  print 'read parameter file'	
  
  # compute number of jobs needed
  njobs=nshots/nshots_perjob
  print 'total nb of jobs:',njobs  
  
  # generate list of all the starting source positions for all jobs
  os_jobs=range(os,os+nshots*ds,nshots_perjob*ds)
  print 'array containing all the position of the first sources:',os_jobs  

  # calls the constructor of NonlinearBatchTaskComposer (creates the object)
  nonlinear_btc = NonlinearBatchTaskComposer(param_reader,njobs,1)
  print nonlinear_btc.subjobfns_list

  # bte is an object that has a method that launches the jobs 
  # (the ones defined by nonlinear_btc)
  bte = BatchTaskExecutor(param_reader)
  prefix = param_reader.prefix
  
  # procedure that launched the job (goes into GetSubjobScripts)
  bte.LaunchBatchTask(prefix,nonlinear_btc)

  # creates the list of .H to be grouped (stores it into "args")
  _,test2 = nonlinear_btc.GetSubjobsInfo()
  test3 = [fn for fns in test2 for fn in fns]
  args=''
  for i in range(0,len(test3)):
    args = '%s %s' % (args,test3[i])
  print "list of .H to be cat'ed",args

  # define output name
  output_name='%s-final.H' % (prefix)  
  print "output name",output_name

  # define temporary file name 
  temporary_name = '%s-temporary-ext-born-adj.H' % (prefix)  

  # concatenate files  
  cmd1 = 'Cat3d axis=6 %s > %s' %(args,temporary_name)
  print "command that concatenates partial images:",cmd1
  subprocess.call(cmd1,shell=True)

  # stack files using incore code
  cmd2 = code_path+"/Bin/stack_batch.x < %s > %s" %(temporary_name,output_name)
  print "command that stacks partial images:",cmd2
  subprocess.call(cmd2,shell=True)

  # don't forget to delete the temporary files
  cmd3 = "Rm %s" %(temporary_name)
  print "command that deletes the temporary files:",cmd3    
  subprocess.call(cmd3,shell=True)

  print "ALL DONE"

  ########################################################################################################################
  # add files using Ettore's code
  #command1 = "/hpcdata/baga7o/code/acoustic/Scripts/Add_files.py %s %s" %(args,output_name)

  # define command that stacks all partial images
  #cmd1 = 'Cat3d axis=6 %s > temp.H;' %(args)
  #cmd2= 'Transp plane=26 < temp.H | Stack3d | Transp reshape=1,4,5 plane=23 > %s out=/scratch/gbarnier/%s@ ; Rm temp.H' % (output_name,output_name)

  #print "command that stacks partial images:",cmd1
  #print "command that stacks partial images:",cmd2  
  
  # launch command
  #subprocess.call(cmd1+cmd2,shell=True)  
  ########################################################################################################################

