#!/usr/bin/python
import copy
import os as os_sys
import sepbase
import tempfile
import unittest
import re
import math
import subprocess

from batch_task_executor import *
import pbs_util

# get the code directory path for the Born forward executable (contains / at the end)
code_path = os_sys.environ['CODE_PATH']

### An example to define a batch task through inheriting the BatchTaskComposer class.
### IMPORTANT: Read /home/zyang03/script/batch_task_executor.py to learn the detailed specifications/explanations of the interfaces defined in BatchTaskComposer.

class NonlinearBatchTaskComposer(BatchTaskComposer):
  def __init__(self, param_reader, njobs, nfiles_perjob):
    BatchTaskComposer.__init__(self)
    self.njobs = njobs
    self.nfiles_perjob = nfiles_perjob
    # Create all subjobs info
    self.subjobids = range(0, self.njobs)
    self.subjobfns_list = [None]*self.njobs
    path_out = param_reader.path_out
    # Create files for each subjob.
    for i in range(0,self.njobs):
       nfiles = self.nfiles_perjob
       self.subjobfns_list[i] = [None]*nfiles
       for j in range(0,nfiles):
         self.subjobfns_list[i][j] = '%s/%s-%s-%s.H' % (path_out,param_reader.prefix,i,j)
    return

  def GetSubjobScripts(self,subjobid):
    '''Return the scripts content for that subjob.'''
    subjobfns = self.subjobfns_list[subjobid]
    print 'subjob id',subjobid
    print 'subjobfns',subjobfns
    scripts=[]

	# print jobname
    jobname = subjobfns[0]   
    print 'jobname is:',jobname 
    
	# print 
    shots=[]
    shots=(re.findall('\d+', jobname ))
    shots = shots[-2:]        
    print 'shots',shots

	# print shot number
    shot_num=int(shots[0])
    print 'shot nb',shot_num

    sub_shot=int(shots[1])
    print 'sub shot:',sub_shot    
    print 'nb of shots per job',nshots_perjob    

    # read from command line
    args_nonlinear=eq_args_from_cmdline
    
    # position of the first source for this shot
    print "source position for this shot [samples]",os_jobs[shot_num]    
    
	# define the command to launch the forward linear modeling for each job
    cmd1 = code_path+'/Bin/mtfwi_linear_fwd_main.x par=%s os=%s ns=%s hzmax=%s hxmax=%s htmax=%s m0=%s < %s > %s \n' %(parfile,os_jobs[shot_num],nshots_perjob,hzmax,hxmax,htmax,m0,model,jobname) 
    print "command to launch the MTFWI linearized fwd code:",cmd1

    scripts.append(cmd1)
    return scripts, str(subjobid)

  def GetSubjobsInfo(self):
    '''Should return two lists, subjobids and subjobfns.'''
    return copy.deepcopy(self.subjobids), copy.deepcopy(self.subjobfns_list)

if __name__ == '__main__':

  print "Run BatchTaskExecutor_test with params:", sys.argv

  # Check basic environment variable setup.
  assert 'SEP' in os_sys.environ, '!Environment var SEP is not set yet, should have your seplib enviroment set up first! Check paths like /opt/SEP/'
  eq_args_from_cmdline,args = sepbase.parse_args(sys.argv)
  param_reader = pbs_util.JobParamReader(eq_args_from_cmdline)

  print "eq_args_from_cmdline",eq_args_from_cmdline
  print "args",args

  # read number of shots
  nshots=int(eq_args_from_cmdline['nshots'])
  print 'nb of shots:',nshots

  # read nb of shots per job
  nshots_perjob=int(eq_args_from_cmdline['nshotjob'])
  print 'nb of shots per job:',nshots_perjob

  # read data file name
  m0=eq_args_from_cmdline['m0']
  print 'read padded background model file name: OK'	

  # read data file name
  model=eq_args_from_cmdline['model']
  print 'read padded input model file name: OK'	

  # read the position of the first source (in nb of samples)
  os=int(eq_args_from_cmdline['os'])
  print 'position of the first source for the first shot os:',os

  # read distance between two sources (in nb of samples)
  ds=int(eq_args_from_cmdline['ds'])
  print 'distance between two consecutive sources:',ds,'[sample]'

  # read hzmax
  hzmax=int(eq_args_from_cmdline['hzmax'])
  print 'maximum half-offset in z:',hzmax,'[sample]'

  # read hxmax
  hxmax=int(eq_args_from_cmdline['hxmax'])
  print 'maximum half-offset in x:',hxmax,'[sample]'

  # read htmax
  htmax=int(eq_args_from_cmdline['htmax'])
  print 'maximum half time-lag:',htmax,'[sample]'
  
  # read parameter file
  parfile=eq_args_from_cmdline['par']
  print 'read parameter file'	
  
  # compute number of jobs needed
  njobs=nshots/nshots_perjob
  print 'total nb of jobs:',njobs  
  
  # generate list of all the starting source positions for all jobs
  os_jobs=range(os,os+nshots*ds,nshots_perjob*ds)
  print 'array containing all the position of the first sources:',os_jobs  

  # calls the constructor of NonlinearBatchTaskComposer (creates the object)
  nonlinear_btc = NonlinearBatchTaskComposer(param_reader,njobs,1)
  print nonlinear_btc.subjobfns_list

  # bte is an object that has a method that launches the jobs 
  # (the ones defined by nonlinear_btc)
  bte = BatchTaskExecutor(param_reader)
  prefix = param_reader.prefix
  
  # procedure that launched the job (goes into GetSubjobScripts)
  bte.LaunchBatchTask(prefix,nonlinear_btc)

  # creates the list of .H to be grouped (stores it into "args")
  _,test2 = nonlinear_btc.GetSubjobsInfo()
  test3 = [fn for fns in test2 for fn in fns]
  args=''
  for i in range(0,len(test3)):
    args = '%s %s' % (args,test3[i])
  print "list of .H to be cat'ed",args

  # define output name
  output_name='%s-final.H' % (prefix)  
  print "output name",output_name

  # define command that cats all shots
  cmd1 = 'Cat3d axis=3 %s > %s' %(args,output_name)
  print "command that concatenates all shots:",cmd1
  
  # launch command
  subprocess.call(cmd1,shell=True)  

  print "ALL DONE"
  
  
  
  
