#!/usr/bin/python
import sep_python as sep
import operator_obj as op 
import sys
import os as os_sys
from python_solver_outcore import write_log_file 
from timeit import default_timer as timer
import numpy as np 

##########################################################################################	
################################ MAIN ####################################################	
##########################################################################################	

if __name__ == '__main__':

	# parsing command line 
	if(len(sys.argv) == 1):
	
		print "##########################################################################################"
		print "################################# WELCOME TO E&G DISPLAY SCRIPT ##########################"
		print "##########################################################################################"		
		print "Welcome to the selfdoc of the e&g display script (2018). This script enables you to"
		print "display results from FWI, VPTFWI and linear inversions conducted using the e&g Python solver"
		print "(Biondi and Barnier, 2017). The script is named vptfwi_display_main.py."
		print "Please report any bug to Guillaume Barnier at barnier@gmail.com \n"

		print "##########################################################################################"
		print "#################################### CALLING THE SCRIPT ##################################"
		print "##########################################################################################"
		print "This is how you call the script in a Makefile:"
		print "vptfwi_display_main.py par=parfile.p \n" 

		print "##########################################################################################"
		print "################################### DESCRIPTION ##########################################"
		print "##########################################################################################"
		print "Three modes are available: linear, fwi, and vptfwi."
		print "For each of these modes, four components are available for display: \n"
		print "I.   Objective function"
		print "II.  Data residuals"
		print "III. Inverted models "	
		print "IV.  Gradients \n"
		print "Each of the display components have different features depending on the inversion type. \n"		

		print "##########################################################################################"				
		print "################################# GENERAL STUFF ##########################################"
		print "##########################################################################################\n"

		print "##################################### DESCRIPTION ########################################\n "
		print "These parameters are required for most display components. \n"

		print "##################################### PARAMETERS #########################################\n "
		print "mode - char"
		print "     [no default]: Choose the type of inversion you wish to analyze. Options are linear,"
		print "     fwi or vptfwi (no capital letters). \n"
		print "nickname - char "
		print "     [no default]: Choose a nickname for the diplay process (e.g., fwi1, vptfwi_35Hz,"
		print "     linear_inversion1). \n"		
		print "obj - char or int"
		print "     [False]: Set this flag to \"true\", \"y\", or \"1\" to display objective functions for"
		print "     the analyzed inversion. \n"		
		print "res - char or int"
		print "     [False]: Set this flag to \"true\", \"y\", or \"1\" to display data residuals for the"
		print "     analyzed inversion. \n"		
		print "mod - char or int"
		print "     [False]: Set this flag to \"true\", \"y\", or \"1\" to display inverted models for the"
		print "     analyzed inversion. \n"		
		print "grad - char or int"
		print "     [False]: Set this flag to \"true\", \"y\", or \"1\" to display gradients for the analyzed"
		print "     inversion. \n"				
		print "remove - char or int"
		print "     [False]: Set this flag to \"true\", \"y\", or \"1\" to remove the temporary files"
		print "     created in the local directory to display the results (can be useful if you want to"
		print "     generate figures for a report or presentation). \n"						
		print "main_wrk_dir - char"
		print "     [no default]: Name of the inversion directory. You can display the results from two"
		print "     different inversions (say your inversion crashed and you had to manually restart it in a"
		print "     different folder). Just type the name of the two main directories sparated by a comma."
		print "     For instance, main_wrk_dir=inversion1,inveersion1_continued \n"
		print "suffix - char"
		print "     [no default]: Name of the inversion suffix. You can display the results from two"
		print "     different inversions (say your inversion crashed and you had to manually restart it in a"
		print "     different folder). Just type the name of the two suffixes sparated by a comma."
		print "     For instance, suffix=suffix1,suffix1_continued. \n"
		print "prec - char or int"
		print "     [False]: Flag only used for linear regularized inversion. Note that the script "
		print "     automatically recognizes whether a regulairzation term was used during the inversion. \n"					
		print "data - char"
		print "		[no default]: Name of the data file used in the inversion. \n"						
		print "true_model - char"
		print "		[no default]: Name of the non-padded true model file. \n"								
		print "main_parfile - char"
		print "     [no default]: Name of the parfile containing dimension information (nz,nx,dz,dx,z_pad,x_pad)."
		print "     If not present, these parameters can be specified directly from the command line. \n"
		print "iteration_axis - int"
		print "     [6]: Iteration axis number. Only needed for extended linear inversions. \n"
		print "comment - char or int"
		print "     [True]: Set this flag to \"true\", \"y\", or \"1\" for comments from the display script.\n" 

		print "##########################################################################################"
		print "################################ I. OBJECTIVE FUNCTION ###################################"
		print "##########################################################################################\n"

		print "################################ DESCRIPTION AND REMARKS #################################"
		print "- For the FWI and linear modes, this feature allows you to display the scaled objective function(s)."		
		print "- For the FWI mode, you have the option to scale it by the first objective function value"
		print "or by 0.5 || d_obs ||^2. For the VPTFWI mode, you also have the option to overlay the value "
		print "that the FWI objective function would take for each inverted model during the inversion. \n"
		
		print "################################### PARAMETERS ###########################################"
		print "obj_grid - char"
		print "     [y]: Display grid for objective function plot. \n"		
		print "obj_min1 - float"
		print "     [0.0]: min1 for objective function plot. \n"
		print "obj_max1 - float"
		print "     [number of iteration]: max1 for objective function plot. \n"		
		print "obj_min2 - float"
		print "     [0.0]: min2 for objective function plot. \n"
		print "obj_max2 - float"
		print "     [1.0]: max2 for objective function plot. \n"				
		print "obj_d1num - float"
		print "     [5.0]: d1num for objective function plot. \n"						
		print "obj_d2num - float"
		print "     [0.1]: d2num for objective function plot. \n"								
		print "obj_legend - char"
		print "	    [y]: legend for objective function plot. \n"										
		print "obj_legendloc - char"
		print "     [y]: legendloc for objective function plot. \n"												
		print "obj_legendloc - char"
		print "     [no default]: legendloc for objective function plot. \n"	
		print "obj_title - char"
		print "     [\"obj function [nickname]\": legendloc for objective function plot. \n"	
		print "dat_norm - char or int"
		print "     [False]: Normalize the objective function by 0.5 || d_obs ||^2 instead of the first"
		print "     objective function value (default). \n"		
		print "fwi_obj - char or int"
		print "     [True]: Set this flag to \"true\", \"y\", or \"1\" to overlay the fwi sampled objective"
		print "     function with the VPTFWI objective functions (only for the VPTFWI mode). \n"		
		print "fwi_obj_scale - char or int"
		print "     [False]: Set this flag to \"true\", \"y\", or \"1\" to overlay the SCALED fwi sampled" 
		print "     objective function with the VPTFWI objective functions (only used for the VPTFWI mode and if fwi_obj=True). \n"		

		print "##########################################################################################"
		print "################################### II. DATA RESIDUALS ###################################"
		print "##########################################################################################\n"

		print "################################ DESCRIPTION AND REMARKS #################################"
		print "- For the FWI and linear modes, this feature provides three options:"
		print "     1. Display the data residual for a shot gather as a function of iteration (\"res_panel1\" flag)."
		print "     2. Display the observed data, predicted data, and data residual for a shot gather."
		print "     as a function of iteration (\"res_panel2_2d\" flag)."
		print "     3. Display an observed and predicted trace for a shot gather as a function of"
		print "     iteration (\"res_panel2_1d\" flag).\n"
		print " - For the VPTFWI mode, this feature also provides three options:"
		print "     1. Display the observed data \"d_obs\", the nonlinear predicted data \"f(m)\", the Born data \"B(m)p\","
		print "     and the VPTFWI predicted data \"f(m) + B(m)p\" on the same panel as a function of iteration."
		print "     2. Display the observed data \"d_obs\", the nonlinear predicted data \"f(m)\","
		print "     the FWI data residual \"f(m) - d_obs\", and the VPTFWI data residual \"f(m) + B(m)p - d_obs\""
		print "     as a function of iteration."
		print "     3. Display an observed, VPTFWI and FWI predicted trace for a shot gather as a function of"
		print "     iteration (\"res_panel2_1d\" flag).\n" 
		
		print "################################### PARAMETERS ###########################################"
		print "res_panel1 - char"
		print "     [True]: Set this flag to \"true\", \"y\", or \"1\" to display a shot residual as a"
		print "     function of iteration. This flag is only used by the linear and FWI modes. \n"     
		print "res_panel2_2d - char"
		print "     [True]: Set this flag to \"true\", \"y\", or \"1\" to display a 2D panel"
		print "     with the observed, predicted, and shot residual as a function of iteration."
		print "     This flag is only used by the linear and FWI modes. \n"
		print "res_panel2_1d - char"
		print "     [True]: Set this flag to to \"true\", \"y\", or \"1\" to display an observed and predicted"
		print "     trace as a function of iteration. Used in all modes. \n"
		print "res1d_trace - float"
		print "     [mid point]: Position in [km] of the trace for the 1D display. Set to the midpoint"
		print "     of the receiver array by default. Only used if the \"res_panel2_1d\" is set to \"True\". \n"
		print "res_jump - int"
		print "     [1]: Subsample iterations by res_jump. For instance, \"res_jump=2\" means you will display"
		print "     only every other iteration. \n"
		print "res_gainpanel - char" 
		print "     [no default]: Set the gainpanel parameter for the 2D data residual display. \n"
		print "res_pclip - float" 
		print "     [no default]: Set the pclip parameter for the 2D data residuals display. \n"
		print "res_tpow - float" 
		print "     [0]: Set the tpow parameter for the 2D data residuals display. \n"		
		print "res_grid - char" 
		print "     [y]: Set grid for the 2D data residual display. \n"
		print "res_pixmaps - char" 
		print "     [y]: Set pixmaps parameter for the 2D data residual display. \n"
		print "res_min1 - float" 
		print "     [no default]: min1 for 2D data residual display. \n"		
		print "res_max1 - float" 
		print "     [no default]: max1 for 2D data residual display. \n"				
		print "res_min2 - float" 
		print "     [no default]: min2 for 2D data residual display. \n"		
		print "res_max2 - float" 
		print "     [no default]: max2 for 2D data residual display. \n"						
		print "res_label1 - char" 
		print "     [time [s]]: label1 for 2D data residual display. \n"								
		print "res_label2 - char" 
		print "     [x [km]]: label2 for 2D data residual display. \n"										
		print "res_d1num - float" 
		print "     [no default]: d1num for 2D data residual display. \n"													
		print "res_d2num - float" 
		print "     [no default]: d2num for 2D data residual display. \n"															
		print "res1d_min1 - float" 
		print "     [no default]: min1 for 1D data residual display. \n"		
		print "res1d_max1 - float" 
		print "     [no default]: max1 for 1D data residual display. \n"				
		print "res1d_min2 - float" 
		print "     [no default]: min2 for 1D data residual display. \n"		
		print "res1d_max2 - float" 
		print "     [no default]: max2 for 1D data residual display. \n"						
		print "res1d_grid - char" 
		print "     [y]: Set grid for the 1D data residual display.\n"								
		print "res1d_pixmaps - char" 
		print "     [y]: Set pixmaps parameter for the 1D data residual display.\n"
		print "res1d_label1 - char" 
		print "     [time [s]]: label1 for 1D data residual display. \n"								
		print "res1d_label2 - char" 
		print "     [pressure [Pa]]: label2 for 1D data residual display. \n"										
		print "res1d_legend - char" 
		print "     [y]: legend for the 1D data residual display. \n"												
		print "res1d_d1num - float" 
		print "     [no default]: d1num for 1D data residual display. \n"													
		print "res1d_d2num - float" 
		print "     [no default]: d2num for 1D data residual display. \n"															

		print "##########################################################################################"
		print "################################## III. INVERTED MODELS ##################################"
		print "##########################################################################################\n"

		print "################################ DESCRIPTION AND REMARKS #################################"
		print "- For the FWI and VPTFWI modes, this features enables you to display inverted models and"
		print " true models. The non-padded true models must be provided for both modes. You have three options: \n"
		print "     1. Display a 2D panel where are inverted models are cat'ed on the third axis and"
		print "     the last slice is the true model (activate the \"mod2d\" and the \"mod2d_axis3\" flags)."
		print "     2. Display a 2D panel where each panel displays the inverted model for a given iteration"
		print "     cat'ed on the second axis with the true model (activate the \"mod2d\" flag)."
		print "     3. Display an inverted trace on a 1D plot as a function of iteration overlayed by the true trace."
		print "     (activate the \"mod1d\" flag). \n"
		print "- For the linear mode, you have three options: \n"
		print "     1. Display the zero-offset section of the inverted model (\"mod2d_zero\" flag)."
		print "     2. Display a panel with a CIG as a function of iteration (\"mod2d_cig1\" flag)."
		print "     3. Display a panel with all CIGs as a function of iteration (\"mod2d_cig2\" flag).\n"

		print "################################### PARAMETERS ###########################################"
		print "mod2d_axis3 - char"
		print "     [True]: Set this flag to \"true\", \"y\", or \"1\" to display a panel with the inverted"
		print "     models cat'ed on the third axis. The last slice is the true model. This flag only used for"
		print "     FWI and VPTFWI modes. \n"
		print "mod2d_axis3 - char"
		print "     [True]: Set this flag to \"true\", \"y\", or \"1\" to display a panel with the inverted"
		print "     models and the true model cat'ed on the second axis. Each panel correpsond to"
		print "     one iteration. This flag is only used for FWI and VPTFWI modes. \n"
		print "mod1d - char"
		print "     [True]: Set this flag to \"true\", \"y\", or \"1\" to display an inverted trace overlayed"
		print "     by the true trace as a function of iteration. This flag is only used for FWI and VPTFWI modes. \n"
		print "mod1d_trace - float"
		print "     [mid point]: Position in [km] of the trace for the 1D model display. Set to the middle"
		print "     of the model by default. \n"
		print "mod2d_zero - char"
		print "     [True]: Set this flag to \"true\", \"y\", or \"1\" to display the zero-offset cross-section"
		print "     of the inverted model. Only used for linear mode. \n"				
		print "mod2d_cig1 - char"
		print "     [True]: Set this flag to \"true\", \"y\", or \"1\" to display one CIG as a function of"
		print "     iteration (you fix the CIG and you loop over iteration on the third axis. Only used in the linear mode. \n"
		print "mod2d_cig2 - char"
		print "     [True]: Set this flag to \"true\", \"y\", or \"1\" to display multiple CIGs for one iteration."
		print "     (you fix an iteration and you display the CIGs for all the x-positions in the models. Only used in linear mode. \n"
		print "mod_pert - char"
		print "     [True]: Feature not available yet. \n"							
		print "mod2d_pclip - float"
		print "     [100]: pclip parameter for the 2D model displays. \n"
		print "mod2d_grid - char"
		print "     [y]: grid parameter for the 2D model displays. \n"
		print "mod2d_gainpanel - char"
		print "     [a]: gainpanel parameter for the 2D model displays. \n"	
		print "mod2d_pixmaps - char"
		print "     [y]: pixmaps parameter for the 2D model displays. \n"			
		print "mod2d_min1 - float"
		print "     [no default]: min1 for 2D model displays. \n"	
		print "mod2d_max1 - float"
		print "     [no default]: max1 for 2D model displays. \n"	
		print "mod2d_min2 - float"
		print "     [no default]: min2 for 2D model displays. \n"	
		print "mod2d_max2 - float"
		print "     [no default]: max2 for 2D model displays. \n"	
		print "mod2d_tpow - float"
		print "     [0.0]: tpow for 2D model displays. \n"			
		print "mod2d_label1 - char"
		print "     [z [km]]: label1 for 2D model displays. \n"		
		print "mod2d_label2 - char"
		print "     [x [km]]: label2 for 2D model displays. \n"		
		print "mod2d_color - char"
		print "     [j]: color for 2D model displays. \n"				
		print "mod2d_newclip - int"
		print "     [1]: newclip for 2D model displays. \n"					
		print "mod2d_bclip - float"
		print "     [1]: bclip for 2D model displays. \n"							
		print "mod2d_eclip - float"
		print "     [1]: eclip for 2D model displays. \n"									
		print "mod2d_allpos - float"
		print "     [n]: allpos for 2D model displays. \n"	
		print "mod2d_wantscalebar - char"
		print "     [n]: wantescalebar for 2D model displays. \n"			
		print "mod2d_barlabel - char"
		print "     [vel [km/s]]: barlabel for 2D model displays. \n"
		print "mod1d_min1 - float"
		print "     [no default]: min1 for 1D model displays. \n"	
		print "mod1d_max1 - float"
		print "     [no default]: max1 for 1D model displays. \n"	
		print "mod1d_min2 - float"
		print "     [no default]: min2 for 1D model displays. \n"	
		print "mod1d_max2 - float"
		print "     [no default]: max2 for 1D model displays. \n"			
		print "mod1d_pixmaps - char"
		print "     [y]: pixmaps parameter for the 1D model displays. \n"	
		print "mod1d_grid - char"
		print "     [y]: grid parameter for the 1D model displays. \n"
		print "mod1d_label1 - char"
		print "     [z [km]]: label1 for 1D model displays. \n"		
		print "mod1d_label2 - char"
		print "     [vel [km/s]]]: label2 for 1D model displays. \n"
		print "mod1d_legend - char"
		print "     [y]: legend for 1D model displays. \n"		

		print "##########################################################################################"
		print "################################## IV. GRADIENTS #########################################"
		print "##########################################################################################\n"

		print "################################ DESCRIPTION AND REMARKS #################################"
		print "This feature allows you to display the gradients of your inversion as a function of iteration."
		print "So far, display feature only available for FWI and VPTFWI modes."
		print "     - The \"grad_disp1\" flag allows you to display the gradients cat'ed on the third axis"
		print "     as a function of iteration."
		print "     - The \"grad_disp2\" flag allows you to compute the \"ideal\" gradient for each iteration, and cats it"
		print "     on the second axis with the gradient from your inversion so you can compare your gradient" 
		print "     with what the \"perfect\" gradient should be. These panels are then cat'ed on the third"
		print "     axis as a function of iteration."
		print "     - The \"grad_vptfwi\" is an option specific to the VPTFWI mode. It allows you to display"
		print "     the tomographic component, the Born component, and the total component of the VPTFIW gradient."
		print "     If the flag is activated, you will display a set of panels (each panel corresponds to"
		print "     one iteration) containing the tomo, Born, total and ideal gradient cat'ed on the second axis."
		print "     - The \"grad_iter\" flag allows you to analyze the gradient for specific iteration and is only used for the"
		print "     VPTFWI mode. The tomo, Born, totla, and ideal gradients are cat'ed on the third axis. The "
		print "     flag is activated by setting it to an iteration number. \n"
		
		print "################################### PARAMETERS ###########################################"
		print "grad_disp1 - char"
		print "     [True]: Set this flag to \"true\", \"y\", or \"1\" to display a panel with gradients"
		print "     cat'ed on the third axis. This flag only used for FWI and VPTFWI modes. \n"			
		print "grad_disp2 - char"
		print "     [True]: Set this flag to \"true\", \"y\", or \"1\" to display a panel with the gradients"
		print "     cat'ed on the second axis with the ideal (smoothed) gradient for each iteration. The third"
		print "     axis loops over iteration. This flag only used for FWI and VPTFWI modes. \n"
		print "grad_vptfwi - char"
		print "     [True]: Set this flag to \"true\", \"y\", or \"1\" to display a panel with the inverted"
		print "     models cat'ed on the third axis. The last slice is the true model. This flag only used for"
		print "     FWI and VPTFWI modes. \n"							
		print "grad_vptfwi_iter - int"
		print "     [True]: Provide the iteration number at which you wish to display the tomo, Born, total and"
		print "     ideal gradients cat'ed on the third axis. Needs to be greater or equal to one."
		print "grad_color - char"
		print "     [j]: Display color for the gradient. \n"		
		print "grad_pclip - char"
		print "     [j]: pclip for the gradient. \n"				
		print "grad_gainpanel - char"
		print "     [e]: gainpanel for the gradient. \n"						
		print "grad_grid - char"
		print "     [n]: grid for the gradient. \n"
		print "grad_d1num - float"
		print "     [no default]: d1num for the gradient. \n"
		print "grad_d2num - float"
		print "     [no default]: d2num for the gradient. \n"		
		print "grad_min1 - float"
		print "     [no default]: min1 for the gradient. \n"		
		print "grad_max1 - float"
		print "     [no default]: max1 for the gradient. \n"				
		print "grad_min2 - float"
		print "     [no default]: min2 for the gradient. \n"						
		print "grad_max2 - float"
		print "     [no default]: max2 for the gradient. \n"
		print "grad_label1 - char"
		print "     [z [km]]: label1 for the gradient. \n"		
		print "grad_label2 - char"
		print "     [x [km]]: label2 for the gradient. \n"				
		print "grad_pixmaps - char"
		print "     [y]: pixmaps for the gradient. \n"
		print "grad_tpow - float"
		print "     [1.5]: tpow for the gradient. \n"		
		print "grad_smooth1 - int"
		print "     [0]: Smoothing parameter when computing the \"ideal\" gradient. This value"
		print "     will be the input of \"rect1\" in the \"Smooth\" SEP program.\n"
		print "grad_smooth2 - int"
		print "     [0]: Smoothing parameter when computing the \"ideal\" gradient. This value"
		print "     will be the input of \"rect2\" in the \"Smooth\" SEP program.\n"
		print "grad_jump - int"
		print "     [1]: Subsample iterations by grad_jump. For instance, \"grad_jump=2\" means you will display"
		print "     only every other iteration. \n"				
		print "grad_pad - char"
		print "     [False]: Set this flag to \"true\", \"y\", or \"1\" to display the gradients"
		print "     with the absorbing boundaries.\n"
								
		##################################################################################

	else:

		##################################################################################	
		############################### READ PARAMETERS ##################################
		###### BULLSHIT PLUMBING STUFF THAT SHOULD BE DONE BY SLAVES INSTEAD OF ME #######
		##################################################################################

		######################### COMMON PART FOR ALL MODES ##############################
		
		# CURRENT DIRECTORY
		cwd = os_sys.getcwd()+"/"

		# READ IF comment
		comment = sep.from_cmd_line("comment",default=True)		
		if (comment == "y" or comment == "true" or comment == "1"): 
			comment = True

		# comment for user
		if (comment): print "                                                                             " 
		if (comment): print "                  ##########################################                 "
		if (comment): print "                  ########### COMMENTS MODE: ON ############                 "
		if (comment): print "                  ##########################################                 "
		if (comment): print "                                                                             " 

		# comment for user
		if (comment != True): print "                                                                         " 
		if (comment != True): print "              ##########################################                 "
		if (comment != True): print "              ########### COMMENTS MODE: OFF ###########                 "
		if (comment != True): print "              ##########################################                 "
		if (comment != True): print "                                                                         " 

		# comment for user
		if (comment): print "                                                                                     "
		if (comment): print "############################### GENERAL PARAMETERS ##################################"

		# MODE: FWI, LINEAR OR VPTFWI
		mode = sep.from_cmd_line("mode")
		if (comment): print "INVERSION TYPE: %s" %(mode)
		
		# DISPLAY NICKNAME 
		nickname = sep.from_cmd_line("nickname")		

		###################### CHOICE OF INVERSION COMPONENTS TO VISUALIZE ###############

		# 1. objective function(s)
		obj = sep.from_cmd_line("obj",default=False)
		if (obj == "y" or obj == "true" or obj == "1"): 
			obj = True

		if (comment and obj == True): print "OBJECTIVE FUNCTION: ON"
		if (comment and obj != True): print "OBJECTIVE FUNCTION: OFF"
			
		# 2. data residual
		res = sep.from_cmd_line("res",default=False)		
		if (res == "y" or res == "true" or res == "1"): 
			res = True
			# get the shot number for the data residual
			res_shot = int(sep.from_cmd_line("shot",default="1"))
			res_shot = res_shot - 1		

		if (comment and res == True): print "RESIDUALS: ON"
		if (comment and res != True): print "RESIDUALS: OFF"

		# 3. models
		mod = sep.from_cmd_line("mod",default=False)			
		if (mod == "y" or mod == "true" or mod == "1"): 
			mod = True
			
		if (comment and mod == True): print "MODELS: ON"
		if (comment and mod != True): print "MODELS: OFF"			
			
		# 4. gradients
		grad = sep.from_cmd_line("grad",default=False)			
		if (grad == "y" or grad == "true" or grad == "1"): 
			grad = True

		if (comment and grad == True): print "GRADIENTS: ON"
		if (comment and grad != True): print "GRADIENTS: OFF"
			
		# 5. nothing to display
		if (obj != True and res != True and mod != True and grad != True):
			if (comment): print "NOT DISPLAYING ANYTHING, WILL EXIT THE SCRIPT"
			exit()

		##################################################################################
			
		# REMOVE THE TEMPORARY FILES
		remove = sep.from_cmd_line("remove",default=True)
		if (remove == "y" or remove == "true" or remove == "1"): 
			remove = True

		if (comment and remove == True): print "REMOVE TEMPORARY FILES: YES"
		if (comment and remove != True): print "REMOVE TEMPORARY FILES: NO"
						
		# MAIN WORK DIRECTORIES
		main_wrk_dir = sep.from_cmd_line("main_wrk_dir",want_list=True)
		if (not isinstance(main_wrk_dir,list)):
			main_wrk_dir = [main_wrk_dir]
		
		# TOTAL NB OF INVERSIONS
		nb_inv = len(main_wrk_dir)

		# INVERSION SUFFIX
		suffix = sep.from_cmd_line("suffix",want_list=True)
		if (not isinstance(suffix,list)):
			suffix = [suffix]

		############################## REGULARIZATION ####################################
		# compute number of objective functions inside main folder
		nb_obj_functions = 0
		cmd = "ls "+main_wrk_dir[0]+"/obj"+suffix[0]+"* | wc -l" 
		nb_obj_functions = int(sep.RunShellCmd_output(cmd))
		sep.RunShellCmd(cmd)

		if (nb_obj_functions > 1):
			reg = True 
		else:
			reg = False
			prec = False
 
		# check if preconditioning was applied for linear inversions
		if (reg == True and mode == "linear"):
			prec = sep.from_cmd_line("prec",default=False)
			if (prec == "y" or prec == "true" or prec == "1"):
				prec = True	
			else:
				prec = False
			if (comment and reg == True and prec != True): print "LINEAR INVERSION: REGULARIZED NON-PRECONDITIONED"
			if (comment and reg == True and prec == True): print "LINEAR INVERSION: REGULARIZED AND PRECONDITIONED"

		if (comment and reg == True and mode != "linear"): print "INVERSION: REGULARIZED"
		if (comment and reg != True and mode != "linear"): print "INVERSION: NON-REGULARIZED"		
					
		##################################################################################
		
		#################################### DATA ########################################
		data = sep.from_cmd_line("data",default=None)
		assert(os_sys.path.isfile(data)),"Error: data file not found"

		cmd = "Get < %s o2 parform=n" %(data)			
		origin_r = sep.RunShellCmd_output(cmd)
		origin_r_float = float(origin_r)		

		cmd = "Get < %s d2 parform=n" %(data)			
		dr = sep.RunShellCmd_output(cmd)		
		dr_float = float(dr)
		
		cmd = "Get < %s n2 parform=n" %(data)							
		nr = sep.RunShellCmd_output(cmd)		
		nr_float = float(nr)

		##################################################################################
			
		#################################### TRUE MODEL ##################################

		# READ TRUE MODEL 
		if (mode == "fwi" or mode == "vptfwi"):

			# read true model (NON-PADDED) 
			true_model = sep.from_cmd_line("true_model",default=None)
			assert(os_sys.path.isfile(true_model)),"Error: true model file not found "
			
			# get the origin of the x-axis
			cmd = "Get < %s o2 parform=n" %(true_model)	
			ox = sep.RunShellCmd_output(cmd)
			ox_float = float(ox)
			
		##################################################################################

		################################ EXTENSION SIZE ##################################

		# GET MAIN PARFILE NAME
		main_parfile = sep.from_cmd_line("main_parfile",default=None)

		# GET MODEL SIZES
		if (main_parfile == None):
			nz = sep.from_cmd_line("nz")
			nx = sep.from_cmd_line("nx")
			z_pad = sep.from_cmd_line("z_pad")
			x_pad = sep.from_cmd_line("x_pad")
			z_pad_int = int(z_pad)				
			x_pad_int = int(x_pad)
			dz = sep.from_cmd_line("dz")
			dz_float = float(dz)
			dx = sep.from_cmd_line("dx")
			dx_float = float(dx)
															
		else:
			nz = sep.from_par_file("nz",main_parfile)
			nx = sep.from_par_file("nx",main_parfile)
			z_pad = sep.from_par_file("z_pad",main_parfile)		
			z_pad_int = int(z_pad)	
			x_pad = sep.from_par_file("x_pad",main_parfile)
			x_pad_int = int(x_pad)				
			dz = sep.from_par_file("dz",main_parfile)		
			dx = sep.from_par_file("dx",main_parfile)	
			dz_float = float(dz)
			dx_float = float(dx)							
		
		# GET REST OF MODEL SIZE
		nz2 = str(int(nz) + 2 * int(z_pad))
		nx2 = str(int(nx) + 2 * int(x_pad))	
		
		# GET EXTENSION SIZE
		iteration_axis = sep.from_cmd_line("iteration_axis",default="6")
		iteration_axis = int(iteration_axis)	
		extension_axis = iteration_axis - 1	
		
		if (comment):
			print "ITERATION AXIS = %s" %(str(iteration_axis))			
		
		if (mode == "fwi" or mode == "vptfwi"):
			hzmax = "0"
			hxmax = "0"
			htmax = "0"			
			
		if (mode == "linear"):
		
			# grab the inverted model from the first inversion folder
			model_instance = cwd+main_wrk_dir[0]+"/model"+suffix[0]+".H"
		
			# get oz (need to take padding into account)
			cmd = "Get < %s o1 parform=n" %(model_instance)	
			oz_pad_float = float(sep.RunShellCmd_output(cmd))
			oz_float = oz_pad_float + z_pad_int * dz_float
			oz = str(oz_float)

			# get ox (need to take padding into account)
			cmd = "Get < %s o2 parform=n" %(model_instance)	
			ox_pad_float = float(sep.RunShellCmd_output(cmd))
			ox_float = ox_pad_float + x_pad_int * dx_float
			ox = str(ox_float)

			# set default values for extensions
			hzmax = "0"
			hxmax = "0"
			htmax = "0"

			if (iteration_axis >= 4):
				# get hzmax 
				cmd = "Get < %s n3 parform=n" %(model_instance)			
				hzmax_int = (int(sep.RunShellCmd_output(cmd)) - 1) / 2
				hzmax = str(hzmax_int)			

			if (iteration_axis >= 5):
				# get hxmax 			
				cmd = "Get < %s n4 parform=n" %(model_instance)			
				hxmax_int = (int(sep.RunShellCmd_output(cmd)) - 1) / 2
				hxmax = str(hxmax_int)						

			if (iteration_axis >= 6):
				# get htmax 			
				cmd = "Get < %s n5 parform=n" %(model_instance)			
				htmax_int = (int(sep.RunShellCmd_output(cmd)) - 1) / 2
				htmax = str(htmax_int)									

			if (comment):
				print "HZMAX = %s" %(hzmax)
				print "HXMAX = %s" %(hxmax)
				print "HTMAX = %s\n" %(htmax)						

		##################################################################################

		############################ INVERTED MODEL AND GRADIENTS ########################

		# GET INVERTED MODEL FROM ALL INVERSION FOLDERS
		inverted_model_list = []
		gradient_list = []
		niter_total = 0
		
		# loop over inversion folders
		# grab the inverted model names
		# grab the total number of iterations
		# grab the extension size
		for iinv in range(nb_inv):			

			# get the inverted model name for the specific inversion 
			inverted_model_name = cwd+main_wrk_dir[iinv]+"/model"+suffix[iinv]+".H"
						
			# add the inverted model name to the list
			inverted_model_list.append(inverted_model_name)
			
			# get the gradient name for the specific inversion 
			gradient_name = cwd+main_wrk_dir[iinv]+"/gradient"+suffix[iinv]+".H"
						
			# add the gradient name to the list
			gradient_list.append(gradient_name)			

			# get number of iteration (in total)
			if (mode == "fwi" or mode == "vptfwi"):
				cmd = "Get < %s n3 parform=n" %(inverted_model_name)	
				niter_total = niter_total+int(sep.RunShellCmd_output(cmd))
				iteration_axis = "3"
				
			elif (mode == "linear" and iteration_axis == 6):
				cmd = "Get < %s n6 parform=n" %(inverted_model_name)	
				niter_total = niter_total+int(sep.RunShellCmd_output(cmd))				
				cig_label2 = "\"time-lag [s]\""

			elif (mode == "linear" and iteration_axis == 5):
				cmd = "Get < %s n5 parform=n" %(inverted_model_name)	
				niter_total = niter_total+int(sep.RunShellCmd_output(cmd))				
				cig_label2 = "\"hx [km]\""
							
			elif (mode == "linear"  and iteration_axis == 4):
				cmd = "Get < %s n4 parform=n" %(inverted_model_name)	
				niter_total = niter_total+int(sep.RunShellCmd_output(cmd))	
				cig_label2 = "\"hz [km]\""
				
			elif (mode == "linear"):
 				cmd = "Get < %s n3 parform=n" %(inverted_model_name)	
				niter_total = niter_total+int(sep.RunShellCmd_output(cmd))	
				
				
		# Cat all the inverted models together
		inverted_model_cat = " ".join(inverted_model_list)	

		# Cat all the gradients 
		gradient_cat = " ".join(gradient_list)					

		############################ UNPRECONDITION INVERTED MODEL #######################

		# in the linear case, check if preconditioning was applied 
		if (mode == "linear" and prec == True and mod == True):
		
			if (comment): print "STARTING TO UNPRECONDITION MODEL..."		
			
			# define model before unprec/prec names and remove the temporary ones
			inverted_model_cat_before_unprec = cwd+nickname+"_inverted_model_cat_before_unprec.H"
			inverted_model_cat_after_unprec = cwd+nickname+"_inverted_model_cat_after_unprec.H"
			
			# cat the total model
			cmd = "Cat3d axis=%s %s > %s" %(iteration_axis,inverted_model_cat,inverted_model_cat_before_unprec)
			sep.RunShellCmd_output(cmd)
			
			# get the path for the dso inverse
			code_path = os_sys.environ['CODE_PATH']	
			exec_path = code_path+"/Bin"
			dso_inv_exec = exec_path+"/dso_inv_main_var_proj.x"		
			
			# apply unpreconditioning
			cmd = dso_inv_exec+" alpha=1.0 adj=0 hzmax=%s hxmax=%s htmax=%s < %s > %s" %(hzmax,hxmax,htmax,inverted_model_cat_before_unprec,inverted_model_cat_after_unprec)
			sep.RunShellCmd_output(cmd)	
			
			# use the same name as the non-preconditioning case
			inverted_model_cat = inverted_model_cat_after_unprec 
			
			# print success
			if (comment): print "... DONE UNPRECONDITIONING MODEL"		
			
			# remove the before preconditioning model
			cmd = "Rm %s" %(inverted_model_cat_before_unprec)

		##################################################################################

		############################ COMPUTE RESIDUALS FOR VPTFWI ########################
		######################### STORE THE BORN AND TOMO GRADIENTS ######################
						
		# compute FWI residual for the case of vptfwi
		# this computation is need for both the objective function part and the data residual part
		# so it is done at the beginning of the script once and for all	
		if ( mode == "vptfwi" and (obj == True or res == True or grad == True) ):
			
			# print success
			if (comment): print "COMPUTING FWI RESIDUAL FOR THE VPTFWI SCHEME..."

			######################## compute FWI residuals ###############################

			# create a list of files that will each contain the value of the FWI residual 
			# for a given inversion and iteration folder
			vptfwi_fwi_data_residual_list = []	
			vptfwi_fwi_obj_function_list =[]
			vptfwi_tomo_gradient_list = []
			vptfwi_born_gradient_list = []			

			# loop over main iteration folders
			for iinv in range(nb_inv):
		
				# get the name of the current directory
				current_dir = main_wrk_dir[iinv]
	
				# get the inverted model name for the specific inversion in order to get the number of iterations
				inverted_model_name = cwd+current_dir+"/model"+suffix[iinv]+".H"

				# determine how many iterations were run for that specific folder
				cmd = "Get < %s n3 parform=n" %(inverted_model_name)				
				niter_inversion = int(str(sep.RunShellCmd_output(cmd)))

				# get the log file name for that specific inversion
				cmd = "ls %s/*log* | grep -v debug_log.txt" %(current_dir)
				vptfwi_log_file = sep.RunShellCmd_output(cmd)

				# grab the lines in the log file where the optimal point is picked
				cmd = "more %s | grep \"best step-length\"" %(vptfwi_log_file)
				optimal_point_line = sep.RunShellCmd_output(cmd)
				optimal_point_line = optimal_point_line.split("\n")

				# loop over the number of iterations for this inversion and grab the name of the optimal point
				for ifold in range(niter_inversion):
				
					# create the name of the file where we are going to put the FWI residual for this specific inversion folder and iteration
					vptfwi_fwi_data_residual_temp = nickname+"vptfwi_fwi_data_residual_file_"+str(iinv)+"_folder"+str(ifold)+"_temp_display.H"
					
					# get the name of the file where we are going to put the tomo gradient for this specific inversion folder name and iteration
					if (ifold == 0):
						vptfwi_tomo_gradient_temp =  cwd+current_dir+"/grad_tomo_"+suffix[iinv]+".H"
						vptfwi_born_gradient_temp =  cwd+current_dir+"/grad_born_"+suffix[iinv]+".H"
					
					else:
						vptfwi_tomo_gradient_temp =  cwd+current_dir+"/grad_tomo_"+suffix[iinv]+str(ifold)+".H"
						vptfwi_born_gradient_temp =  cwd+current_dir+"/grad_born_"+suffix[iinv]+str(ifold)+".H"														

					vptfwi_tomo_gradient_list.append(vptfwi_tomo_gradient_temp)
					vptfwi_born_gradient_list.append(vptfwi_born_gradient_temp)					

					# grab the number and folder name for the optimal point at very first step (before iteration 1)
					if (ifold == 0):

						# first point is 0
						point = "0"
						
						# get the vp inversion folder name with point 0
						vp_folder = cwd+current_dir+"/vp_inversion_it"+str(ifold)+"_point"+point
					
					else:	

						# compute iteration number
						ifold = ifold - 1

						# get optimal point
						if ("c2" in optimal_point_line[ifold]):
							point = "2"
						elif ("c1" in optimal_point_line[ifold]):
							point = "1"
						else:
							point = "3"								

						# get the vp inversion folder name								
						vp_folder = cwd+current_dir+"/vp_inversion_it"+str(ifold)+"_point"+point							
			
					# extract the first data residual from optimal point and append to fwi residual list
					vptfwi_fwi_data_residual_from_folder = vp_folder+"/residual"+suffix[iinv]+"_vp_inversion_it"+str(ifold)+"_point"+point+"1.H" 
					
					# compute the FWI objective function for this residual
					cmd = "Window3d < %s n4=1 f4=0 | Attr param=1 want=norm | colrm 1 5" %(vptfwi_fwi_data_residual_from_folder) 			
					vptfwi_fwi_residual_obj_function_value = float(sep.RunShellCmd_output(cmd)) 
					vptfwi_fwi_residual_obj_function_value = float(0.5 * vptfwi_fwi_residual_obj_function_value * vptfwi_fwi_residual_obj_function_value)

					# create the name of the file that will contain this value
					vptfwi_fwi_residual_obj_function_value_file = cwd+"/"+nickname+"_vptfwi_fwi_obj_func_value_"+current_dir+"it_"+str(ifold)+"_point"+point+".H"
					
					# create the file of 1 element
					cmd = "Spike n1=1 d1=1 n2=1 d2=1 n3=1 d3=1 mag=%s > %s" %(vptfwi_fwi_residual_obj_function_value,vptfwi_fwi_residual_obj_function_value_file)
					sep.RunShellCmd_output(cmd)					
					
					# append fwi objective function list 
					vptfwi_fwi_obj_function_list.append(vptfwi_fwi_residual_obj_function_value_file)
					
					if (res == True):

						# window the shot from the fwi residual file for first iteration
						cmd = "Window3d n3=1 f3=%s n4=1 f4=0 < %s > %s" %(res_shot,vptfwi_fwi_data_residual_from_folder,vptfwi_fwi_data_residual_temp)
						sep.RunShellCmd_output(cmd)

					# append fwi residual file list
					vptfwi_fwi_data_residual_list.append(vptfwi_fwi_data_residual_temp)
			
			# once we did that for all inversion folders, display all the fwi residual files in a list, ready to be cat'ed					
			vptfwi_fwi_data_residual_temp_cat = " ".join(vptfwi_fwi_data_residual_list)						
			vptfwi_fwi_obj_function_temp_cat = " ".join(vptfwi_fwi_obj_function_list)
			vptfwi_tomo_gradient_temp_cat = " ".join(vptfwi_tomo_gradient_list)			
			vptfwi_born_gradient_temp_cat = " ".join(vptfwi_born_gradient_list)		
			
			# print success
			if (comment): print ".. DONE COMPUTING FWI RESIDUAL FOR THE VPTFWI SCHEME"

		##################################################################################

		##################################################################################				
		############################### OBJECTIVE FUNCTIONS DISPLAY ######################
		##################################################################################

		if (obj == True):
		
			# create empty lists
			total_objective_function_list = []
			data_objective_function_list = []			
			model_objective_function_list = []		
		
			# divide the total number of iteration by 10
			obj_d1num_default = str(int(niter_total/10.0))	
		
			# read objective function display parameters
			obj_grid = sep.from_cmd_line("obj_grid",default="y")
			obj_min1 = sep.from_cmd_line("obj_min1",default="0")
			obj_min2 = sep.from_cmd_line("obj_min2",default="0")
			obj_max1 = sep.from_cmd_line("obj_max1",default=str(niter_total))
			obj_max2 = sep.from_cmd_line("obj_max2",default="1")			
			obj_d1num = sep.from_cmd_line("obj_d1num",default=obj_d1num_default)
			obj_d2num = sep.from_cmd_line("obj_d2num",default="0.1")																					
			obj_legend = sep.from_cmd_line("obj_legend",default="y")
			obj_legendloc = sep.from_cmd_line("obj_legendloc",default="tr")
			obj_title = sep.from_cmd_line("obj_title",default="\"obj func [%s]\"" %(nickname)) 
			
			# flag to check if the user wants to normalize the objective function
			# by 0.5 || d_obs ||^2 instead of the first objective function value
			dat_norm = sep.from_cmd_line("dat_norm",default=False)
			if (dat_norm == "y" or dat_norm == "true" or dat_norm == "1"): 
				dat_norm = True
			else:
				dat_norm = False			 	

			if (comment): print "                                                                                     "
			if (comment): print "########################### OBJECTIVE FUNCTION PARAMETERS ###########################"
			if (comment and dat_norm == True): print "NORMALIZE BY DATA NORM: YES"
			if (comment and dat_norm != True): print "NORMALIZE BY FIRST OBJECTIVE FUNCTION VALUE: YES"			

			fwi_obj = sep.from_cmd_line("fwi_obj",default=True)
			if (fwi_obj == "y" or fwi_obj == "true" or fwi_obj == "1"): 
				fwi_obj = True

			fwi_obj_scale = sep.from_cmd_line("fwi_obj_scale",default=True)
			if (fwi_obj_scale == "y" or fwi_obj_scale == "true" or fwi_obj_scale == "1"): 
				fwi_obj_scale = True 

			if (comment and mode == "vptfwi" and fwi_obj and fwi_obj_scale != True): print "NON-NORMALIZED FWI OBJECTIVE FUNCTION FOR VPTFWI SCHEME: YES"
			if (comment and mode == "vptfwi" and fwi_obj and fwi_obj_scale ): print "NORMALIZED FWI OBJECTIVE FUNCTION FOR VPTFWI SCHEME: YES"
								 	
			# case where the inversion is regularized (and/or preconditioned in the linear case)
			if (reg == True):

				obj_curvelabel = "\"total:data:model\""

				# loop over inversion folders
				for iinv in range(nb_inv):

					# total objective function
					total_objective_function = cwd+main_wrk_dir[iinv]+"/obj"+suffix[iinv]+".H"
					assert(os_sys.path.isfile(total_objective_function)),"Error: total objective function not found for directory %s" %(nb_inv)					
					total_objective_function_list.append(total_objective_function)

					# data objective function
					data_objective_function = cwd+main_wrk_dir[iinv]+"/obj"+suffix[iinv]+"1.H"
					assert(os_sys.path.isfile(data_objective_function)),"Error: data objective function not found for directory %s" %(nb_inv)
					data_objective_function_list.append(data_objective_function)

					# model objective function					
					model_objective_function = cwd+main_wrk_dir[iinv]+"/obj"+suffix[iinv]+"2.H"
					assert(os_sys.path.isfile(data_objective_function)),"Error: model objective function not found for directory %s" %(nb_inv)
					model_objective_function_list.append(model_objective_function)

				# create the name for the final objective function files
				total_objective_function_final = cwd+nickname+"_total_objective_function.H"
				data_objective_function_final = cwd+nickname+"_data_objective_function.H"
				model_objective_function_final = cwd+nickname+"_model_objective_function.H"								

				# display the content of the list containing the objective function files (for each inversion)
				# in a line and separated by spaces (before being cat'ed)
				total_objective_function_cat = " ".join(total_objective_function_list)
				data_objective_function_cat = " ".join(data_objective_function_list)	
				model_objective_function_cat = " ".join(model_objective_function_list)						

				# create the total objective function file
				cmd = "Cat3d axis=1 %s > %s" %(total_objective_function_cat,total_objective_function_final)
				sep.RunShellCmd(cmd)

				# create the data objective function file
				cmd = "Cat3d axis=1 %s > %s" %(data_objective_function_cat,data_objective_function_final)
				sep.RunShellCmd(cmd)						

				# create the model objective function file
				cmd = "Cat3d axis=1 %s > %s" %(model_objective_function_cat,model_objective_function_final)
				sep.RunShellCmd(cmd)						
									
				if (mode != "vptfwi"):

					# display the objective functions								
					cmd = "Cat3d axis=2 %s %s %s | Scale | Graph label1=\"iteration nb\" grid=%s min1=%s max1=%s min2=%s max2=%s d1num=%s d2num=%s legend=%s legendloc=%s curvelabel=%s title=%s | Tube &" %(total_objective_function_final,data_objective_function_final,model_objective_function_final,obj_grid,obj_min1,obj_max1,obj_min2,obj_max2,obj_d1num,obj_d2num,obj_legend,obj_legendloc,obj_curvelabel,obj_title)
					sep.RunShellCmd(cmd,synch=False)
				
				if (mode == "vptfwi" and fwi_obj == True):

					obj_curvelabel = "\"total:data:model:fwi\""

					# create the name of the vptfwi fwi objective function
					vptfwi_fwi_objective_function_final = cwd+nickname+"_vptfwi_fwi_data_objective_function.H"

					# create a temporary file where we cat all the VPTFWI data/model/total scaled objective functions
					vptfwi_objective_functions_scaled_final = cwd+nickname+"_vptfwi_objective_functions_scale_final.H"
					
					if (fwi_obj_scale == True):
					
						# create and scale the vptfwi fwi objective function
						cmd = "Cat3d axis=1 %s | Scale > %s" %(vptfwi_fwi_obj_function_temp_cat,vptfwi_fwi_objective_function_final)
						sep.RunShellCmd(cmd)							
							
						# cat and scale the VPTFWI objective functions
						cmd = "Cat3d axis=2 %s %s %s | Scale > %s" %(total_objective_function_final,data_objective_function_final,model_objective_function_final,vptfwi_objective_functions_scaled_final)
						sep.RunShellCmd(cmd)		
											
					else:

						# create the vptfwi fwi objective function
						cmd = "Cat3d axis=1 %s > %s" %(vptfwi_fwi_obj_function_temp_cat,vptfwi_fwi_objective_function_final)
						sep.RunShellCmd(cmd)	
						
						# cat the VPTFWI objective functions
						cmd = "Cat3d axis=2 %s %s %s > %s" %(total_objective_function_final,data_objective_function_final,model_objective_function_final,vptfwi_objective_functions_scaled_final)						
						sep.RunShellCmd(cmd)		

					cmd = "Cat3d axis=2 %s %s | Scale | Graph label1=\"iteration nb\" grid=%s min1=%s max1=%s min2=%s max2=%s d1num=%s d2num=%s legend=%s legendloc=%s curvelabel=%s title=%s | Tube &" %(vptfwi_objective_functions_scaled_final,vptfwi_fwi_objective_function_final,obj_grid,obj_min1,obj_max1,obj_min2,obj_max2,obj_d1num,obj_d2num,obj_legend,obj_legendloc,obj_curvelabel,obj_title)
					sep.RunShellCmd(cmd,synch=False)					
			
			# case where the inversion is NOT regularized (nor preconditioned)
			# there is only one objective functions
			else:

				obj_curvelabel = "\"total\""

				# loop over inversion folders
				for iinv in range(nb_inv):

					# total objective function
					total_objective_function = cwd+main_wrk_dir[iinv]+"/obj"+suffix[iinv]+".H"
					assert(os_sys.path.isfile(total_objective_function)),"Error: total objective function not found for directory %s" %(nb_inv)					
					total_objective_function_list.append(total_objective_function)
					
				# create the name for the final objective function file
				total_objective_function_final = cwd+nickname+"_total_objective_function.H"
	
				# display the content of the list containing the objective function files (for each inversion)
				total_objective_function_cat = " ".join(total_objective_function_list)

				# create the total objective function file
				cmd = "Cat3d axis=1 %s > %s" %(total_objective_function_cat,total_objective_function_final)
				sep.RunShellCmd(cmd)
				
				if (dat_norm == False):
					# display objective function normalized by the first objective function value 
					cmd = "Scale < %s | Graph label1=\"iteration nb\" label2=\"norm obj function\" grid=%s min1=%s max1=%s min2=%s max2=%s d1num=%s d2num=%s legend=%s legendloc=%s curvelabel=%s title=%s | Tube &" %(total_objective_function_final,obj_grid,obj_min1,obj_max1,obj_min2,obj_max2,obj_d1num,obj_d2num,obj_legend,obj_legendloc,obj_curvelabel,obj_title)
					sep.RunShellCmd(cmd,synch=False)
				
				else:
				
					# display objective function normalized by 0.5 || d_obs ||^2
					cmd = "Attr < %s param=1 want=norm | colrm 1 5" %(data) 			
					data_norm_value = float(sep.RunShellCmd_output(cmd)) 
					data_norm_value = float(0.5 * data_norm_value * data_norm_value)
					data_norm_value = 1 / data_norm_value
					
					# display objective function normalized by the first value 
					cmd = "Add %s scale=%s | Graph label1=\"iteration nb\" label2=\"dat norm obj function\" grid=%s min1=%s max1=%s min2=%s max2=%s d1num=%s d2num=%s legend=%s legendloc=%s curvelabel=%s title=%s | Tube &" %(total_objective_function_final,data_norm_value,obj_grid,obj_min1,obj_max1,obj_min2,obj_max2,obj_d1num,obj_d2num,obj_legend,obj_legendloc,obj_curvelabel,obj_title)
					sep.RunShellCmd(cmd,synch=False)
					
				if (mode == "vptfwi" and fwi_obj == True):

					obj_curvelabel = "\"total:data:fwi\""

					# create the name of the vptfwi fwi objective function
					vptfwi_fwi_objective_function_final = cwd+nickname+"_vptfwi_fwi_data_objective_function.H"
					
					if (fwi_obj_scale == True):

						# create and scale the vptfwi fwi objective function
						cmd = "Cat axis=1 %s | Scale > %s" %(vptfwi_fwi_obj_function_temp_cat,vptfwi_fwi_objective_function_final)
						sep.RunShellCmd(cmd)							
					
					else:
						# create the vptfwi fwi objective function
						cmd = "Cat axis=1 %s > %s" %(vptfwi_fwi_obj_function_temp_cat,vptfwi_fwi_objective_function_final)
						sep.RunShellCmd(cmd)						
					
					cmd = "Cat axis=2 %s %s %s | Scale | Graph label1=\"iteration nb\" grid=%s min1=%s max1=%s min2=%s max2=%s d1num=%s d2num=%s legend=%s legendloc=%s curvelabel=%s title=%s | Tube &" %(total_objective_function_final,data_objective_function_final,vptfwi_fwi_objective_function_final,obj_grid,obj_min1,obj_max1,obj_min2,obj_max2,obj_d1num,obj_d2num,obj_legend,obj_legendloc,obj_curvelabel,obj_title)
					sep.RunShellCmd(cmd,synch=False)										
					
		##################################################################################				
		########################## DATA RESIDUALS DISPLAY ################################
		##################################################################################

		if (res == True):			
		
			# compute default profile x-position in [km] (middle of domain)
			mid_trace_pos_float = float(ox_float + float(nx) * dx_float * 0.5) 
			mid_trace_pos_str = str(mid_trace_pos_float)		
		
			# get the trace position in [km] for the 1D display
			res1d_trace_x_req_pos_str = sep.from_cmd_line("res1d_trace",default=mid_trace_pos_str)
			res1d_trace_x_pos_float = float(res1d_trace_x_req_pos_str)
			
			# find the index corresponding 
			res1d_trace_f = int((res1d_trace_x_pos_float - origin_r_float) / dr_float)
			nr_int = int(nr)
			assert(res1d_trace_f <= nr_int-1),"Error: \"res1d_trace\" required is out of bounds"				
			
			# find the position of the trace after interpolation
			res1d_trace_x_pos_str = str(origin_r_float + res1d_trace_f * dr_float)
					
			# get the "jump" for Window3d
			res_jump = sep.from_cmd_line("res_jump",default="1")
			
			# get the display parameters for the 2D display
			res_gainpanel = sep.from_cmd_line("res_gainpanel",default="a")
			res_pclip = sep.from_cmd_line("res_pclip",default="100")
			#res_title = sep.from_cmd_line("res_title",default="\"data residuals [%s]\""%(nickname))								
			res_grid = sep.from_cmd_line("res_grid",default="y")
			res_pixmaps = sep.from_cmd_line("res_pixmaps",default="y")
			res_min1 = sep.from_cmd_line("res_min1",default="\"\"") 
			res_max1 = sep.from_cmd_line("res_max1",default="\"\"")
			res_min2 = sep.from_cmd_line("res_min2",default="\"\"")
			res_max2 = sep.from_cmd_line("res_max2",default="\"\"")	
			res_label1 = sep.from_cmd_line("res_label1",default="\"time [s]\"") 
			res_label2 = sep.from_cmd_line("res_label2",default="\"x [km]\"") 				 
			res_d1num = sep.from_cmd_line("res_d1num",default="\"\"") 
			res_d2num = sep.from_cmd_line("res_d2num",default="\"\"") 
			res_tpow = sep.from_cmd_line("res_tpow",default="0") 

			# get the display parameters for the 1D display			
			res1d_min1 = sep.from_cmd_line("res1d_min1",default="\"\"") 
			res1d_max1 = sep.from_cmd_line("res1d_max1",default="\"\"")
			res1d_min2 = sep.from_cmd_line("res1d_min2",default="\"\"")
			res1d_max2 = sep.from_cmd_line("res1d_max2",default="\"\"")
			#res1d_title = sep.from_cmd_line("res1d_title",default="\"trace at %s [km] [%s]\""%(res1d_trace_x_pos_str,nickname))						
			res1d_grid = sep.from_cmd_line("res1d_grid",default="n")
			res1d_pixmaps = sep.from_cmd_line("res1d_pixmaps",default="y") 
			res1d_label1 = sep.from_cmd_line("res1d_label1",default="\"time [s]\"") 
			res1d_label2 = sep.from_cmd_line("res1d_label2",default="\"pressure [Pa]\"") 			
			res1d_legend = sep.from_cmd_line("res1d_legend",default="y") 	
			res1d_d1num = sep.from_cmd_line("res1d_d1num",default="\"\"") 
			res1d_d2num = sep.from_cmd_line("res1d_d2num",default="\"\"") 						
			
			# titles
			res1d_title = "\"trace at %s [km] [%s]\""%(res1d_trace_x_pos_str,nickname)
			res2d_title1 = "\"data residuals [%s]\""%(nickname)
			res2d_title2 = "\"OBS:PRED:RES [%s]\"" %(nickname) 
				
			# read type of display for data residuals
			res_panel1 = sep.from_cmd_line("res_panel1",default=False)
			res_panel2_2d = sep.from_cmd_line("res_panel2_2d",default=True)
			res_panel2_1d = sep.from_cmd_line("res_panel2_1d",default=True)						

			# res_panel1 = display residuals cat'ed on third axis	
			if (res_panel1 == "y" or res_panel1 == "true" or res_panel1 == "1"): 
				res_panel1 = True

			# res_panel2_2d = display obs/pred/residuals
			if (res_panel2_2d == "y" or res_panel2_2d == "true" or res_panel2_2d == "1"): 
				res_panel2_2d = True

			# res_panel2_2d = display 1d trace
			if (res_panel2_1d == "y" or res_panel2_1d == "true" or res_panel2_1d == "1"): 
				res_panel2_1d = True				

			# comments for user
			if (comment): print "                                                                                     "
			if (comment): print "############################ DATA RESIDUALS PARAMETERS ##############################"
			if (comment and res_panel1): print "DISPLAY RESIDUALS ON 3RD AXIS: YES"					
			if (comment and res_panel2_2d): print "DISPLAY OBS/PRED/RES ON 2ND AXIS: YES"
			if (comment and res_panel1): print "DISPLAY 1D TRACE AT POSITION %s [km]: YES" %(res1d_trace_x_pos_str)
						
			# get the name of all the data residual files and make a list
			data_residual_list = []				
			data_residual_temp_list = []	

			for iinv in range(nb_inv):
			
				# create temporary residual file name with only one shot for each inversion
				data_residual_temp = nickname+"_data_residual_temp_display"+str(iinv)+".H" 				

				# get name of data residual file from inversion folder
				if (reg == True):
					data_residual_from_folder = cwd+main_wrk_dir[iinv]+"/residual"+suffix[iinv]+"1.H"

				else:
					data_residual_from_folder = cwd+main_wrk_dir[iinv]+"/residual"+suffix[iinv]+".H"				
		
				cmd = "Window3d n3=1 f3=%s < %s > %s" %(res_shot,data_residual_from_folder,data_residual_temp)
				sep.RunShellCmd(cmd)
				data_residual_temp_list.append(data_residual_temp)
					
			data_residual_temp_cat = " ".join(data_residual_temp_list)

			############################### FWI AND LINEAR ###############################

			if (mode != "vptfwi"):
			
				# res_panel1: just display the data residual on one panel as a function of iteration	
				if (res_panel1 == True):

					# command that cats the list of data residual files and displays them
					cmd = "Cat3d axis=3 %s | Window3d j3=%s | Tpow tpow=%s | Grey gainpanel=%s pclip=%s grid=%s title=%s d1num=%s d2num=%s label1=%s label2=%s min1=%s max1=%s min2=%s max2=%s | Tube pixmaps=%s &" %(data_residual_temp_cat,res_jump,res_tpow,res_gainpanel,res_pclip,res_grid,res2d_title1,res_d1num,res_d2num,res_label1,res_label2,res_min1,res_max1,res_min2,res_max2,res_pixmaps) 
					sep.RunShellCmd(cmd,synch=False)
		
				# display observed data / predicted data / data residuals on axis 2
				if (res_panel2_2d == True or res_panel2_1d == True):
				
					# create temporary residual file name with all iterations of the data residual
					data_residual_cated_temp = nickname+"_data_residual_cated_temp_display.H"
					data_cated_temp = nickname+"_data_cated_temp_display.H"		
					predicted_data_cated_temp = nickname+"_predicted_data_cated_temp_display.H"							 					

					# concatenate the residual files and subsample if required by user
					cmd = "Cat3d axis=3 %s | Window3d j3=%s > %s" %(data_residual_temp_cat,res_jump,data_residual_cated_temp) 
					sep.RunShellCmd(cmd)		

					# compute number of iterations after subsampling (needed to pad the observed data)
					cmd = "Get < %s n3 parform=n" %(data_residual_cated_temp)
					niter_res = str(sep.RunShellCmd_output(cmd))	

					# create a padded observed data file and compute predicted data
					cmd = "Window3d n3=1 f3=%s < %s | Pad n3=%s extend=1 > %s" %(res_shot,data,niter_res,data_cated_temp)
					sep.RunShellCmd(cmd)

					# compute predicted data				
					cmd = "Add %s %s scale=1,1 > %s" %(data_cated_temp,data_residual_cated_temp,predicted_data_cated_temp)
					sep.RunShellCmd(cmd)

					# display a shot gather as a function of iteration
					if (res_panel2_2d == True):

						# Cat on axis 2 and display
						cmd = "Cat3d axis=2 %s %s %s | Tpow tpow=%s | Grey gainpanel=%s pclip=%s grid=%s d1num=%s d2num=%s label1=%s label2=%s min1=%s max1=%s min2=%s max2=%s title=%s | Tube pixmaps=%s &" %(data_cated_temp,predicted_data_cated_temp,data_residual_cated_temp,res_tpow,res_gainpanel,res_pclip,res_grid,res_d1num,res_d2num,res_label1,res_label2,res_min1,res_max1,res_min2,res_max2,res2d_title2,res_pixmaps)
						sep.RunShellCmd(cmd,synch=False)

					# display a trace as a function of iteration (without the residual)
					if (res_panel2_1d == True):
				
						data_cated_trace_temp = nickname+"_data_cated_trace_temp_display.H"		
						predicted_data_cated_trace_temp = nickname+"_predicted_data_cated_trace_temp_display.H"					
	
						# extract trace from observed data
						cmd = "Window3d n2=1 f2=%s < %s | Transp plane=23 > %s" %(res1d_trace_f,data_cated_temp,data_cated_trace_temp)
						sep.RunShellCmd(cmd)
				
						# extract trace from predicted data
						cmd = "Window3d n2=1 f2=%s < %s | Transp plane=23 > %s" %(res1d_trace_f,predicted_data_cated_temp,predicted_data_cated_trace_temp)
						sep.RunShellCmd(cmd)

						# Cat on axis=2 and display the traces 
						cmd = "Cat3d axis=2 %s %s | Graph grid=%s min1=%s max1=%s min2=%s max2=%s d1num=%s d2num=%s label1=%s label2=%s title=%s legend=%s curvelabel=\"predicted:true\" | Tube pixmaps=%s &" %(predicted_data_cated_trace_temp,data_cated_trace_temp,res1d_grid,res1d_min1,res1d_max1,res1d_min2,res1d_max2,res1d_d1num,res1d_d2num,res1d_label1,res1d_label2,res1d_title,res1d_legend,res1d_pixmaps)
						sep.RunShellCmd(cmd,synch=False)

			############################### VPTFWI #######################################

			if (mode == "vptfwi"):

				######################## compute VPTFWI data residuals ###################
			
				# create temporary residual file name with all iterations of the data residual
				vptfwi_data_residual_cated_temp = nickname+"_vptfwi_data_residual_cated_temp_display.H"
				vptfwi_predicted_data_cated_temp = nickname+"_vptfwi_predicted_data_cated_temp_display.H"
				observed_data_cated_temp = nickname+"_observed_data_cated_temp_display.H"		
				vptfwi_born_data_cated_temp = nickname+"_vptfwi_born_data_cated_temp_display.H"
				nonlinear_predicted_data_cated_temp	= nickname+"_nonlinear_predicted_data_cated_temp_display.H"			
				vptfwi_fwi_data_residual_cated_temp = nickname+"_fwi_data_residual_cated_temp_display.H"
	
				# concatenate the vptfwi residual file and subsample if needed
				cmd = "Cat3d axis=3 %s | Window3d j3=%s > %s" %(data_residual_temp_cat,res_jump,vptfwi_data_residual_cated_temp) 
				sep.RunShellCmd(cmd)		
		
				# compute number of iterations after subsampling
				cmd = "Get < %s n3 parform=n" %(vptfwi_data_residual_cated_temp)
				niter_res = str(sep.RunShellCmd_output(cmd))	
			
				##########################################################################

				######################## compute observed data ###########################
												
				# create a padded observed data file 
				cmd = "Window3d n3=1 f3=%s < %s | Pad n3=%s extend=1 > %s" %(res_shot,data,niter_res,observed_data_cated_temp)
				sep.RunShellCmd(cmd)

				##########################################################################
			
				######################## compute VPTFWI predicted data ###################
			
				# compute vptfwi predicted data				
				cmd = "Add %s %s scale=1,1 > %s" %(observed_data_cated_temp,vptfwi_data_residual_cated_temp,vptfwi_predicted_data_cated_temp)
				sep.RunShellCmd(cmd)

				##########################################################################

				######################## compute FWI data residual #######################
				# this is done at the beginning of the script because the objective function also
				# requires it to be computed
				
				# still need to subsample and cat all the residual files (over inversions and iterations)
				cmd = "Cat axis=3 %s | Window3d j3=%s > %s" %(vptfwi_fwi_data_residual_temp_cat,res_jump,vptfwi_fwi_data_residual_cated_temp)
				sep.RunShellCmd(cmd)				
				
				########################### compute Born data ############################

				# compute Born predicted data				
				cmd = "Add %s %s scale=1,-1 > %s" %(vptfwi_data_residual_cated_temp,vptfwi_fwi_data_residual_cated_temp,vptfwi_born_data_cated_temp)
				sep.RunShellCmd(cmd)

				##########################################################################				

				########################### compute nonlinear data #######################

				# compute nonlinear predicted data				
				cmd = "Add %s %s scale=1,1 > %s" %(vptfwi_fwi_data_residual_cated_temp,observed_data_cated_temp,nonlinear_predicted_data_cated_temp)
				sep.RunShellCmd(cmd)

				##########################################################################			
						
				# 6. Create one "data" panel with: 
					# . observed data
					# . nonlinear data
					# . Born data
					# . vptfwi predicted data = nonlinear data + Born data
									
				cmd = "Cat axis=2 %s %s %s %s | Tpow tpow=%s | Grey title=\"[obs:f(m):Born:pred] [%s]\" gainpanel=%s pclip=%s grid=%s d1num=%s d2num=%s label1=%s label2=%s min1=%s max1=%s min2=%s max2=%s | Tube pixmaps=%s &" %(observed_data_cated_temp,nonlinear_predicted_data_cated_temp,vptfwi_born_data_cated_temp,vptfwi_predicted_data_cated_temp,res_tpow,nickname,res_gainpanel,res_pclip,res_grid,res_d1num,res_d2num,res_label1,res_label2,res_min1,res_max1,res_min2,res_max2,res_pixmaps)
				sep.RunShellCmd(cmd,synch=False)
				
				# 7. Create one "residual" panel with: 
					# . observed data
					# . nonlinear data
					# . FWI residual
					# . VPTFWI residual 																	

				cmd = "Cat axis=2 %s %s %s %s | Tpow tpow=%s | Grey gainpanel=%s pclip=%s grid=%s d1num=%s d2num=%s label1=%s label2=%s min1=%s max1=%s min2=%s max2=%s title=\"[obs:f(m):FWI res:VPTFWI res] [%s] \" | Tube pixmaps=%s &" %(observed_data_cated_temp,nonlinear_predicted_data_cated_temp,vptfwi_fwi_data_residual_cated_temp,vptfwi_data_residual_cated_temp,res_tpow,res_gainpanel,res_pclip,res_grid,res_d1num,res_d2num,res_label1,res_label2,res_min1,res_max1,res_min2,res_max2,nickname,res_pixmaps)
				sep.RunShellCmd(cmd,synch=False)
				
				# display a trace as a function of iteration (without the residual)
				if (res_panel2_1d == True):

					# 8. Visualize in 1D the FWI predicted trace vs true vs. VPTFWI predicted trace

					observed_data_cated_trace_temp = nickname+"_data_cated_trace_temp_display.H"		
					vptfwi_predicted_data_cated_trace_temp = nickname+"_vptfwi_predicted_data_cated_trace_temp_display.H"					
					nonlinear_predicted_data_cated_trace_temp = nickname+"_fwi_predicted_data_cated_trace_temp_display.H"					

					# extract trace from observed data
					cmd = "Window3d n2=1 f2=%s < %s | Transp plane=23 > %s" %(res1d_trace_f,observed_data_cated_temp,observed_data_cated_trace_temp)
					sep.RunShellCmd(cmd)
		
					# extract trace from fwi predicted data
					cmd = "Window3d n2=1 f2=%s < %s | Transp plane=23 > %s" %(res1d_trace_f,nonlinear_predicted_data_cated_temp,nonlinear_predicted_data_cated_trace_temp)
					sep.RunShellCmd(cmd)
				
					# extract trace from vptfwi predicted data
					cmd = "Window3d n2=1 f2=%s < %s | Transp plane=23 > %s" %(res1d_trace_f,vptfwi_predicted_data_cated_temp,vptfwi_predicted_data_cated_trace_temp)
					sep.RunShellCmd(cmd)				

					# Cat on axis=2 and display the traces 
					cmd = "Cat3d axis=2 %s %s %s | Graph grid=%s min1=%s max1=%s min2=%s max2=%s d1num=%s d2num=%s label1=%s label2=%s title=%s legend=%s curvelabel=\"obs:vptfwi pred:nonlinear pred\" | Tube pixmaps=%s &" %(observed_data_cated_trace_temp,vptfwi_predicted_data_cated_trace_temp,nonlinear_predicted_data_cated_trace_temp,res1d_grid,res1d_min1,res1d_max1,res1d_min2,res1d_max2,res1d_d1num,res1d_d2num,res1d_label1,res1d_label2,res1d_title,res1d_legend,res1d_pixmaps)
					sep.RunShellCmd(cmd,synch=False)
					
		##################################################################################				
		################################## MODEL DISPLAY #################################
		##################################################################################
			
		if (mod == True):

			mod2d_pclip = sep.from_cmd_line("mod2d_pclip",default="100")
			mod2d_title = sep.from_cmd_line("mod2d_title",default="\"inverted model [%s] \"" %(nickname)) 
			mod2d_grid = sep.from_cmd_line("mod2d_grid",default="y")
			mod2d_gainpanel = sep.from_cmd_line("mod2d_gainpanel",default="a")				
			mod2d_pixmaps = sep.from_cmd_line("mod2d_pixmaps",default="y") 
			mod2d_min1 = sep.from_cmd_line("mod2d_min1",default="\"\"") 
			mod2d_max1 = sep.from_cmd_line("mod2d_max1",default="\"\"")
			mod2d_min2 = sep.from_cmd_line("mod2d_min2",default="\"\"")
			mod2d_max2 = sep.from_cmd_line("mod2d_max2",default="\"\"")	
			mod2d_tpow = sep.from_cmd_line("mod2d_tpow",default="0")
			mod2d_label1 = sep.from_cmd_line("mod2d_label1",default="\"z [km]\"") 
			mod2d_label2 = sep.from_cmd_line("mod2d_label2",default="\"x [km]\"") 	
			mod_pert1 = sep.from_cmd_line("mod_pert1",default=False) 
			mod_pert2 = sep.from_cmd_line("mod_pert2",default=False) 

			if (mod_pert1 == "y" or mod_pert1 == "true" or mod_pert1 == "1"): 
				mod_pert1 = True									
			if (mod_pert2 == "y" or mod_pert2 == "true" or mod_pert2 == "1"): 
				mod_pert2 = True
				
			############################### FWI AND VPTFWI ###############################
			if (mode == "fwi" or mode == "vptfwi"):
			
				# get the display parameters
				mod2d_color = sep.from_cmd_line("mod2d_color",default="j")							
				mod2d_newclip = sep.from_cmd_line("mod2d_newclip",default="1")
				mod2d_bclip = sep.from_cmd_line("mod2d_bclip",default="1.5")
				mod2d_eclip = sep.from_cmd_line("mod2d_eclip",default="4.5")			
				mod2d_allpos = sep.from_cmd_line("mod2d_allpos",default="n")			
				mod2d_wantscalebar = sep.from_cmd_line("mod2d_wantscalebar",default="1")																
				mod2d_barlabel = sep.from_cmd_line("mod2d_barlabel",default="\"vel [km/s]\"") 	

				mod1d_min1 = sep.from_cmd_line("mod1d_min1",default="\"\"") 
				mod1d_max1 = sep.from_cmd_line("mod1d_max1",default="\"\"")
				mod1d_min2 = sep.from_cmd_line("mod1d_min2",default="1.35")
				mod1d_max2 = sep.from_cmd_line("mod1d_max2",default="4.5")						 																								
				mod1d_grid = sep.from_cmd_line("mod1d_grid",default="y")
				mod1d_pixmaps = sep.from_cmd_line("mod1d_pixmaps",default="y") 
				mod1d_label1 = sep.from_cmd_line("mod1d_label1",default="\"z [km]\"") 
				mod1d_label2 = sep.from_cmd_line("mod1d_label2",default="\"vel [km/s]\"") 			
				mod1d_legend = sep.from_cmd_line("mod1d_legend",default="y") 
				
				modpert_pclip = sep.from_cmd_line("modpert_pclip",default="100") 					
				modpert_color = sep.from_cmd_line("modpert_color",default="j")							
				modpert_newclip = sep.from_cmd_line("modpert_newclip",default="0")
				modpert_bclip = sep.from_cmd_line("modpert_bclip",default="\"\"")
				modpert_eclip = sep.from_cmd_line("modpert_eclip",default="\"\"")															
				modpert_allpos = sep.from_cmd_line("modpert_allpos",default="n")														
				modpert_wantscalebar = sep.from_cmd_line("modpert_wantscalebar",default="y")																		
				modpert_barlabel = sep.from_cmd_line("modpert_barlabel",default="\"dvel [km/s]\"") 	
				modpert_grid = sep.from_cmd_line("modpert_grid",default="y")																						
				modpert_gainpanel = sep.from_cmd_line("modpert_gainpanel",default="a")																										
				modpert_min1 = sep.from_cmd_line("modpert_min1",default="\"\"") 
				modpert_max1 = sep.from_cmd_line("modpert_max1",default="\"\"")
				modpert_min2 = sep.from_cmd_line("modpert_min2",default="\"\"")
				modpert_max2 = sep.from_cmd_line("modpert_max2",default="\"\"")																													
				modpert_pixmaps = sep.from_cmd_line("modpert_pixmaps",default="y")																										
						
				# compute default profile x-position in [km] (middle of domain)
				mid_profile_pos_float = float(ox_float + float(nx) * dx_float * 0.5) 
				mid_profile_pos_str = str(mid_profile_pos_float)

				# get desired profile position
				mod_profile_x_req_pos_str = sep.from_cmd_line("mod1d_trace",default=mid_profile_pos_str)
				mod_profile_x_req_pos_float = float(mod_profile_x_req_pos_str)

				# get the index for that position			
				mod_profile_f = int((mod_profile_x_req_pos_float - ox_float) / dx_float)

				# find the position of the trace after interpolation
				mod_profile_x_pos_str = str(ox_float + mod_profile_f * dx_float)

				# create title for CIG gathers		
				mod1d_title = "\"profile at x=%s [km] [%s]\"" %(mod_profile_x_pos_str,nickname)

				# display flags
				mod2d_axis3 = sep.from_cmd_line("mod2d_axis3",default=False)
				if (mod2d_axis3 == "y" or mod2d_axis3 == "true" or mod2d_axis3 == "1"): 
					mod2d_axis3 = True

				mod2d_axis2 = sep.from_cmd_line("mod2d_axis2",default=True)
				if (mod2d_axis2 == "y" or mod2d_axis3 == "true" or mod2d_axis3 == "1"): 
					mod2d_axis2 = True
	
				mod1d = sep.from_cmd_line("mod1d",default=True)
				if (mod1d == "y" or mod1d == "true" or mod1d == "1"): 
					mod1d = True	

				# comments for user
				if (comment): print "                                                                                     "
				if (comment): print "############################ MODEL DISPLAY PARAMETERS ###############################"
				if (comment and mod2d_axis3): print "DISPLAY STANDARD MODEL (CAT'ED ON 3RD AXIS): YES"					
				if (comment and mod2d_axis3): print "DISPLAY COMPARE PANEL FOR MODEL: YES"
				if (comment and mod1d): print "DISPLAY INVERTED TRACE AT %s [KM]: YES" %(mod_profile_x_pos_str)									
	
				# declare the name of a "super inverted model" (padded along the third axis to the number of iterations)
				super_inverted_model = nickname+"_super_inverted_model_temp_display.H" 	
				super_inverted_model_shifted = nickname+"_super_inverted_model_shifted_temp_display.H" 	

				# declare the name for a "shifted super true model - super true model"
				super_inverted_model_shift_inverted_model_diff = nickname+"_super_inverted_model_shifted_inverted_model_diff.H"
			
				# declare the name of a "super true model" (padded along the third axis to the number of iterations)
				super_true_model = nickname+"_super_true_model_temp_display.H" 				

				# declare the name of a "super initial model" (padded along the third axis to the number of iterations)
				initial_model = nickname+"_initial_model_temp_display.H" 	
				super_initial_model = nickname+"_super_initial_model_temp_display.H" 	
				
				# declare the name for a "super true-initial model"
				super_true_initial_diff_model = nickname+"_super_true_initial_diff_model_temp_display.H"

				# declare the name for a "super inverted-initial model"				
				super_inverted_initial_diff_model = nickname+"_super_inverted_initial_diff_model_temp_display.H"
				
				# declare the name for a "super true-inverted model"				
				super_true_inverted_diff_model = nickname+"_super_true_inverted_diff_model_temp_display.H"				

				# cat all the inverted models and window the absorbing boundaries if required to generate a "super inverted model"
				cmd = "Cat3d axis=3 %s | Window3d n1=%s f1=%s n2=%s f2=%s > %s" %(inverted_model_cat,str(nz),str(z_pad),str(nx),str(x_pad),super_inverted_model) 						
				sep.RunShellCmd(cmd)

				# extract the first panel from the inverted super model to obtain the initial model
				cmd = "Cat3d axis=3 %s | Window3d n1=%s f1=%s n2=%s f2=%s n3=1 f3=0 > %s" %(inverted_model_cat,str(nz),str(z_pad),str(nx),str(x_pad),initial_model)	
				sep.RunShellCmd(cmd)

				# display 2D panels of inverted models and true model cat'ed on the third axis		
				if (mod2d_axis3 == True):

					# change the cat'ing axis						
					mod2d_axis = "3"		
					
					# display model		
					cmd = "Cat3d axis=%s %s %s | Grey pclip=%s color=%s title=%s newclip=%s bclip=%s eclip=%s allpos=%s wantscalebar=%s grid=%s gainpanel=%s barlabel=%s min1=%s max1=%s min2=%s max2=%s | Tube pixmaps=%s &" %(mod2d_axis,super_inverted_model,true_model,mod2d_pclip,mod2d_color,mod2d_title,mod2d_newclip,mod2d_bclip,mod2d_eclip,mod2d_allpos,mod2d_wantscalebar,mod2d_grid,mod2d_gainpanel,mod2d_barlabel,mod2d_min1,mod2d_max1,mod2d_min2,mod2d_max2,mod2d_pixmaps)
					sep.RunShellCmd(cmd,synch=False)

				# 1. Cat all inverted models with the true model on axis=2
				# 2. Cat al panels on axis=3
				if (mod2d_axis2 == True):

					# change the cat'ing axis
					mod2d_axis = "2"		

					# pad the true model
					cmd = "Pad n3=%s extend=1 < %s > %s" %(niter_total,true_model,super_true_model)
					sep.RunShellCmd(cmd)	

					# display model		
					cmd = "Cat3d axis=%s %s %s | Grey pclip=%s color=%s title=%s newclip=%s bclip=%s eclip=%s allpos=%s wantscalebar=%s grid=%s gainpanel=%s barlabel=%s min1=%s max1=%s min2=%s max2=%s | Tube pixmaps=%s &" %(mod2d_axis,super_inverted_model,super_true_model,mod2d_pclip,mod2d_color,mod2d_title,mod2d_newclip,mod2d_bclip,mod2d_eclip,mod2d_allpos,mod2d_wantscalebar,mod2d_grid,mod2d_gainpanel,mod2d_barlabel,mod2d_min1,mod2d_max1,mod2d_min2,mod2d_max2,mod2d_pixmaps)					
					sep.RunShellCmd(cmd,synch=False)		

				# Cat the true model with the inverted models (1D plot)
				if (mod1d == True):

					# create the name for the 1D "super inverted model"
					super_inverted_model_1d = nickname+"_super_inverted_model_temp_display_1d.H" 
					super_true_model_1d = nickname+"_true_model_temp_display_1d.H"

					# extract the trace of the inverted model as a function of iteration
					cmd = "Window3d n2=1 f2=%s < %s | Transp plane=23 > %s" %(mod_profile_f,super_inverted_model,super_inverted_model_1d)
					sep.RunShellCmd(cmd)			
				
					# extract the trace from the true model and pad it
					cmd = "Window3d n2=1 f2=%s < %s | Pad n2=%s extend=1 | Transp plane=23 > %s" %(mod_profile_f,super_true_model,niter_total,super_true_model_1d)
					sep.RunShellCmd(cmd)
				
					# cat the two traces on the second axis
					cmd = "Cat3d axis=2 %s %s | Graph grid=%s min1=%s max1=%s min2=%s max2=%s label1=%s label2=%s title=%s legend=%s curvelabel=\"inverted:true\" | Tube pixmaps=%s &" %(super_inverted_model_1d,super_true_model_1d,mod1d_grid,mod1d_min1,mod1d_max1,mod1d_min2,mod1d_max2,mod1d_label1,mod1d_label2,mod1d_title,mod1d_legend,mod1d_pixmaps)
					sep.RunShellCmd(cmd,synch=False)
					
				if (mod_pert1 == True):
				
					############# DESCRIPTION #############
					# for each iteration:
					# 1. left panel:   m_i    - m_initial
					# 2. right panel:  m_true - m_initial
					#######################################

					# comment for user
					if (comment and mod_pert1): print "DISPLAY UPDATES FROM STARTING MODEL VS. IDEAL UPDATE FROM STARTING MODEL"					
					
					# 2. right panel: create a super model which is the difference between the true and the initial model
					
					# pad the true model
					cmd = "Pad n3=%s extend=1 < %s > %s" %(niter_total,true_model,super_true_model)
					sep.RunShellCmd(cmd)					

					# pad the initial model
					cmd = "Pad n3=%s extend=1 < %s > %s" %(niter_total,initial_model,super_initial_model)
					sep.RunShellCmd(cmd)										

					# compute m_true - m_initial (will be shown on the right panel)
					cmd = "Add %s %s scale=1,-1 > %s" %(super_true_model,super_initial_model,super_true_initial_diff_model)
					sep.RunShellCmd(cmd)

					# 1. left panel: create a super model which is the difference between the current model and the initial model
					cmd = "Add %s %s scale=1,-1 > %s" %(super_inverted_model,super_initial_model,super_inverted_initial_diff_model)
					sep.RunShellCmd(cmd)

					# change the title
					modpert_title = "\"[inverted-intial] [left]:[true-initial] [right]\""						

					# Cat on the second axis and display
					cmd = "Cat axis=2 %s %s | Grey pclip=%s color=%s title=%s newclip=%s bclip=%s eclip=%s allpos=%s wantscalebar=%s grid=%s gainpanel=%s barlabel=%s min1=%s max1=%s min2=%s max2=%s | Tube pixmaps=%s &" %(super_inverted_initial_diff_model,super_true_initial_diff_model,modpert_pclip,modpert_color,modpert_title,modpert_newclip,modpert_bclip,modpert_eclip,modpert_allpos,modpert_wantscalebar,modpert_grid,modpert_gainpanel,modpert_barlabel,modpert_min1,modpert_max1,modpert_min2,modpert_max2,modpert_pixmaps)	
					sep.RunShellCmd(cmd,synch=False)					
					
				if (mod_pert2 == True):

					############# DESCRIPTION #############
					# for each iteration:
					# 1. left panel:   m_{i+1} - m_i
					# 2. right panel:  m_true  - m_i
					#######################################

					if (comment and mod_pert2): print "DISPLAY UPDATES FROM PREVIOUS MODEL VS. IDEAL UPDATE FROM PREVIOUS MODEL"					
					
					#2 . right panel: compute the difference between m_true and super inverted model

					# pad the true model
					cmd = "Pad n3=%s extend=1 < %s > %s" %(niter_total,true_model,super_true_model)
					sep.RunShellCmd(cmd)	

					# create m_true - m_inverted
					cmd = "Add %s %s scale=1,-1 > %s" %(super_true_model,super_inverted_model,super_true_inverted_diff_model)
					sep.RunShellCmd(cmd)						
					
					#1 . left panel: take away the first element of the super inverted model and pad at the end 
					cmd = "Window3d f3=1 < %s | Pad end3=1 extend=1 > %s" %(super_inverted_model,super_inverted_model_shifted)
					sep.RunShellCmd(cmd)					
					
					# take the difference between the shifted super inverted model and the super inverted model
					cmd = "Add %s %s scale=1,-1 > %s" %(super_inverted_model_shifted,super_inverted_model,super_inverted_model_shift_inverted_model_diff)
					sep.RunShellCmd(cmd)
					
					# change the title
					# change the title
					modpert_title = "\"[model updates] [left]:[true - inverted] [right]\""													
					
					# Cat on the second axis and display
					cmd = "Cat axis=2 %s %s | Grey pclip=%s color=%s title=%s newclip=%s bclip=%s eclip=%s allpos=%s wantscalebar=%s grid=%s gainpanel=%s barlabel=%s min1=%s max1=%s min2=%s max2=%s | Tube pixmaps=%s &" %(super_inverted_model_shift_inverted_model_diff,super_true_inverted_diff_model,modpert_pclip,modpert_color,modpert_title,modpert_newclip,modpert_bclip,modpert_eclip,modpert_allpos,modpert_wantscalebar,modpert_grid,modpert_gainpanel,modpert_barlabel,modpert_min1,modpert_max1,modpert_min2,modpert_max2,modpert_pixmaps)	
					sep.RunShellCmd(cmd,synch=False)	
					

				if (comment): print "                                                                                     "
			##############################################################################

			############################### LINEAR CASE ##################################
			if (mode == "linear"):
			
				# display zero-offset section
				mod2d_zero = sep.from_cmd_line("mod2d_zero",default=True)
				if (mod2d_zero == "y" or mod2d_zero == "true" or mod2d_zero == "1"): 
					mod2d_zero = True

				# display CIGs
				mod2d_cig1 = sep.from_cmd_line("mod2d_cig1",default=True)
				if (mod2d_cig1 == "y" or mod2d_cig1 == "true" or mod2d_cig1 == "1"): 
					mod2d_cig1 = True

				# display CIGs
				mod2d_cig2 = sep.from_cmd_line("mod2d_cig2",default=True)
				if (mod2d_cig2 == "y" or mod2d_cig2 == "true" or mod2d_cig2 == "1"): 
					mod2d_cig2 = True
					
				# Cat all the inverted models together
				model_cat = " ".join(inverted_model_list)

				# allow possibility to display with absorbing boundaries
				mod2d_pad = sep.from_cmd_line("mod2d_pad",default=False) 	 																				
				if (mod2d_pad == "y" or mod2d_pad == "true" or mod2d_pad == "1"): 
					mod2d_pad = True	
					mod2d_nz = nz2
					mod2d_nx = nx2
					mod2d_fz = "0"
					mod2d_fx = "0"												 
				else:
					mod2d_nz = nz
					mod2d_nx = nx
					mod2d_fz = z_pad
					mod2d_fx = x_pad					
	
				# comments for user
				if (comment): print "                                                                                     "
				if (comment): print "######################### LINEAR MODEL DISPLAY PARAMETERS ###########################"
				if (comment and mod2d_zero): print "DISPLAY ZERO-OFFSET SECTION OF INVERTED MODEL: YES"
	
				# display zero-offset section with iterations
				if (mod2d_zero == True):
					
					n3 = 1 
					f3 = 0
					n4 = 1
					f4 = 0
					f5 = 0
					 					
					if (extension_axis == 5):
						f5 = htmax
						f5 = str(f5)					
					if (extension_axis == 4):
						f4 = hxmax
						f4 = str(f4)
						n5 = niter_total	
						n5 = str(n5)			
					if (extension_axis == 3):
						f3 = hzmax																
						f3 = str(f3)
						n4 = niter_total				
						n4 = str(n4)
					if (extension_axis == None):
						n3 = niter_total								
						n3 = str(n3)
						
					cmd = "Cat3d axis=%s %s | Window3d n1=%s f1=%s n2=%s f2=%s n3=%s f3=%s n4=%s f4=%s n5=1 f5=%s | Tpow tpow=%s | Grey gainpanel=%s pclip=%s grid=%s min1=%s max1=%s min2=%s max2=%s label1=%s label2=%s title=\"zero-offset section [%s]\" | Tube pixmaps=%s &" %(iteration_axis,model_cat,mod2d_nz,mod2d_fz,mod2d_nx,mod2d_fx,n3,f3,n4,f4,f5,mod2d_tpow,mod2d_gainpanel,mod2d_pclip,mod2d_grid,mod2d_min1,mod2d_max1,mod2d_min2,mod2d_max2,mod2d_label1,mod2d_label2,nickname,mod2d_pixmaps)
					print "cmd",cmd
					sep.RunShellCmd(cmd,synch=False)							

				# display one CIG with iterations
				if (mod2d_cig1 == True and extension_axis != None):
				
					# compute default CIG position in [km] in middle of domain
					mid_cig_pos_float = float(ox_float + float(nx) * dx_float * 0.5) 
					mid_cig_pos_str = str(mid_cig_pos_float)

					# get the desired CIG position
					cig_x_req_pos_str = sep.from_cmd_line("cig_pos",default=mid_cig_pos_str)
					cig_x_req_pos_float = float(cig_x_req_pos_str)
					
					# get the index for that position			
					cig_f = int((cig_x_req_pos_float - ox_float) / dx_float)

					# find the position of the trace after interpolation
					cig_x_pos_str = str(ox_float + cig_f * dx_float)

					# create title for CIG gathers		
					cig_title = "\"CIG at position x=%s [km]\"" %(cig_x_pos_str)

					# comment for user
					if (comment and mod2d_cig1): print "DISPLAY CIG AS A FUNCTION OF ITERATION AT POSITION X=%s [KM]: YES" %(cig_x_pos_str)					
			
					cmd = "Cat3d axis=%s %s | Window3d n1=%s f1=%s n2=1 f2=%s | Tpow tpow=%s | Grey gainpanel=%s pclip=%s grid=%s min1=%s max1=%s min2=%s max2=%s label1=%s label2=%s title=%s | Tube pixmaps=%s &" %(iteration_axis,model_cat,mod2d_nz,mod2d_fz,cig_f,mod2d_tpow,mod2d_gainpanel,mod2d_pclip,mod2d_grid,mod2d_min1,mod2d_max1,mod2d_min2,mod2d_max2,mod2d_label1,cig_label2,cig_title,mod2d_pixmaps)
					print "cmd cig as a function of it",cmd
					sep.RunShellCmd(cmd,synch=False)

				# display many CIGs within one iteration
				if (mod2d_cig2 == True and extension_axis != None):
					
					# pick the iteration number
					iter_cig = sep.from_cmd_line("iter_cig",default=niter_total)
					iter_cig = int(iter_cig) - 1
					iter_cig = str(iter_cig)
					mod2d_jump = sep.from_cmd_line("mod2d_jump",default="1")					

					if (comment and mod2d_cig2): print "DISPLAY ALL CIGs FOR ITERATION %s: YES" %(iter_cig)
					
					if (extension_axis == 5):
					
						cmd = "Cat3d axis=%s %s | Window3d n1=%s f1=%s f2=%s j2=%s n3=1 f3=0 n4=1 f4=0 n6=1 f6=%s | Transp plane=23 | Tpow tpow=%s | Grey gainpanel=%s pclip=%s grid=%s min1=%s max1=%s min2=%s max2=%s label1=%s label2=%s title=\"CIGs [%s]\" | Tube pixmaps=%s &" %(iteration_axis,model_cat,mod2d_nz,mod2d_fz,mod2d_fx,mod2d_jump,iter_cig,mod2d_tpow,mod2d_gainpanel,mod2d_pclip,mod2d_grid,mod2d_min1,mod2d_max1,mod2d_min2,mod2d_max2,mod2d_label1,cig_label2,nickname,mod2d_pixmaps)
						sep.RunShellCmd(cmd,synch=False)					
					
					if (extension_axis == 4):
					
						cmd = "Cat3d axis=%s %s | Window3d n1=%s f1=%s f2=%s j2=%s n3=1 f3=0 n5=1 f5=%s | Transp plane=23 | Tpow tpow=%s | Grey gainpanel=%s pclip=%s grid=%s min1=%s max1=%s min2=%s max2=%s label1=%s label2=%s title=\"CIGs [%s]\" | Tube pixmaps=%s &" %(iteration_axis,model_cat,mod2d_nz,mod2d_fz,mod2d_fx,mod2d_jump,iter_cig,mod2d_tpow,mod2d_gainpanel,mod2d_pclip,mod2d_grid,mod2d_min1,mod2d_max1,mod2d_min2,mod2d_max2,mod2d_label1,cig_label2,nickname,mod2d_pixmaps)
						sep.RunShellCmd(cmd,synch=False)
						
					if (extension_axis == 3):
					
						cmd = "Cat3d axis=%s %s | Window3d n1=%s f1=%s f2=%s j2=%s n4=1 f4=%s | Transp plane=23 | Tpow tpow=%s | Grey gainpanel=%s pclip=%s grid=%s min1=%s max1=%s min2=%s max2=%s label1=%s label2=%s title=\"CIGs [%s]\" | Tube pixmaps=%s &" %(iteration_axis,model_cat,mod2d_nz,mod2d_fz,mod2d_fx,mod2d_jump,iter_cig,mod2d_tpow,mod2d_gainpanel,mod2d_pclip,mod2d_grid,mod2d_min1,mod2d_max1,mod2d_min2,mod2d_max2,mod2d_label1,cig_label2,nickname,mod2d_pixmaps)
						sep.RunShellCmd(cmd,synch=False)	
						
												
		##################################################################################					
			
		############################### GRADIENT #########################################
		if (grad == True):

			grad_color = sep.from_cmd_line("grad_color",default="j")
			grad_pclip = sep.from_cmd_line("grad_pclip",default="\"\"")				 
			grad_gainpanel = sep.from_cmd_line("gainpanel",default="e") 				
			grad_grid = sep.from_cmd_line("grad_grid",default="n")
			grad_d1num = sep.from_cmd_line("grad_d1num",default="\"\"") 
			grad_d2num = sep.from_cmd_line("grad_d2num",default="\"\"")
			grad_min1 = sep.from_cmd_line("grad_min1",default="\"\"") 
			grad_max1 = sep.from_cmd_line("grad_max1",default="\"\"")
			grad_min2 = sep.from_cmd_line("grad_min2",default="\"\"")
			grad_max2 = sep.from_cmd_line("grad_max2",default="\"\"")
			grad_title1 = "\"gradient [%s]\"" %(nickname)
			grad_title2 = "\"gradient [%s] / ideal gradient\"" %(nickname)
			grad_title3 = "\"tomo / Born / total / ideal [%s]\"" %(nickname)						
			grad_title4 = "\"tomo gradient [%s]:Born gradient [%s]:total gradient [%s]:ideal gradient [%s]" %(nickname,nickname,nickname,nickname)
			grad_label1 = sep.from_cmd_line("grad_label1",default="\"z[km]\"") 
			grad_label2 = sep.from_cmd_line("grad_label2",default="\"x[km]\"") 
			grad_pixmaps = sep.from_cmd_line("grad_pixmaps",default="y")
			grad_tpow = sep.from_cmd_line("grad_tpow",default="1.5")
			grad_rect1 = sep.from_cmd_line("grad_smooth1",default="0")
			grad_rect2 = sep.from_cmd_line("grad_smooth2",default="0")
			grad_jump = sep.from_cmd_line("grad_jump",default="1")	

			# command to determine if the user wants to display the padded zone around the model			 		
			grad_pad = sep.from_cmd_line("grad_pad",default=False) 
			if (grad_pad == "y" or grad_pad == "true" or grad_pad == "1"): 
				grad_pad = True					

			# plot the gradients cat'ed over the third axis as a function of iteration
			grad_disp1 = sep.from_cmd_line("grad_disp1",default=True)
			if (grad_disp1 == "y" or grad_disp1 == "true" or grad_disp1 == "1"): 
				grad_disp1 = True							 		

			# plot the gradients cat'ed over the third axis as a function of iteration
			grad_disp2 = sep.from_cmd_line("grad_disp2",default=True)
			if (grad_disp2 == "y" or grad_disp2 == "true" or grad_disp2 == "1"): 
				grad_disp2 = True

			grad_vptfwi = sep.from_cmd_line("grad_vptfwi",default=True)
			if (grad_vptfwi == "y" or grad_vptfwi == "true" or grad_vptfwi == "1"): 
				grad_vptfwi = True

			if (comment): print "######################### GRADIENT DISPLAY PARAMETERS ###############################"

			# window the gradients
			if (grad_pad == True):

				nz_temp = nz2
				nx_temp = nx2
				z_pad_temp = "0" 
				x_pad_temp = "0"
			
			else:
				nz_temp = nz
				nx_temp = nx
				z_pad_temp = z_pad 
				x_pad_temp = x_pad

			if (mode == "vptfwi" or mode == "fwi"):

				# comment for user
				if (comment and grad_disp1): print "DISPLAY GRADIENT CAT'ED ON THIRD AXIS: YES"
				if (comment and grad_disp2): print "DISPLAY GRADIENT CAT'ED ON THIRD AXIS COMPARED WITH TRUE GRADIENT: YES"			

				if (grad_disp1 == True):

					# cat all gradients on 3rd axis and display them 
					cmd = "Cat3d axis=3 %s | Window3d n1=%s f1=%s n2=%s f2=%s j3=%s | Scale | Tpow tpow=%s | Grey color=%s gainpanel=%s grid=%s d1num=%s d2num=%s min1=%s max1=%s min2=%s max2=%s title=%s label1=%s label2=%s pclip=%s | Tube pixmaps=%s &" %(gradient_cat,nz_temp,z_pad_temp,nx_temp,x_pad_temp,grad_jump,grad_tpow,grad_color,grad_gainpanel,grad_grid,grad_d1num,grad_d2num,grad_min1,grad_max1,grad_min2,grad_max2,grad_title1,grad_label1,grad_label2,grad_pclip,grad_pixmaps)
					sep.RunShellCmd(cmd,synch=False)
				
				if (grad_disp2 == True):
								
					# create a "super" inverted model
					super_inverted_model = nickname+"_inverted_model_temp_display.H" 	
					cmd = "Cat3d axis=3 %s | Window3d n1=%s f1=%s n2=%s f2=%s j3=%s > %s" %(inverted_model_cat,nz,z_pad,nx,x_pad,grad_jump,super_inverted_model) 						
					sep.RunShellCmd(cmd)				

					# create a "super" true model 
					super_true_model = nickname+"_true_model_temp_display.H" 

 					# pad the super true model 
					cmd = "Pad n3=%s extend=1 < %s | Window3d j3=%s > %s" %(niter_total,true_model,grad_jump,super_true_model)
					sep.RunShellCmd(cmd)				
	
					# compute "ideal" gradients and smooth them
					super_ideal_gradient = nickname+"_super_ideal_gradient_temp_display.H" 					
					cmd = "Add %s %s scale=1,-1 | Smooth rect1=%s rect2=%s | Scale > %s" %(super_inverted_model,super_true_model,grad_rect1,grad_rect2,super_ideal_gradient)
					sep.RunShellCmd(cmd)
					
					# create a "super" gradient
					super_gradient = nickname+"_super_gradient_temp_display.H" 						
					cmd =  "Cat3d axis=3 %s | Window3d n1=%s f1=%s n2=%s f2=%s j3=%s | Tpow tpow=%s | Scale > %s" %(gradient_cat,nz_temp,z_pad_temp,nx_temp,x_pad_temp,grad_jump,grad_tpow,super_gradient)  								
					sep.RunShellCmd(cmd)					
				
					# for each iteration compute the "ideal" gradient and display it next to the computed gradient
					# cat super gradient and ideal gradients on 2nd axis
					cmd = "Cat3d axis=2 %s %s | Grey color=%s gainpanel=%s grid=%s d1num=%s d2num=%s min1=%s max1=%s min2=%s max2=%s title=%s label1=%s label2=%s pclip=%s | Tube pixmaps=%s &" %(super_gradient,super_ideal_gradient,grad_color,grad_gainpanel,grad_grid,grad_d1num,grad_d2num,grad_min1,grad_max1,grad_min2,grad_max2,grad_title2,grad_label1,grad_label2,grad_pclip,grad_pixmaps)
					sep.RunShellCmd(cmd,synch=False)	

					if (mode == "vptfwi" and grad_vptfwi == True):
					
						# comment for user
						if (comment): print "DISPLAY TOMO, BORN, TOTAL AND IDEAL GRADIENTS: YES"								
						
						# for the vptfwi case, plot the tomo / Born / total / ideal gradients on the 2nd axis
						vptfwi_tomo_gradient_temp_cat_total = nickname+"_vptfwi_tomo_gradient_cat_total_temp_display.H"
						vptfwi_born_gradient_temp_cat_total = nickname+"_vptfwi_born_gradient_cat_total_temp_display.H"
						vptfwi_total_gradient_temp_cat_total = nickname+"_vptfwi_total_gradient_cat_total_temp_display.H"
						vptfwi_total_gradient_panel_temp_cat_total = nickname+"_vptfwi_total_gradient_panel_cat_total_temp_display.H"

						cmd = "Cat3d axis=3 %s | Window3d n1=%s f1=%s n2=%s f2=%s j3=%s | Tpow tpow=%s > %s" %(vptfwi_tomo_gradient_temp_cat,nz_temp,z_pad_temp,nx_temp,x_pad_temp,grad_jump,grad_tpow,vptfwi_tomo_gradient_temp_cat_total) 
						sep.RunShellCmd(cmd)							
						cmd = "Cat3d axis=3 %s | Window3d n1=%s f1=%s n2=%s f2=%s j3=%s | Tpow tpow=%s > %s" %(vptfwi_born_gradient_temp_cat,nz_temp,z_pad_temp,nx_temp,x_pad_temp,grad_jump,grad_tpow,vptfwi_born_gradient_temp_cat_total)
						sep.RunShellCmd(cmd)							
						cmd = "Cat3d axis=3 %s | Window3d n1=%s f1=%s n2=%s f2=%s j3=%s | Tpow tpow=%s > %s" %(gradient_cat,nz_temp,z_pad_temp,nx_temp,x_pad_temp,grad_jump,grad_tpow,vptfwi_total_gradient_temp_cat_total)
						sep.RunShellCmd(cmd)							
						
						# cat all together
						cmd = "Cat3d axis=2 %s %s %s | Scale > %s" %(vptfwi_tomo_gradient_temp_cat_total,vptfwi_born_gradient_temp_cat_total,vptfwi_total_gradient_temp_cat_total,vptfwi_total_gradient_panel_temp_cat_total)
						sep.RunShellCmd(cmd)						

						# display
						cmd = "Cat3d axis=2 %s %s | Grey color=%s gainpanel=%s grid=%s d1num=%s d2num=%s min1=%s max1=%s min2=%s max2=%s title=%s label1=%s label2=%s pclip=%s | Tube pixmaps=%s &" %(vptfwi_total_gradient_panel_temp_cat_total,super_ideal_gradient,grad_color,grad_gainpanel,grad_grid,grad_d1num,grad_d2num,grad_min1,grad_max1,grad_min2,grad_max2,grad_title3,grad_label1,grad_label2,grad_pclip,grad_pixmaps)
						sep.RunShellCmd(cmd,synch=False)
						
						# pick iteration that you want to analyze
						grad_vptfwi_iter = sep.from_cmd_line("grad_vptfwi_iter",default=None)

						if (grad_iter != None):
						
							# cat gradients on third axis
							# extract the gradient at that iteration
							f_grad_iter = str(int(grad_iter) - 1)
							if (f_grad_iter < 0): 
								print "iteration number should be >=1"
								sys.exit()								

							# comment for user
							if (comment): print "DISPLAY TOMO, BORN, TOTAL AND IDEAL GRADIENTS FOR ITERATION %s: YES" %(grad_iter)
						
							vptfwi_tomo_gradient_temp_it = nickname+"_vptfwi_tomo_gradient_it_temp_display.H"
							vptfwi_born_gradient_temp_it = nickname+"_vptfwi_born_gradient_it_temp_display.H"
							vptfwi_total_gradient_temp_it = nickname+"_vptfwi_total_gradient_it_temp_display.H"
							super_ideal_gradient_it = nickname+"_super_ideal_gradient_it_temp_display.H"						

							# extract tomo gradient
							cmd = "Window3d n3=1 f3=%s < %s > %s" %(f_grad_iter,vptfwi_tomo_gradient_temp_cat_total,vptfwi_tomo_gradient_temp_it)
							sep.RunShellCmd(cmd)					

							# extract Born gradient
							cmd = "Window3d n3=1 f3=%s < %s > %s" %(f_grad_iter,vptfwi_born_gradient_temp_cat_total,vptfwi_born_gradient_temp_it)						
							sep.RunShellCmd(cmd)											
						
							# extract total gradient
							cmd = "Window3d n3=1 f3=%s < %s > %s" %(f_grad_iter,vptfwi_total_gradient_temp_cat_total,vptfwi_total_gradient_temp_it)						
							sep.RunShellCmd(cmd)							
						
							# extract ideal gradient
							cmd = "Window3d n3=1 f3=%s < %s > %s" %(f_grad_iter,super_ideal_gradient,super_ideal_gradient_it)						
							sep.RunShellCmd(cmd)
						
							cmd = "Cat3d axis=3 %s %s %s %s %s | Grey  color=%s gainpanel=%s grid=%s d1num=%s d2num=%s min1=%s max1=%s min2=%s max2=%s title=%s label1=%s label2=%s pclip=%s | Tube pixmaps=%s &" %(vptfwi_tomo_gradient_temp_it,super_ideal_gradient_it,vptfwi_born_gradient_temp_it,super_ideal_gradient_it,vptfwi_total_gradient_temp_it,grad_color,grad_gainpanel,grad_grid,grad_d1num,grad_d2num,grad_min1,grad_max1,grad_min2,grad_max2,grad_title4,grad_label1,grad_label2,grad_pclip,grad_pixmaps)
							sep.RunShellCmd(cmd,synch=False)   	
																
		##################################################################################	
		
		##################################################################################	
		############################### END DISPLAY SCRIPT ###############################
		##################################################################################

		##################################################################################	
		########################### REMOVE TEMPORARY DISPLAY FILES #######################
		##################################################################################

		if (comment): print "############################### TEMPORARY FILES #####################################"

		if (remove == True):

			# delete the temporary data residual files
			print "REMOVING ALL TEMPORARY FILES \n"
			cmd = "Rm %s*.H" %(nickname)
			sep.RunShellCmd(cmd)
 		
		else:
		
			print "SAVING ALL TEMPORARY FILES \n"

		if (comment): print "############################### END DISPLAY SCRIPT ##################################"		
		
		
		
		