#!/usr/bin/python
import sep_python as sep
import operator_obj as op 
import sys,os as os_sys
import numpy as np 
from python_solver_outcore import write_log_file 
from timeit import default_timer as timer

##########################################################################################	
################################ MAIN ####################################################	
##########################################################################################	

if __name__ == '__main__':
##########################################################################################		
############################### TFWI MAIN  ###############################################	
##########################################################################################
	
	if (len(sys.argv) == 1):
        # write self documentation	
		print 'self doc not available (yet)' 

	else:

		##################################################################################	
		############################### READ PARAMETERS ##################################
		##################################################################################

		##################################################################################
		############################## PLUMBING STUFF ####################################
		##################################################################################
		# GET SOLVER PATH
		python_path_list = os_sys.environ['PYTHONPATH'].split(":")
		indx = [i for i, s in enumerate(python_path_list) if '/python_solver' in s]
		pathsource = python_path_list[indx[0]]

		# GET SCRATCH PATH
		scratch_path = sep.sep_find_datapath()
		scratch_path = scratch_path.replace("/","\/")

		# GET CURRENT DIRECTORY
		cwd = os_sys.getcwd()+"/"

		# GET THE SOURCE CODE PATH
		code_path = os_sys.environ['CODE_PATH']

		# GET NAME OF BATCH TASKS EXECUTORS		
		born_adj = code_path+"/Scripts/batch_task_executor_born_adj.py"		
		ext_tomo_adj = code_path+"/Scripts/batch_task_executor_ext_tomo_adj.py"	
		dso_inv_exec = code_path+"/Bin/dso_inv_main_var_proj.x"
		
		# GET THE NAME OF THE COMPUTER (E.G., MAZAMA, RCF, ETC.)
		if ("sep" in cwd):
			computer = "rcf"
		else:
			computer = "mazama"
		
		############################## DONE BULLSHIT STUFF ###############################

		##################################################################################
		############################## READ PARAMETERS ###################################
		##################################################################################
		
		# READ MAIN FOLDER NAME
		main_folder = sep.from_cmd_line("main_folder")
		
		# READ MODE (EITHER COMPUTING GRADIENT OR VISUALIZATION)
		mode = sep.from_cmd_line("mode")
		
		# READ NICKNAME
		nickname = sep.from_cmd_line("nickname",default=main_folder)		
		
		# GET MODEL SIZES FROM INVERSION PARFILE
		main_parfile = sep.from_cmd_line("main_parfile")		
		nz = sep.from_par_file("nz",main_parfile)
		nx = sep.from_par_file("nx",main_parfile)
		z_pad = sep.from_par_file("z_pad",main_parfile)
		x_pad = sep.from_par_file("x_pad",main_parfile)		
		nz2 = str(int(nz) + 2 * int(z_pad))
		nx2 = str(int(nx) + 2 * int(x_pad))
	
		# GRADIENT COMPUTATION
		if (mode == "gradient"):
		
			# create gradient computation folder
			cmd = "mkdir -p %s" %(main_folder)
			sep.RunShellCmd_output(cmd)		

			# create the log file for the gradients computation
			main_log_file = cwd+main_folder+"/main_log_file.txt"
			cmd = "rm -f %s; touch %s"%(main_log_file,main_log_file)
			sep.RunShellCmd(cmd)	

			write_log_file(main_log_file,"#####################################################################################")
			write_log_file(main_log_file,"######################## SCRIPT THAT COMPUTES VPTFWI GRADIENTS ######################")
			write_log_file(main_log_file,"#####################################################################################\n")			
	
			# compute Born gradient		
			born = sep.from_cmd_line("born",default=True)
			if (born == "y" or born == "true" or born == "1"): 
				born = True	
				write_log_file(main_log_file,"will compute Born gradient\n ")

			# compute tomo gradient			
			tomo = sep.from_cmd_line("tomo",default=True)
			if (tomo == "y" or tomo == "true" or tomo == "1"): 
				tomo = True				
				write_log_file(main_log_file,"will compute tomo gradient \n")
				
			# READ DATA RESIDUAL
			data_residual = sep.from_cmd_line("data_residual")
			assert(os_sys.path.isfile(data_residual)),"Error: data file not found"		

			# READ REFLECTIVITY 
			reflectivity = sep.from_cmd_line("reflectivity")
			assert(os_sys.path.isfile(reflectivity)),"Error: reflectivity file not found"		
		
			# READ PADDED BACKGROUND VELOCITY MODEL
			vel = sep.from_cmd_line("vel")
			assert(os_sys.path.isfile(vel)),"Error: background velocity file not found"
		
			# READ ITERATION AXIS
			iteration_axis = sep.from_cmd_line("iteration_axis")

			# SET EXTENSIONS TO ZERO BY DEFAULT					
			hzmax = "0"
			hxmax = "0"
			htmax = "0"

			if (iteration_axis > "3"):
				cmd = "Get < %s n3 parform=n" %(reflectivity)			
				hzmax_int = (int(sep.RunShellCmd_output(cmd)) - 1) / 2
				hzmax = str(hzmax_int)

			if (iteration_axis > "4"):
				cmd = "Get < %s n4 parform=n" %(reflectivity)			
				hxmax_int = (int(sep.RunShellCmd_output(cmd)) - 1) / 2
				hxmax = str(hxmax_int)		

			if (iteration_axis > "5"):
				cmd = "Get < %s n5 parform=n" %(reflectivity)			
				htmax_int = (int(sep.RunShellCmd_output(cmd)) - 1) / 2
				htmax = str(htmax_int)
				
			# READ IF REFLECTIVITY IS PRECONDITIONED
			prec = sep.from_cmd_line("prec",default=False)
			if (prec == "y" or prec == "true" or prec == "1"): 
				prec = True	
				write_log_file(main_log_file,"vptfwi was preconditioned \n")
			
			# READ ACQUISITION PARAMETERS
			os = sep.from_cmd_line("os") # first source position
			ds = sep.from_cmd_line("ds") # source spacing
			nshots = sep.from_cmd_line("nshots") # number of shots 
			nshotjob = sep.from_cmd_line("nshotjob") # number of shots per job 

			# READ PBS PARAMETERS
			pbs_script = code_path+"/Scripts/pbs_tmpl_"+computer+".sh"
			pbs_template = sep.from_cmd_line("pbs_template",default=pbs_script)
			pbs_template = pbs_template.replace("/","\/")		
			queues = sep.from_cmd_line("queues",want_list=False,default="sep,default")		
			queues_cap = sep.from_cmd_line("queues_cap",want_list=False,default="30,30")	
			total_jobs_cap = sep.from_cmd_line("total_jobs_cap",want_list=False,default="34,200")
		
			# READ ITERATION NUMBER 
			iteration = sep.from_cmd_line("iteration",want_list=True)
			if (isinstance(iteration,list)):
				iteration_list = iteration

			else:
				iteration_list = [iteration]
				
			write_log_file(main_log_file,"gradient will be computed for iterations: %s \n" %(iteration_list))				
			
			# DEFINE BORN AND TOMO GRADIENT LISTS
			tomo_gradient_list = []
			born_gradient_list = []		  
			
			##################################################################################		
			############################## COMPUTE GRADIENTS #################################
			##################################################################################		

			# loop over iterations
			for iiteration in iteration_list:
		
				f_iteration = str(int(iiteration)-1)
		
				if ( (born == True) or (tomo == True) ):
			
					# extract data residuals and put it in folder
					data_residual_it = cwd+main_folder+"/"+nickname+"_data_residual_it%s.H" %(iiteration)
					cmd = "Window3d n4=1 f4=%s < %s > %s" %(f_iteration,data_residual,data_residual_it) 
					sep.RunShellCmd_output(cmd)			
		
				################################ BORN ####################################
		
				if (born == True):

					# write in log file
					write_log_file(main_log_file,"computing Born gradient for iteration #%s \n" %(iiteration))					
			
					# define Born gradient file name
					born_gradient_it = main_folder+"/"+nickname+"_born_gradient_it%s.H" %(iiteration)
		
					# compute Born gradient
					wrk_name = nickname+"_born_adj"
					cmd1 = "rm -f wrk/%s*;" %(wrk_name) 
					cmd2 = "%s pbs_template=%s vel=../%s par=../%s queues=%s os=%s nshots=%s nshotjob=%s ds=%s queues_cap=%s path_out=wrk prefix=%s total_jobs_cap=%s data=%s;" %(born_adj,pbs_template,vel,main_parfile,queues,os,nshots,nshotjob,ds,queues_cap,wrk_name,total_jobs_cap,data_residual_it)
					cmd2 = cmd2.replace("\/","/")						
					cmd3 = "Mv %s-final.H %s;" %(wrk_name,born_gradient_it)
					cmd4 = "Rm wrk/%s*.H;" %(wrk_name)
					cmd5 = "rm -f wrk/%s*;" %(wrk_name)
					cmd = cmd1+cmd2+cmd3+cmd4+cmd5	
					sep.RunShellCmd(cmd,True,True) # first one prints the command and the second one prints the output
				
					# cat all born gradients into one file
					born_gradient_list.append(born_gradient_it)					

				################################ TOMO ####################################

				if (tomo == True):

					# write in log file
					write_log_file(main_log_file,"computing tomo gradient for iteration #%s \n" %(iiteration))
			
					# extract reflectivity
					if (prec == True):

						# write to log file
						write_log_file(main_log_file,"linearized extended inversion was computed using preconditioning \n")

						# define name for preconditioned and unpreconditioned reflectivity slice for that iteration
						reflectivity_prec_it = main_folder+"/"+nickname+"_reflectivity_prec_it%s.H" %(iiteration)
						reflectivity_unprec_it = main_folder+"/"+nickname+"_reflectivity_unprec_it%s.H" %(iiteration)
							
						# window the reflectivity for that specific iteration
						cmd = "Window3d n%s=1 f%s=%s < %s > %s" %(iteration_axis,iteration_axis,f_iteration,reflectivity,reflectivity_prec_it) 							
						sep.RunShellCmd_output(cmd)
			
						# unprecondition the reflectivity
						cmd = dso_inv_exec+" alpha=1.0 adj=0 hzmax=%s hxmax=%s htmax=%s < %s > %s" %(hzmax,hxmax,htmax,reflectivity_prec_it,reflectivity_unprec_it)
						sep.RunShellCmd_output(cmd)
									
					else:

						# write to log file
						write_log_file(main_log_file,"linearized extended inversion was computed without preconditioning \n") 				
				
						# define name for unpreconditioned reflectivity slice for that iteration											
						reflectivity_unprec_it = main_folder+"/"+nickname+"_reflectivity_unprec_it%s.H" %(iiteration)
							
						# window the reflectivity for that specific iteration
						cmd = "Window3d n%s=1 f%s=%s < %s > %s" %(iteration_axis,iteration_axis,f_iteration,reflectivity,reflectivity_unprec_it) 							
						sep.RunShellCmd_output(cmd)

					# define tomo gradient file name
					tomo_gradient_it = main_folder+"/"+nickname+"_tomo_gradient_it%s.H" %(iiteration)
			
					# compute tomo gradient
					wrk_name = nickname+"_tomo_adj"
					cmd1 = "rm -f wrk/%s*;" %(wrk_name) 
					cmd2 = "%s pbs_template=%s vel=../%s hzmax=%s hxmax=%s htmax=%s par=../%s queues=%s os=%s nshots=%s nshotjob=%s ds=%s queues_cap=%s path_out=wrk prefix=%s total_jobs_cap=%s data=%s reflectivity=../%s;" %(ext_tomo_adj,pbs_template,vel,hzmax,hxmax,htmax,main_parfile,queues,os,nshots,nshotjob,ds,queues_cap,wrk_name,total_jobs_cap,data_residual_it,reflectivity_unprec_it)
					cmd2 = cmd2.replace("\/","/")						
					cmd3 = "Mv %s-final.H %s;" %(wrk_name,tomo_gradient_it)
					cmd4 = "Rm wrk/%s*.H;" %(wrk_name)					
					cmd5 = "rm -f wrk/*%s*;" %(wrk_name)
					cmd = cmd1+cmd2+cmd3+cmd4+cmd5
					sep.RunShellCmd(cmd,True,True) # first one prints the command and the second one prints the output				

					# cat all tomo gradient into one file
					tomo_gradient_list.append(tomo_gradient_it)
		
			# finished iterating over iteration number: cat the born and tomo gradients
			if (born == True):
			
				# create a file that contains all tomo gradients
				born_gradient_total = main_folder+"/"+nickname+"_born_gradient_total.H" 
				born_gradient_join = " ".join(born_gradient_list)										
				cmd = "Cat3d axis=3 %s > %s" %(born_gradient_join,born_gradient_total)
				sep.RunShellCmd_output(cmd)
				write_log_file(main_log_file,"finished computing all Born gradients \n")
			
			if (tomo == True):
			
				# create a file that contains all tomo gradients
				tomo_gradient_total = main_folder+"/"+nickname+"_tomo_gradient_total.H"
				tomo_gradient_join = " ".join(tomo_gradient_list)										
				cmd = "Cat3d axis=3 %s > %s" %(tomo_gradient_join,tomo_gradient_total)				
				sep.RunShellCmd_output(cmd)
				write_log_file(main_log_file,"finished computing all tomo gradients \n")
		
			################################ TOTAL #######################################
			if ( (born == True) and (tomo == True) ):

				# define total gradient file name
				total_gradient = main_folder+"/"+nickname+"_total_gradient_total.H"
				cmd = "Add %s %s scale=1,1 > %s" %(tomo_gradient_total,born_gradient_total,total_gradient)
				sep.RunShellCmd_output(cmd)
			

			write_log_file(main_log_file,"#####################################################################################")
			write_log_file(main_log_file,"######################## FINISHED COMPUTING VPTFWI GRADIENTS ########################")
			write_log_file(main_log_file,"#####################################################################################\n")
	
		##################################################################################		
		############################## DISPLAY GRADIENTS #################################
		##################################################################################	

		if (mode == "display"):
			
			disp1 = sep.from_cmd_line("disp1",default=True)
			if (disp1 == "y" or disp1 == "true" or disp1 == "1"): 
				disp1 = True				 			
			
			disp2 = sep.from_cmd_line("disp2",default=True)
			if (disp2 == "y" or disp2 == "true" or disp2 == "1"): 
				disp2 = True					
			
			# 1. specify iteration number and cat tomo/Born/total on 3rd axis			
			if ( disp1 == True or disp2 == True ):
			
				# read iteration at which you want to display the gradients
				iter_display = sep.from_cmd_line("iter_display")
			
				# read display parameters
				disp1_color = sep.from_cmd_line("disp1_color",default="j")
				disp1_pclip = sep.from_cmd_line("disp1_pclip",default="\"\"")				 
				disp1_gainpanel = sep.from_cmd_line("gainpanel",default="e") 				
				disp1_grid = sep.from_cmd_line("disp1_grid",default="n")
				disp1_d1num = sep.from_cmd_line("disp1_d1num",default="\"\"") 
				disp1_d2num = sep.from_cmd_line("disp1_d2num",default="\"\"")
				disp1_min1 = sep.from_cmd_line("disp1_min1",default="\"\"") 
				disp1_max1 = sep.from_cmd_line("disp1_max1",default="\"\"")
				disp1_min2 = sep.from_cmd_line("disp1_min2",default="\"\"")
				disp1_max2 = sep.from_cmd_line("disp1_max2",default="\"\"")
				disp1_title = sep.from_cmd_line("disp1_title",default="\"tomo gradient it%s:Born gradient it%s:total gradient it%s\"" %(iter_display,iter_display,iter_display))
				disp2_title = sep.from_cmd_line("disp2_title",default="\"tomo : Born : total [it%s]\"" %(iter_display))				
				disp1_label1 = sep.from_cmd_line("disp1_label1",default="\"z[km]\"") 
				disp1_label2 = sep.from_cmd_line("disp1_label2",default="\"x[km]\"") 
				disp1_pixmaps = sep.from_cmd_line("disp1_pixmaps",default="y") 			
				disp1_tpow = sep.from_cmd_line("disp1_tpow",default="0") 

				# get the names of the gradients
				tomo_gradient_it = main_folder+"/"+nickname+"_tomo_gradient_it%s.H" %(iter_display)
				tomo_gradient_it_windowed = main_folder+"/"+nickname+"_tomo_gradient_it%s_windowed.H" %(iter_display)				
				born_gradient_it = main_folder+"/"+nickname+"_born_gradient_it%s.H" %(iter_display)
				born_gradient_it_windowed = main_folder+"/"+nickname+"_born_gradient_it%s_windowed.H" %(iter_display)				
				total_gradient_it_windowed = main_folder+"/"+nickname+"_total_gradient_it%s_windowed.H" %(iter_display)				

				# choose if you want to display the padded zone
				disp1_pad = sep.from_cmd_line("disp1_pad",default=False)
				if (disp1_pad == "y" or disp1_pad == "true" or disp1_pad == "1"): 
					disp1_pad = True					

				# window the gradients
				if (disp1_pad == True):

					nz_temp = nz2
					nx_temp = nx2
					z_pad_temp = "0" 
					x_pad_temp = "0"
					
				else:
					nz_temp = nz
					nx_temp = nx
					z_pad_temp = z_pad 
					x_pad_temp = x_pad			
					
				cmd = "Window3d n1=%s f1=%s n2=%s f2=%s < %s > %s" %(nz,z_pad,nx,x_pad,tomo_gradient_it,tomo_gradient_it_windowed)
				sep.RunShellCmd_output(cmd)			
				cmd = "Window3d n1=%s f1=%s n2=%s f2=%s < %s > %s" %(nz,z_pad,nx,x_pad,born_gradient_it,born_gradient_it_windowed)
				sep.RunShellCmd_output(cmd)			

				# compute sum for this iteration
				cmd = "Add %s %s scale=1,1 > %s" %(tomo_gradient_it_windowed,born_gradient_it_windowed,total_gradient_it_windowed)
				sep.RunShellCmd_output(cmd)							

				if (disp1 == True):

					# display gradients cat'ed on 3rd axis 
					cmd = "Cat axis=3 %s %s %s | Scale | Tpow tpow=%s | Grey color=%s gainpanel=%s grid=%s d1num=%s d2num=%s min1=%s max1=%s min2=%s max2=%s titles=%s label1=%s label2=%s pclip=%s | Tube pixmaps=%s &" %(tomo_gradient_it_windowed,born_gradient_it_windowed,total_gradient_it_windowed,disp1_tpow,disp1_color,disp1_gainpanel,disp1_grid,disp1_d1num,disp1_d2num,disp1_min1,disp1_max1,disp1_min2,disp1_max2,disp1_title,disp1_label1,disp1_label2,disp1_pclip,disp1_pixmaps)
					sep.RunShellCmd(cmd,synch=False)	

				if (disp2 == True):				

					# display gradients cat'ed on 2nd axis 
					cmd = "Cat axis=2 %s %s %s | Scale | Tpow tpow=%s | Grey color=%s gainpanel=%s grid=%s d1num=%s d2num=%s min1=%s max1=%s min2=%s max2=%s title=%s label1=%s label2=%s pclip=%s | Tube pixmaps=%s &" %(tomo_gradient_it_windowed,born_gradient_it_windowed,total_gradient_it_windowed,disp1_tpow,disp1_color,disp1_gainpanel,disp1_grid,disp1_d1num,disp1_d2num,disp1_min1,disp1_max1,disp1_min2,disp1_max2,disp2_title,disp1_label1,disp1_label2,disp1_pclip,disp1_pixmaps)
					sep.RunShellCmd(cmd,synch=False)	


			# 3. cat the total gradient for each iteration on the 3rd axis
			
			# 4. cat the tomo/Born/total on the second axis and loop on the 3rd axis over iteration
			
			
			
	
	
	
	
	
	
	
	
	
	
			