#!/usr/bin/python
import sep_python as sep
import operator_obj as op 
import sys
import os as os_sys
import re
from python_solver_outcore import write_log_file 
from timeit import default_timer as timer

##########################################################################################	
################################ MAIN ####################################################	
##########################################################################################	

if __name__ == '__main__':
##########################################################################################		
############################### VPTFWI MAIN  #############################################
##########################################################################################
	
	if (len(sys.argv) == 1):
        # write self documentation	
		print 'self doc' 

	else:

		##################################################################################	
		############################### READ PARAMETERS ##################################
		##################################################################################
		
		print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
		print "!!!!!!!!!!!!!!!!!!!!!!!!! VPTFWI INVERSION !!!!!!!!!!!!!!!!!!!!!!"
		print "!!!!!!!!!!!!!! A PRODUCT DESIGNED AND DEVELOPED BY E&G !!!!!!!!!!"		  
		print "!!!!!!!!!!!!!!!!!!!!!!!!!! COPYRIGHT 2017 !!!!!!!!!!!!!!!!!!!!!!!"		  		
		print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n"	
		
		# get the path to the solver
		python_path_list = os_sys.environ['PYTHONPATH'].split(":")
		indx = [i for i, s in enumerate(python_path_list) if '/python_solver' in s]
		pathsource = python_path_list[indx[0]]
		
		# get the scratch path
		scratch_path = sep.sep_find_datapath()
		scratch_path = scratch_path.replace("/","\/")
		
		# create executable names
		nl_solver = pathsource+"/generic_non_linear_prob.py"
		l_solver = 	pathsource+"/generic_linear_prob.py"	
		l_solver_backslash = l_solver.replace("/","\/")	
		
		# get the main directory where the executables are
		# ex: /data/biondo/gbarnier/code/acoustic
		# does not contain any slash at the end of the string
		code_path = os_sys.environ['CODE_PATH']
		code_path_backslash = code_path.replace("/","\/")	

		# get the directory where the executables are
		exec_path = code_path+"/Bin"	
		exec_path_backslash = exec_path.replace("/","\/")			

		# get the script directory path
		scripts_path = code_path+"/Scripts"
		scripts_path_backslash = scripts_path.replace("/","\/")			

		# import script for parsing
		parsing_script = scripts_path+"/vptfwi_vp_parse_main.py"
		parsing_script_backslash = parsing_script.replace("/","\/")		
		
		# import VPTFWI type
		# std = standard 
		# onepar = one parameter problem 
		vptfwi_type = sep.from_cmd_line("vptfwi_type",default="std")

		# import reg type (default is preconditioned)
		reg_type = sep.from_cmd_line("reg_type")
		
		# template parfiles
		nonlinear_parfile_template = scripts_path+"/vptfwi.parfile.templates/"+vptfwi_type+".vptfwi."+reg_type+".nonlinear.template.p"
		lin_fwd_parfile_template = scripts_path+"/vptfwi.parfile.templates/"+vptfwi_type+".vptfwi."+reg_type+".lin.fwd.template.p"
		lin_adj_parfile_template = scripts_path+"/vptfwi.parfile.templates/"+vptfwi_type+".vptfwi."+reg_type+".lin.adj.template.p"
		lin_fwd_dso_parfile_template = scripts_path+"/vptfwi.parfile.templates/"+vptfwi_type+".vptfwi."+reg_type+".fwd.dso.template.p"
		lin_adj_dso_parfile_template = scripts_path+"/vptfwi.parfile.templates/"+vptfwi_type+".vptfwi."+reg_type+".adj.dso.template.p"
		vp_inv_lin_fwd_parfile_template = scripts_path+"/vptfwi.parfile.templates/"+vptfwi_type+".vptfwi.vp.inv."+reg_type+".fwd.template.p"
		vp_inv_lin_adj_parfile_template = scripts_path+"/vptfwi.parfile.templates/"+vptfwi_type+".vptfwi.vp.inv."+reg_type+".adj.template.p"	
		vp_inv_lin_fwd_dso_parfile_template = scripts_path+"/vptfwi.parfile.templates/"+vptfwi_type+".vptfwi.vp.inv."+reg_type+".fwd.dso.template.p"			
		vp_inv_lin_adj_dso_parfile_template = scripts_path+"/vptfwi.parfile.templates/"+vptfwi_type+".vptfwi.vp.inv."+reg_type+".adj.dso.template.p"	

		# name of the working directory (for the main inversion)
		main_wrk_dir = sep.from_cmd_line("wrk_dir")

		# prefix for the vptfwi inversion
		prefix = sep.from_cmd_line("prefix")
		
		# prefix for the wrk directory for the batch task executors
		# we need to keep that name relatively short to avoid PBS issues
		wrk_prefix = sep.from_cmd_line("wrk_prefix")		
		
		# read name for the general parfile  
		main_parfile = sep.from_cmd_line("main_parfile")
		main_parfile = main_parfile.replace("/","\/")		

		# define name for jobs prefix for nonlinear parfile
		wrk_prefix_nl_name1 = wrk_prefix+"nltp1"
		wrk_prefix_nl_name2 = wrk_prefix+"nltp2"		 

		# define name for jobs prefix for adj parfile (no need for fwd parfile)
		wrk_prefix_adj_name1 = wrk_prefix+"adtp1"
		wrk_prefix_adj_name2 = wrk_prefix+"adtp2"		
		
		# define name for jobs prefix for fwd VP parfile 		
		wrk_prefix_vp_fwd_name = wrk_prefix+"vpftp"		
		
		# define name for jobs prefix for fwd VP parfile 				
		wrk_prefix_vp_adj_name = wrk_prefix+"vpatp"		

		# read the suffix of the vptfwi inversion
		suffix = sep.from_cmd_line("suffix")

		# check if user wants debug mode
		debug = sep.from_cmd_line("debug") 

		# read data file name
		data_file = sep.from_cmd_line("data")
		assert(os_sys.path.isfile(data_file)),"Error: data file not found"			
		data_file = data_file.replace("/","\/")				

		# read initial model file name (non-extended and padded)
		initial_model = sep.from_cmd_line("init_model")
		assert(os_sys.path.isfile(initial_model)),"Error: initial \"nonlinear\" model file not found"
		initial_model = initial_model.replace("/","\/")
		
		# read the starting model used for VP inversions
		vp_initial_model = sep.from_cmd_line("vp_init_model")		
		assert(os_sys.path.isfile(vp_initial_model)),"Error: initial vp model file not found"		

		# read inverted model file name (non-extended and padded)
		inverted_model = sep.from_cmd_line("inv_model")

		# read log file name
		main_log_file = sep.from_cmd_line("log_file")

		# number of iterations
		niter_outer = sep.from_cmd_line("niter_outer") # number of nonlinear iterations
		niter_vp = sep.from_cmd_line("niter_vp") # number iterations for variable projection 

		# read acquisition parameters
		os = sep.from_cmd_line("os") # first source position
		ds = sep.from_cmd_line("ds") # source spacing
		nshots = sep.from_cmd_line("nshots") # number of shots 
		nshotjob = sep.from_cmd_line("nshotjob") # number of shots per job 

		# read PBS parameters
		pbs_template = sep.from_cmd_line("pbs_template")
		pbs_template = pbs_template.replace("/","\/")		
		queues = sep.from_cmd_line("queues",want_list=False)		
		queues_cap = sep.from_cmd_line("queues_cap",want_list=False)	
		total_jobs_cap = sep.from_cmd_line("total_jobs_cap",want_list=False)			
		
		# read parameters for inversion from command line
		hzmax = sep.from_cmd_line("hzmax") # offsets in z
		hxmax = sep.from_cmd_line("hxmax") # offsets in x		
		htmax = sep.from_cmd_line("htmax") # time-lags
		
		# read the epsilon value
		epsilon = sep.from_cmd_line("epsilon")		

		# read the alpha value - this is the initial guess of the first step length
		alpha = sep.from_cmd_line("alpha")		

		# read name of padded mask - if it doesn't exist, create a padded mask with ones everywhere
		mask_file = sep.from_cmd_line("mask",default=None) 
		
		if (mask_file == None):
			cmd = "Math file1=%s exp='file1*0.0+1.0' > %s" %(initial_model,prefix+".mask.gradient.H")
			sep.RunShellCmd(cmd)
			mask_file = prefix+".mask.gradient.H"
			mask_file = mask_file.replace("/","\/")	

		else:
			mask_file = mask_file.replace("/","\/")								

		# read whether to restart inversion or not
		restart = sep.from_cmd_line("restart",default="n") 
		if (restart == "y" or restart == "true" or restart == "1"): 
			restart = True	

		# read the bound values (minval and maxval)
		minval = sep.from_cmd_line("minval",default=" ")		
		maxval = sep.from_cmd_line("maxval",default=" ")						

		##################################################################################	
		############################### CREATE PARFILES ##################################
		##################################################################################	
	
		####################### CREATE OUTPUT FILE NAMES #################################
		
		# name for the nonlinear parfile (the one that contains the linear inversion)
		nonlinear_parfile = "Par/"+prefix+"."+vptfwi_type+".vptfwi."+reg_type+".nonlinear.p"

		# name for the linearized fwd parfile (this one is not used because we do not compute the linearized fwd with the VP)
		lin_fwd_parfile = "Par/"+prefix+"."+vptfwi_type+".vptfwi."+reg_type+".lin.fwd.p"

		# name for the linearized adj parfile 
		lin_adj_parfile = "Par/"+prefix+"."+vptfwi_type+".vptfwi."+reg_type+".lin.adj.p"		

		# name for the DSO fwd parfile 
		lin_fwd_dso_parfile	= "Par/"+prefix+"."+vptfwi_type+".vptfwi."+reg_type+".fwd.dso.p"		

		# name for the DSO adj parfile 
		lin_adj_dso_parfile	= "Par/"+prefix+"."+vptfwi_type+".vptfwi."+reg_type+".adj.dso.p" 
		lin_adj_dso_parfile = lin_adj_dso_parfile.replace("/","\/")

		# name for the variable projection fwd parfile
		vp_inv_lin_fwd_parfile	= "Par/"+prefix+"."+vptfwi_type+".vptfwi.vp.inv."+reg_type+".fwd.p" 
		vp_inv_lin_fwd_parfile = vp_inv_lin_fwd_parfile.replace("/","\/")		
		
		# name for the variable projection adj parfile
		vp_inv_lin_adj_parfile	= "Par/"+prefix+"."+vptfwi_type+".vptfwi.vp.inv."+reg_type+".adj.p" 		
		vp_inv_lin_adj_parfile = vp_inv_lin_adj_parfile.replace("/","\/")	

		if (reg_type == "reg"):
			# name for the variable projection fwd DSO parfile		
			vp_inv_lin_fwd_dso_parfile = "Par/"+prefix+"."+vptfwi_type+".vptfwi.vp.inv."+reg_type+".fwd.dso.p" 		
			vp_inv_lin_fwd_dso_parfile = vp_inv_lin_fwd_dso_parfile.replace("/","\/")	

			# name for the variable projection adj DSO parfile		
			vp_inv_lin_adj_dso_parfile = "Par/"+prefix+"."+vptfwi_type+".vptfwi.vp.inv."+reg_type+".adj.dso.p" 		
			vp_inv_lin_adj_dso_parfile = vp_inv_lin_adj_dso_parfile.replace("/","\/")	
				
		####################### APPEND NONLINEAR PARFILE #################################
		
		# change temp names
		cmd = "sed 's/name1/%s/g' < %s > %s" %(wrk_prefix_nl_name1,nonlinear_parfile_template,nonlinear_parfile)
		sep.RunShellCmd(cmd)
		cmd = "sed -i 's/name2/%s/g' %s" %(wrk_prefix_nl_name2,nonlinear_parfile)
		sep.RunShellCmd(cmd)

		# change the name of the scripts
		cmd = "sed -i 's/scriptspath/%s/g' %s" %(scripts_path_backslash,nonlinear_parfile)
		sep.RunShellCmd(cmd)
		
		# change the name of the linear solver
		cmd = "sed -i 's/linearsolver/%s/g' %s" %(l_solver_backslash,nonlinear_parfile)
		sep.RunShellCmd(cmd)		

		# change the name of the executable path
		cmd = "sed -i 's/execpath/%s/g' %s" %(exec_path_backslash,nonlinear_parfile)
		sep.RunShellCmd(cmd)	

		# fill out the name of parsing script
		cmd = "sed -i 's/vp_parsing_script/%s/g' %s" %(parsing_script_backslash,nonlinear_parfile)
		sep.RunShellCmd(cmd)

		# fill out the name of main log file
		cmd = "sed -i 's/main_log_file=/main_log_file=%s/g' %s" %(main_log_file,nonlinear_parfile)
		sep.RunShellCmd(cmd)				

		# fill out the name of main work directory
		cmd = "sed -i 's/main_wrk_dir=/main_wrk_dir=%s/g' %s" %(main_wrk_dir,nonlinear_parfile)
		sep.RunShellCmd(cmd)				

		# fill out the name of main suffix
		cmd = "sed -i 's/main_suffix=/main_suffix=%s/g' %s" %(suffix,nonlinear_parfile)
		sep.RunShellCmd(cmd)

		# change the name of the current background model
		cmd = "sed -i 's/current_background_template.H/%s/g' %s" %(prefix+".current.background.H",nonlinear_parfile)
		sep.RunShellCmd(cmd)				
		
		# change the name of the PBS template
		cmd = "sed -i 's/pbs_template=/%s/g' %s" %("pbs_template="+pbs_template,nonlinear_parfile)
		sep.RunShellCmd(cmd)			
		
		# change the name of the queues
		cmd = "sed -i 's/queues=/%s/g' %s" %("queues="+queues,nonlinear_parfile)
		sep.RunShellCmd(cmd)			

		# change the name of the queues cap
		cmd = "sed -i 's/queues_cap=/%s/g' %s" %("queues_cap="+queues_cap,nonlinear_parfile)
		sep.RunShellCmd(cmd)								
				
		# change the name of the total jobs cap
		cmd = "sed -i 's/total_jobs_cap=/%s/g' %s" %("total_jobs_cap="+total_jobs_cap,nonlinear_parfile)
		sep.RunShellCmd(cmd)				
				
		# change the name of the main parfile (the one for the propagation)
		cmd = "sed -i 's/par=/%s/g' %s" %("par=..\/"+main_parfile,nonlinear_parfile)
		sep.RunShellCmd(cmd)					
				
		# change the name of the main parfile (the one for the propagation) without the ../ 
		# this is needed for the spreading operator in the case of onepar
		cmd = "sed -i 's/par_up=/%s/g' %s" %("par="+main_parfile,nonlinear_parfile)
		sep.RunShellCmd(cmd)							
				
		# change the acquisition parameters
		cmd = "sed -i 's/os=/%s/g' %s" %("os="+os,nonlinear_parfile)
		sep.RunShellCmd(cmd)
		cmd = "sed -i 's/ds=/%s/g' %s" %("ds="+ds,nonlinear_parfile)
		sep.RunShellCmd(cmd)						
		cmd = "sed -i 's/nshots=/%s/g' %s" %("nshots="+nshots,nonlinear_parfile)		
		sep.RunShellCmd(cmd)								
		cmd = "sed -i 's/nshotjob=/%s/g' %s" %("nshotjob="+nshotjob,nonlinear_parfile)		
		sep.RunShellCmd(cmd)					
				
		# change the data and data residual name
		cmd = "sed -i 's/data_template.H/%s/g' %s" %(data_file,nonlinear_parfile)
		sep.RunShellCmd(cmd)
		cmd = "sed -i 's/data_res_template.H/%s/g' %s" %(prefix+".data.res.H",nonlinear_parfile)
		sep.RunShellCmd(cmd)				
				
		# change the extension parameters
		cmd = "sed -i 's/hzmax=/%s/g' %s" %("hzmax="+hzmax,nonlinear_parfile)
		sep.RunShellCmd(cmd)
		cmd = "sed -i 's/hxmax=/%s/g' %s" %("hxmax="+hxmax,nonlinear_parfile)
		sep.RunShellCmd(cmd)
		cmd = "sed -i 's/htmax=/%s/g' %s" %("htmax="+htmax,nonlinear_parfile)
		sep.RunShellCmd(cmd)				
				
		# change epsilon
		cmd = "sed -i 's/epsilon=/%s/g' %s" %("epsilon="+epsilon,nonlinear_parfile)
		sep.RunShellCmd(cmd)
		
		# change debug parameter
		cmd = "sed -i 's/debug=/%s/g' %s" %("debug="+debug,nonlinear_parfile)
		sep.RunShellCmd(cmd)	
		
		# change number of linear iterations for variable projection
		cmd = "sed -i 's/niter=/%s/g' %s" %("niter="+niter_vp,nonlinear_parfile)
		sep.RunShellCmd(cmd)								
				
		# change the name of starting model for the extended linearized inversion
		cmd = "sed -i 's/init_model=/%s/g' %s" %("init_model="+vp_initial_model,nonlinear_parfile)
		sep.RunShellCmd(cmd)					
				
		# change name of parameter files for VP inversion
		cmd = "sed -i 's/fwd_cmd_file=/%s/g' %s" %("fwd_cmd_file="+vp_inv_lin_fwd_parfile,nonlinear_parfile)
		sep.RunShellCmd(cmd)						
		cmd = "sed -i 's/adj_cmd_file=/%s/g' %s" %("adj_cmd_file="+vp_inv_lin_adj_parfile,nonlinear_parfile)		
		sep.RunShellCmd(cmd)					

		if (reg_type == "reg"):
			cmd = "sed -i 's/fwd_cmd_reg_file=/%s/g' %s" %("fwd_cmd_reg_file="+vp_inv_lin_fwd_dso_parfile,nonlinear_parfile)
			sep.RunShellCmd(cmd)						
			cmd = "sed -i 's/adj_cmd_reg_file=/%s/g' %s" %("adj_cmd_reg_file="+vp_inv_lin_adj_dso_parfile,nonlinear_parfile)		
			sep.RunShellCmd(cmd)								
	
		####################### APPEND FORWARD PARFILE ###################################	
		# no linearized fwd is applied for VPTFWI - the first step length is computed/set manually

		# create the parfile for the linearized FWD
		cmd = "sed 's/message/%s/g' < %s > %s" %("nothing to be done",lin_fwd_parfile_template,lin_fwd_parfile)
		sep.RunShellCmd(cmd)

		####################### APPEND ADJOINT PARFILE ###################################		

		# change temp names
		cmd = "sed 's/name1/%s/g' < %s > %s" %(wrk_prefix_adj_name1,lin_adj_parfile_template,lin_adj_parfile)
		sep.RunShellCmd(cmd)
		cmd = "sed -i 's/name2/%s/g' %s" %(wrk_prefix_adj_name2,lin_adj_parfile)
		sep.RunShellCmd(cmd)	
		
		# change the name of the scripts
		cmd = "sed -i 's/scriptspath/%s/g' %s" %(scripts_path_backslash,lin_adj_parfile)
		sep.RunShellCmd(cmd)
		
		# change the name of the executable path
		cmd = "sed -i 's/execpath/%s/g' %s" %(exec_path_backslash,lin_adj_parfile)
		sep.RunShellCmd(cmd)		
		
		# change the temporary output names
		cmd = "sed -i 's/junk1_template.H/%s/g' %s" %(scratch_path+prefix+".temporary.adj.output1.H",lin_adj_parfile)
		sep.RunShellCmd(cmd)		
		cmd = "sed -i 's/junk2_template.H/%s/g' %s" %(scratch_path+prefix+".temporary.adj.output2.H",lin_adj_parfile)
		sep.RunShellCmd(cmd)				
		# this one is for the case where you are using a onepar inversion and you have to stack the tomo and Born adjoints 
		cmd = "sed -i 's/junk3_template.H/%s/g' %s" %(scratch_path+prefix+".temporary.adj.output3.H",lin_adj_parfile)
		sep.RunShellCmd(cmd)						

		# change the acquisition parameters
		cmd = "sed -i 's/os=/%s/g' %s" %("os="+os,lin_adj_parfile)
		sep.RunShellCmd(cmd)
		cmd = "sed -i 's/ds=/%s/g' %s" %("ds="+ds,lin_adj_parfile)
		sep.RunShellCmd(cmd)						
		cmd = "sed -i 's/nshots=/%s/g' %s" %("nshots="+nshots,lin_adj_parfile)		
		sep.RunShellCmd(cmd)								
		cmd = "sed -i 's/nshotjob=/%s/g' %s" %("nshotjob="+nshotjob,lin_adj_parfile)		
		sep.RunShellCmd(cmd)			
		
		# change the extension parameters
		cmd = "sed -i 's/hzmax=/%s/g' %s" %("hzmax="+hzmax,lin_adj_parfile)
		sep.RunShellCmd(cmd)
		cmd = "sed -i 's/hxmax=/%s/g' %s" %("hxmax="+hxmax,lin_adj_parfile)
		sep.RunShellCmd(cmd)
		cmd = "sed -i 's/htmax=/%s/g' %s" %("htmax="+htmax,lin_adj_parfile)
		sep.RunShellCmd(cmd)		
		
		# change the name of the PBS template
		cmd = "sed -i 's/pbs_template=/%s/g' %s" %("pbs_template="+pbs_template,lin_adj_parfile)
		sep.RunShellCmd(cmd)			
		
		# change the name of the queues
		cmd = "sed -i 's/queues=/%s/g' %s" %("queues="+queues,lin_adj_parfile)
		sep.RunShellCmd(cmd)			

		# change the name of the queues cap
		cmd = "sed -i 's/queues_cap=/%s/g' %s" %("queues_cap="+queues_cap,lin_adj_parfile)
		sep.RunShellCmd(cmd)								
				
		# change the name of the total jobs cap
		cmd = "sed -i 's/total_jobs_cap=/%s/g' %s" %("total_jobs_cap="+total_jobs_cap,lin_adj_parfile)
		sep.RunShellCmd(cmd)		
				
		# change the name of the main parfile (the one for the propagation)
		cmd = "sed -i 's/par=/%s/g' %s" %("par=..\/"+main_parfile,lin_adj_parfile)
		sep.RunShellCmd(cmd)	
		
		# change the name of the main parfile (the one for the propagation) without the ../
		# this is needed for the spreading operator in the case of onepar		
		cmd = "sed -i 's/par_up=/%s/g' %s" %("par="+main_parfile,lin_adj_parfile)
		sep.RunShellCmd(cmd)			

		# change the name of the output born adjoint
		cmd = "sed -i 's/output_born_masked_template.H/%s/g' %s" %(prefix+".output.born.adjoint.masked.H",lin_adj_parfile)
		sep.RunShellCmd(cmd)

		# change the name of the output born adjoint
		cmd = "sed -i 's/output_tomo_masked_template.H/%s/g' %s" %(prefix+".output.tomo.adjoint.masked.H",lin_adj_parfile)
		sep.RunShellCmd(cmd)

		# fill out the name of parsing script
		cmd = "sed -i 's/vp_parsing_script/%s/g' %s" %(parsing_script_backslash,lin_adj_parfile)
		sep.RunShellCmd(cmd)

		# fill out the name of main log file
		cmd = "sed -i 's/main_log_file=/main_log_file=%s/g' %s" %(main_log_file,lin_adj_parfile)
		sep.RunShellCmd(cmd)				

		# fill out the name of main work directory
		cmd = "sed -i 's/main_wrk_dir=/main_wrk_dir=%s/g' %s" %(main_wrk_dir,lin_adj_parfile)
		sep.RunShellCmd(cmd)				

		# fill out the name of main suffix
		cmd = "sed -i 's/main_suffix=/main_suffix=%s/g' %s" %(suffix,lin_adj_parfile)
		sep.RunShellCmd(cmd)

		# change the name of the gradient mask
		cmd = "sed -i 's/mask_template.H/%s/g' %s" %(mask_file,lin_adj_parfile)
		sep.RunShellCmd(cmd)		

		# change the name of the folder
		cmd = "sed -i 's/InversionFolder/%s/g' %s" %(main_wrk_dir,lin_adj_parfile)
		sep.RunShellCmd(cmd)		
		
		# change the name of the inversion folder to store Born/tomo gradient
		cmd = "sed -i 's/inversion_folder/%s/g' %s" %(main_wrk_dir,lin_adj_parfile)
		sep.RunShellCmd(cmd)

		# change the name of the file to store the intermediate gradients (Born and tomo)
 		cmd = "sed -i 's/SUFFIX/%s/g' %s" %(suffix,lin_adj_parfile)
 		sep.RunShellCmd(cmd)
 		
 		if (vptfwi_type == "onepar"):
			cmd = "sed -i 's/before_spread/%s/g' %s" %("input_m0.H",lin_adj_parfile)
			sep.RunShellCmd(cmd)	 			
			cmd = "sed -i 's/after_spread/%s/g' %s" %(prefix+".current.background.adj.H",lin_adj_parfile)
			sep.RunShellCmd(cmd)	
			
		########################## APPEND DSO PARFILES ###################################			

		#### change the DSO adjoint
		
		# set the output to zero (has to be the size of a non-extended model)
		cmd = "sed 's/velocity_template.H/%s/g' < %s > %s" %(initial_model,lin_adj_dso_parfile_template,lin_adj_dso_parfile)
		sep.RunShellCmd(cmd)	
		
		#### change the DSO forward

		# fill out the name of parsing script
		cmd = "sed 's/vp_parsing_script/%s/g' < %s > %s" %(parsing_script_backslash,lin_fwd_dso_parfile_template,lin_fwd_dso_parfile)
		sep.RunShellCmd(cmd)

		# fill out the name of main log file
		cmd = "sed -i 's/main_log_file=/main_log_file=%s/g' %s" %(main_log_file,lin_fwd_dso_parfile)
		sep.RunShellCmd(cmd)				

		# fill out the name of main work directory
		cmd = "sed -i 's/main_wrk_dir=/main_wrk_dir=%s/g' %s" %(main_wrk_dir,lin_fwd_dso_parfile)
		sep.RunShellCmd(cmd)				

		# fill out the name of main suffix
		cmd = "sed -i 's/main_suffix=/main_suffix=%s/g' %s" %(suffix,lin_fwd_dso_parfile)
		sep.RunShellCmd(cmd)		

		# change the name of the executable
		cmd = "sed -i 's/execpath/%s/g' %s" %(exec_path_backslash,lin_fwd_dso_parfile)
		sep.RunShellCmd(cmd)

		# change the extension parameters
		cmd = "sed -i 's/hzmax=/%s/g' %s" %("hzmax="+hzmax,lin_fwd_dso_parfile)
		sep.RunShellCmd(cmd)
		cmd = "sed -i 's/hxmax=/%s/g' %s" %("hxmax="+hxmax,lin_fwd_dso_parfile)
		sep.RunShellCmd(cmd)
		cmd = "sed -i 's/htmax=/%s/g' %s" %("htmax="+htmax,lin_fwd_dso_parfile)
		sep.RunShellCmd(cmd)	

		# change the scale parameter (epsilon)
		cmd = "sed -i 's/scale=/%s/g' %s" %("scale="+epsilon,lin_fwd_dso_parfile)
		sep.RunShellCmd(cmd)		

		# change the temporary output name
		cmd = "sed -i 's/dummy2/%s/g' %s" %(scratch_path+prefix+".inverted.model.vp.inversion.unprec.pad.after.dso.junk.H",lin_fwd_dso_parfile)
		sep.RunShellCmd(cmd)		

		################ APPEND FWD LINEAR PARFILE FOR VARIABLE PROJECTION ###############
		cmd = "sed 's/name1/%s/g' < %s > %s" %(wrk_prefix_vp_fwd_name,vp_inv_lin_fwd_parfile_template,vp_inv_lin_fwd_parfile)
		sep.RunShellCmd(cmd)

		# change the name of the executable
		cmd = "sed -i 's/execpath/%s/g' %s" %(exec_path_backslash,vp_inv_lin_fwd_parfile)
		sep.RunShellCmd(cmd)	
		
		# change the name of the scripts
		cmd = "sed -i 's/scriptspath/%s/g' %s" %(scripts_path_backslash,vp_inv_lin_fwd_parfile)
		sep.RunShellCmd(cmd)			

		# change the extension parameters
		cmd = "sed -i 's/hzmax=/%s/g' %s" %("hzmax="+hzmax,vp_inv_lin_fwd_parfile)
		sep.RunShellCmd(cmd)
		cmd = "sed -i 's/hxmax=/%s/g' %s" %("hxmax="+hxmax,vp_inv_lin_fwd_parfile)
		sep.RunShellCmd(cmd)
		cmd = "sed -i 's/htmax=/%s/g' %s" %("htmax="+htmax,vp_inv_lin_fwd_parfile)
		sep.RunShellCmd(cmd)

		# change the temporary output name
		cmd = "sed -i 's/junk1_template.H/%s/g' %s" %(scratch_path+prefix+".temporary.vp.inversion.output.fwd.H",vp_inv_lin_fwd_parfile)
		sep.RunShellCmd(cmd)

		# change the name of the PBS template
		cmd = "sed -i 's/pbs_template=/%s/g' %s" %("pbs_template="+pbs_template,vp_inv_lin_fwd_parfile)
		sep.RunShellCmd(cmd)	

		# change the name of the queues
		cmd = "sed -i 's/queues=/%s/g' %s" %("queues="+queues,vp_inv_lin_fwd_parfile)
		sep.RunShellCmd(cmd)			

		# change the name of the queues cap
		cmd = "sed -i 's/queues_cap=/%s/g' %s" %("queues_cap="+queues_cap,vp_inv_lin_fwd_parfile)
		sep.RunShellCmd(cmd)								
				
		# change the name of the total jobs cap
		cmd = "sed -i 's/total_jobs_cap=/%s/g' %s" %("total_jobs_cap="+total_jobs_cap,vp_inv_lin_fwd_parfile)
		sep.RunShellCmd(cmd)

		# change the acquisition parameters
		cmd = "sed -i 's/os=/%s/g' %s" %("os="+os,vp_inv_lin_fwd_parfile)
		sep.RunShellCmd(cmd)
		cmd = "sed -i 's/ds=/%s/g' %s" %("ds="+ds,vp_inv_lin_fwd_parfile)
		sep.RunShellCmd(cmd)						
		cmd = "sed -i 's/nshots=/%s/g' %s" %("nshots="+nshots,vp_inv_lin_fwd_parfile)		
		sep.RunShellCmd(cmd)								
		cmd = "sed -i 's/nshotjob=/%s/g' %s" %("nshotjob="+nshotjob,vp_inv_lin_fwd_parfile)		
		sep.RunShellCmd(cmd)	
		
		# change the name of the main parfile (the one for the propagation)
		cmd = "sed -i 's/par=/%s/g' %s" %("par=..\/"+main_parfile,vp_inv_lin_fwd_parfile)
		sep.RunShellCmd(cmd)		
		
		# change the name of the model (reflectivity)
		if (reg_type == "prec"):
			cmd = "sed -i 's/model=/%s/g' %s" %("model="+scratch_path+prefix+".temporary.vp.inversion.output.fwd.H",vp_inv_lin_fwd_parfile)
			sep.RunShellCmd(cmd)	
		else:		
			cmd = "sed -i 's/model=/%s/g' %s" %("model=input.H",vp_inv_lin_fwd_parfile)
			sep.RunShellCmd(cmd)	

		# change the name of the background velocity model
		cmd = "sed -i 's/vel=/%s/g' %s" %("vel=..\/"+prefix+".current.background.H",vp_inv_lin_fwd_parfile)
		sep.RunShellCmd(cmd)
		
		################ APPEND ADJ LINEAR PARFILE FOR VARIABLE PROJECTION ###############
		
		cmd = "sed 's/name1/%s/g' < %s > %s" %(wrk_prefix_vp_adj_name,vp_inv_lin_adj_parfile_template,vp_inv_lin_adj_parfile)
		sep.RunShellCmd(cmd)		
		
		if (reg_type == "prec"):
			# change the name of the intermediary output
			cmd = "sed -i 's/junk1_template.H/%s/g' %s" %(scratch_path+prefix+".temporary.vp.inversion.output.adj.H",vp_inv_lin_adj_parfile)
			sep.RunShellCmd(cmd)

		# change the name of the executable
		cmd = "sed -i 's/execpath/%s/g' %s" %(exec_path_backslash,vp_inv_lin_adj_parfile)
		sep.RunShellCmd(cmd)	
		
		# change the name of the scripts
		cmd = "sed -i 's/scriptspath/%s/g' %s" %(scripts_path_backslash,vp_inv_lin_adj_parfile)
		sep.RunShellCmd(cmd)			
		
		# change the extension parameters
		cmd = "sed -i 's/hzmax=/%s/g' %s" %("hzmax="+hzmax,vp_inv_lin_adj_parfile)
		sep.RunShellCmd(cmd)
		cmd = "sed -i 's/hxmax=/%s/g' %s" %("hxmax="+hxmax,vp_inv_lin_adj_parfile)
		sep.RunShellCmd(cmd)
		cmd = "sed -i 's/htmax=/%s/g' %s" %("htmax="+htmax,vp_inv_lin_adj_parfile)
		sep.RunShellCmd(cmd)	
		
		# change the name of the PBS template
		cmd = "sed -i 's/pbs_template=/%s/g' %s" %("pbs_template="+pbs_template,vp_inv_lin_adj_parfile)
		sep.RunShellCmd(cmd)	

		# change the name of the queues
		cmd = "sed -i 's/queues=/%s/g' %s" %("queues="+queues,vp_inv_lin_adj_parfile)
		sep.RunShellCmd(cmd)			

		# change the name of the queues cap
		cmd = "sed -i 's/queues_cap=/%s/g' %s" %("queues_cap="+queues_cap,vp_inv_lin_adj_parfile)
		sep.RunShellCmd(cmd)								
				
		# change the name of the total jobs cap
		cmd = "sed -i 's/total_jobs_cap=/%s/g' %s" %("total_jobs_cap="+total_jobs_cap,vp_inv_lin_adj_parfile)
		sep.RunShellCmd(cmd)		
			
		# change the name of the background velocity model
		cmd = "sed -i 's/vel=/%s/g' %s" %("vel=..\/"+prefix+".current.background.H",vp_inv_lin_adj_parfile)
		sep.RunShellCmd(cmd)		
		
		# change the acquisition parameters
		cmd = "sed -i 's/os=/%s/g' %s" %("os="+os,vp_inv_lin_adj_parfile)
		sep.RunShellCmd(cmd)
		cmd = "sed -i 's/ds=/%s/g' %s" %("ds="+ds,vp_inv_lin_adj_parfile)
		sep.RunShellCmd(cmd)						
		cmd = "sed -i 's/nshots=/%s/g' %s" %("nshots="+nshots,vp_inv_lin_adj_parfile)		
		sep.RunShellCmd(cmd)								
		cmd = "sed -i 's/nshotjob=/%s/g' %s" %("nshotjob="+nshotjob,vp_inv_lin_adj_parfile)		
		sep.RunShellCmd(cmd)			
		
		# change the name of the main parfile (the one for the propagation)
		cmd = "sed -i 's/par=/%s/g' %s" %("par=..\/"+main_parfile,vp_inv_lin_adj_parfile)
		sep.RunShellCmd(cmd)	
		
		################ APPEND LINEAR DSO PARFILE FOR VARIABLE PROJECTION ###############
		if (reg_type == "reg"):
			
			# fwd DSO 
			# change extension parameters
			cmd = "sed 's/hzmax=/%s/g' < %s > %s" %("hzmax="+hzmax,vp_inv_lin_fwd_dso_parfile_template,vp_inv_lin_fwd_dso_parfile)
			sep.RunShellCmd(cmd)
			cmd = "sed -i 's/hxmax=/%s/g' %s" %("hxmax="+hxmax,vp_inv_lin_fwd_dso_parfile)
			sep.RunShellCmd(cmd)
			cmd = "sed -i 's/htmax=/%s/g' %s" %("htmax="+htmax,vp_inv_lin_fwd_dso_parfile)
			sep.RunShellCmd(cmd)
			
			# change the name of the executable
			cmd = "sed -i 's/execpath/%s/g' %s" %(exec_path_backslash,vp_inv_lin_fwd_dso_parfile)
			sep.RunShellCmd(cmd)					

			# adj DSO 
			# change extension parameters
			cmd = "sed 's/hzmax=/%s/g' < %s > %s" %("hzmax="+hzmax,vp_inv_lin_adj_dso_parfile_template,vp_inv_lin_adj_dso_parfile)
			sep.RunShellCmd(cmd)
			cmd = "sed -i 's/hxmax=/%s/g' %s" %("hxmax="+hxmax,vp_inv_lin_adj_dso_parfile)
			sep.RunShellCmd(cmd)
			cmd = "sed -i 's/htmax=/%s/g' %s" %("htmax="+htmax,vp_inv_lin_adj_dso_parfile)
			sep.RunShellCmd(cmd)				
			# change the name of the executable
			cmd = "sed -i 's/execpath/%s/g' %s" %(exec_path_backslash,vp_inv_lin_adj_dso_parfile)
			sep.RunShellCmd(cmd)								
		
		################ CLEAN UP VP INVERSION FOLDER WHEN RESTART = Y ###################		
		if ( (restart == "y") and (check_parfiles != "y") ):
			
			# get the iteration number at which the solver crashed
			cmd = "more %s | grep \"iter = \" | tail -1" %(cwd+"/"+main_wrk_dir+"/"+main_log_file)
			outrestart = sep.RunShellCmd_output(cmd)
			string_expression = re.compile("iter = ([-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?)")			
			find = string_expression.search(outrestart)
			assert(find !=None), "Error! Could not find iteration number for restarting!"
			iter_crash = find.group(1)
			
			# remove the vp inversion folder corresponding to that nonlinear iteration
			cmd = "Rm %s/vp_inversion_it%s_point*/*.H" %(main_wrk_dir,iter_crash) 
			sep.RunShellCmd(cmd)
			cmd = "rm -rf %s/vp_inversion_it%s_point*" %(main_wrk_dir,iter_crash) 
			sep.RunShellCmd(cmd)

		########################## LAUNCH VPTFWI INVERSION ###############################
		# read "check" parameter which controls whether to launch the inversion or not
		# if "y", then the inversion is not launched, only the parfiles are created so that we can QC them before launching the whole thing
		check_parfiles = sep.from_cmd_line("check_parfiles")
		manual_launch = sep.from_cmd_line("manual_launch",default=False)		
		# res_panel1 = display residuals cat'ed on third axis	
		if (manual_launch == "y" or manual_launch == "true" or manual_launch == "1"): 
			manual_launch = True	
		else:
			manual_launch = False				

		if (check_parfiles == 'y'):
		
			lin_adj_dso_parfile = lin_adj_dso_parfile.replace("\/","/")
			vp_inv_lin_fwd_parfile = vp_inv_lin_fwd_parfile.replace("\/","/")
			vp_inv_lin_adj_parfile = vp_inv_lin_adj_parfile.replace("\/","/")			

			if (reg_type == "reg"):
				vp_inv_lin_fwd_dso_parfile = vp_inv_lin_fwd_dso_parfile.replace("\/","/")
				vp_inv_lin_adj_dso_parfile = vp_inv_lin_adj_dso_parfile.replace("\/","/")			
												
			print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
			print "!! NOT LAUNCHING VPTFWI - ONLY GENERATING THE PARFILES FOR QC !!!"  
			print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
			if (vptfwi_type == "std"):
				print "VPTFWI TYPE: STANDARD INVERSION"
			else:
				print "VPTFWI TYPE: ONE PARAMETER INVERSION"			
			if (reg_type == "reg"):
				print "REGULARIZATION TYPE: REGULARIZED"
			else:
				print "REGULARIZATION TYPE: PRECONDITIONED"
			print "nonlinear parfile: %s" %(nonlinear_parfile) 
			print "lin fwd parfile: %s" %(lin_fwd_parfile) 	
			print "lin adj parfile: %s" %(lin_adj_parfile)
			print "lin fwd dso parfile: %s" %(lin_fwd_dso_parfile)
			print "lin adj dso parfile: %s" %(lin_adj_dso_parfile)
			print "VP lin fwd parfile: %s" %(vp_inv_lin_fwd_parfile)									 						     	
			print "VP lin adj parfile: %s" %(vp_inv_lin_adj_parfile)
			if (reg_type == "reg"):
				print "VP lin fwd DSO parfile: %s" %(vp_inv_lin_fwd_dso_parfile)									 						     	
				print "VP lin adj DSO parfile: %s" %(vp_inv_lin_adj_dso_parfile)												 						     				
			print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n"
						
		else:

			lin_adj_dso_parfile = lin_adj_dso_parfile.replace("\/","/")
			vp_inv_lin_fwd_parfile = vp_inv_lin_fwd_parfile.replace("\/","/")
			vp_inv_lin_adj_parfile = vp_inv_lin_adj_parfile.replace("\/","/")
			
			if (reg_type == "reg"):
				vp_inv_lin_fwd_dso_parfile = vp_inv_lin_fwd_dso_parfile.replace("\/","/")
				vp_inv_lin_adj_dso_parfile = vp_inv_lin_adj_dso_parfile.replace("\/","/")				
									
			print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
			print "!!!!!!!!!!!!!!!!!!!! PARFILES CREATED: !!!!!!!!!!!!!!!!!!!!!!!!!!"  
			print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
			if (vptfwi_type == "std"):
				print "VPTFWI TYPE: STANDARD INVERSION"
			else:
				print "VPTFWI TYPE: ONE PARAMETER INVERSION"			
			if (reg_type == "reg"):
				print "REGULARIZATION TYPE: REGULARIZED"
			else:
				print "REGULARIZATION TYPE: PRECONDITIONED"
			print "nonlinear parfile: %s" %(nonlinear_parfile) 
			print "lin fwd parfile: %s" %(lin_fwd_parfile) 	
			print "lin adj parfile: %s" %(lin_adj_parfile)
			print "lin fwd dso parfile: %s" %(lin_fwd_dso_parfile)
			print "lin adj dso parfile: %s" %(lin_adj_dso_parfile)
			print "VP lin fwd parfile: %s" %(vp_inv_lin_fwd_parfile)									 						     	
			print "VP lin adj parfile: %s" %(vp_inv_lin_adj_parfile)
			if (reg_type == "reg"):
				print "VP lin fwd DSO parfile: %s" %(vp_inv_lin_fwd_dso_parfile)									 						     	
				print "VP lin adj DSO parfile: %s" %(vp_inv_lin_adj_dso_parfile)												 						     				
			print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n"

			if (manual_launch == True):
				print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n"   	
				print "!!!!!!!!!!!!!!!!!!!! MANUAL LAUNCH !!!!!!!!!!!!!!!!!!!!!!!!!!\n"   	
				print "!!! JUST DISPLAYING THE COMMAND TO LAUNCH THE INVERSION !!!!!\n"   	
				print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n"   	

				cmd = "%s fwd_nl_cmd_file=%s fwd_cmd_file=%s adj_cmd_file=%s fwd_cmd_reg_file=%s adj_cmd_reg_file=%s data=%s init_model=%s inv_model=%s suffix=%s iteration_movies=obj,model,gradient,residual niter=%s log_file=%s debug=%s wrk_dir=%s epsilon=%s alpha=%s restart=%s minval=%s maxval=%s" %(nl_solver,nonlinear_parfile,lin_fwd_parfile,lin_adj_parfile,lin_fwd_dso_parfile,lin_adj_dso_parfile,data_file,initial_model,inverted_model,suffix,niter_outer,main_log_file,debug,main_wrk_dir,epsilon,alpha,restart,minval,maxval) 
				print "command to launch the inversion: %s" %(cmd)

			else:
				print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n"   				
				print "!!!!!!!!!!!!!!!!!!!! LAUNCHING THE INVERSION: !!!!!!!!!!!!!!!!!!!!!!!!\n"   	
				print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n"   				
				
				cmd = "%s fwd_nl_cmd_file=%s fwd_cmd_file=%s adj_cmd_file=%s fwd_cmd_reg_file=%s adj_cmd_reg_file=%s data=%s init_model=%s inv_model=%s suffix=%s iteration_movies=obj,model,gradient,residual niter=%s log_file=%s debug=%s wrk_dir=%s epsilon=%s alpha=%s restart=%s minval=%s maxval=%s" %(nl_solver,nonlinear_parfile,lin_fwd_parfile,lin_adj_parfile,lin_fwd_dso_parfile,lin_adj_dso_parfile,data_file,initial_model,inverted_model,suffix,niter_outer,main_log_file,debug,main_wrk_dir,epsilon,alpha,restart,minval,maxval) 
				print "command to launch the inversion: %s" %(cmd)
				cmd = cmd.replace("\/","/")		
				sep.RunShellCmd(cmd)			
	
		print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
		print "!!!!!!!!!!!!!!!!!! FINISHED VPTFWI INVERSION  !!!!!!!!!!!!!!!!!!!"  
		print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"	
	
	
	
		
