#!/usr/bin/python
import sep_python as sep
import operator_obj as op 
import sys,os as os_sys
import numpy as np 
from python_solver_outcore import write_log_file 
from timeit import default_timer as timer

##########################################################################################	
################################ MAIN ####################################################	
##########################################################################################	

if __name__ == '__main__':
##########################################################################################		
############################### TFWI MAIN  ###############################################	
##########################################################################################
	
	if (len(sys.argv) == 1):
        # write self documentation	
		print 'self doc not available (yet)' 

	else:

		##################################################################################	
		############################### READ PARAMETERS ##################################
		##################################################################################

		############################## BULLSHIT STUFF ####################################
		# GET SOLVER PATH
		python_path_list = os_sys.environ['PYTHONPATH'].split(":")
		indx = [i for i, s in enumerate(python_path_list) if '/python_solver' in s]
		pathsource = python_path_list[indx[0]]

		# GET SCRATCH PATH
		scratch_path = sep.sep_find_datapath()
		scratch_path = scratch_path.replace("/","\/")

		# GET CURRENT DIRECTORY
		cwd = os_sys.getcwd()+"/"

		# GET THE SOURCE CODE PATH
		code_path = os_sys.environ['CODE_PATH']

		############################# READ AND SETUP PARAMETERS ##########################
		# CREATE EXECUTABLE NAMES
		l_solver = pathsource+"/generic_linear_prob.py"	
		nl_modeling = code_path+"/Scripts/batch_task_executor_nonlinear.py"

		# READ NAME AND CREATE WORKING MAIN DIRECTORY
		# this directory will contain:
		# background data and background residuals for all different velocity models
		# subfolders where the extended linearized inversions are run
		main_wrk_dir = sep.from_cmd_line("main_wrk_dir")
		sep.RunShellCmd("mkdir -p %s"%(main_wrk_dir))

		# CREATE LOGIFLE FOR SAMPLING
		main_log_file = cwd+main_wrk_dir+"/main_log_file.txt"
		cmd = "rm -f %s; touch %s"%(main_log_file,main_log_file)
		sep.RunShellCmd(cmd)	

		# DEBUG MODE FOR THE MAIN SCRIPT
		debug_script = sep.from_cmd_line("debug_script","no")
		if (debug_script == "n" or debug_script == "no" or  debug_script == "0"):
			debug_script = False
		else:
			debug_script = True

		# SET DEBUG MODE FOR MAIN SCRIPT
		sep.debug = debug_script
		sep.debug_file = main_wrk_dir+"/debug_script.txt"

		write_log_file(main_log_file,"#####################################################################################")
		write_log_file(main_log_file,"#################### SAMPLING OBJECTIVE FUNCTION FOR VPTFWI AND FWI #################")
		write_log_file(main_log_file,"#####################################################################################\n")

		# READ PARFILE NAMES	
		# the user needs to provide template parameters files for the linearized fwd/adj (standard)
		# the parfile will be modified in order to change the background velocity value
		lin_fwd_parfile_template = sep.from_cmd_line("lin_fwd_parfile")
		lin_fwd_parfile_template = lin_fwd_parfile_template.replace("/","\/")				
		lin_adj_parfile_template = sep.from_cmd_line("lin_adj_parfile")
		lin_adj_parfile_template = lin_adj_parfile_template.replace("/","\/")	

		# READ AND CHECK FOR THE PRESENCE OF REGULARIZATION PARFILES
		reg_fwd_parfile = sep.from_cmd_line("reg_fwd_parfile",default=None)
		reg_adj_parfile = sep.from_cmd_line("reg_adj_parfile",default=None)
		reg_parfiles = False
		
		if (reg_fwd_parfile == None and reg_adj_parfile != None):
			os_sys.exit('error: reg_fwd_parfile not provided')
			
		if (reg_adj_parfile == None and reg_fwd_parfile != None):
			os_sys.exit('error: reg_adj_parfile not provided')

		if (reg_adj_parfile != None and reg_fwd_parfile != None):
			reg_parfiles = True

		# PROCESS PREFIX
		process_prefix = sep.from_cmd_line("prefix")

		# READ DATAFILE NAME
		data_file = sep.from_cmd_line("data")
		#data_file = data_file.replace("/","\/")		

		# READ NAME OF STARTING EXTENDED REFLECTIVITY (ZEROS EVERYWHERE) 
		reflectivity_init_file = sep.from_cmd_line("reflectivity_init")
		reflectivity_init_file = reflectivity_init_file.replace("/","\/")				

		# DEBUG MODE FOR THE LINEARIZED INVERSIONS
		debug = sep.from_cmd_line("debug")				

		# READ BACKGROUND MODEL NAME
		m0 = sep.from_cmd_line("m0")
		m0 = m0.replace("/","\/")	

		# READ NAME OF MAIN PARFILE
		main_parfile = sep.from_cmd_line("main_parfile")
		main_parfile = cwd+main_parfile	

		# READ ACQUISITION PARAMETERS
		os = sep.from_cmd_line("os") # first source position
		ds = sep.from_cmd_line("ds") # source spacing
		nshots = sep.from_cmd_line("nshots") # number of shots 
		nshotjob = sep.from_cmd_line("nshotjob") # number of shots per job 

		# READ PBS PARAMETERS
		pbs_template = sep.from_cmd_line("pbs_template")
		queues = sep.from_cmd_line("queues",want_list=False)		
		queues_cap = sep.from_cmd_line("queues_cap",want_list=False)		
		total_jobs_cap = sep.from_cmd_line("total_jobs_cap",want_list=False)	

		#READ NUMBER OF ITERATIONS FOR VARIABLE PROJECTION
		niter = sep.from_cmd_line("niter")

		###################### READ AND SET VELOCITY RANGE FOR MODELS ####################
		
		vel0 = sep.from_cmd_line("vel0",conv="float",want_list=True)

		# if user provides a list of vel0, ignore dvel and nvel
		if (isinstance(vel0,list)):
			vel_list = vel0
			nvel = len(vel_list)
			dvel = 1.0
			print "vel list",vel_list
		
		else:	
			dvel = sep.from_cmd_line("dvel",conv="float")
			nvel = sep.from_cmd_line("nvel",conv="int")
			velf = vel0 + nvel * dvel
			vel_list = np.asarray([float(x)*dvel+vel0  for x in range(nvel)])

		# WRITE LIST OF VELOCITY VALUES TESTED TO LOG FILE
		write_log_file(main_log_file,"list of velocity values tested [km/s]: %s" %(vel_list))

		##################################################################################		

		###################### READ AND SET EPSILON VALUES ###############################

		eps = sep.from_cmd_line("eps",conv="float",want_list=True)

		# if user provides a list (more than one epsilon value)
		if (isinstance(eps,list)):
			eps_list = eps
			neps = len(eps_list)
			deps = 1.0

		else:
			eps_list = [eps]
			neps = 1
			deps = 1.0
			
		# WRITE LIST OF EPSILON VALUES TESTED TO LOG FILE
		write_log_file(main_log_file,"list of epsilon values tested: %s" %(eps_list))

		##################################################################################		

		###################### READ AND SET NUMBER OF ITERATIONS #########################
		niter = sep.from_cmd_line("niter",conv="int",want_list=True)

		# if user provides a list (more than one iteration number value)
		if (isinstance(niter,list)):
			niter_list = niter
			
			# make sure the length of niter_list is the same as eps_list
			if ( len(niter_list) != len(eps_list) ):
				print "error: check number of iteration list"
				os_sys.exit()

		# if the user only provides one number of iteration
		else:
			niter_list = [int(niter)]

			# create a list of the same length as eps_list filled with the unique value
			# provided by the user
			
			for i_iter in range(len(eps_list)-1):
				niter_list.append(niter_list[0])

		# WRITE LIST OF NUMBER OF ITERATION VALUES TESTED FOR EACH EPSILON
		write_log_file(main_log_file,"list of number of iterations: %s\n" %(niter_list))	
			
		##################################################################################

		# BUILD PYTHON ARRAY WHERE WE STORE THE VALUES OF OBJECTIVE FUNCTION AS A FUNCTION OF EPSILON AND MODEL POINTS	 
		vel_array = np.asarray(vel_list,dtype=np.float32) # velocity values
		eps_array = np.asarray(eps_list,dtype=np.float32) # epsilon values
		niter_array = np.asarray(niter_list,dtype=np.float32) # values for the number of iteration
		data_objective_function_array = np.zeros([len(vel_list),len(eps_list)]) # data objective function
		model_objective_function_array = np.zeros([len(vel_list),len(eps_list)]) # model objective function
		total_objective_function_array = np.zeros([len(vel_list),len(eps_list)]) # total objective function
		fwi_objective_function_array = np.zeros([len(vel_list)]) # FWI objective function
		
		# AXIS INFORMATION FOR SEP HEADER FILES
		header_vptfwi = [[nvel,vel_array[0],dvel,'velocity [km/s]'],[neps,eps_array[0],deps,'epsilon']]
		header_fwi = [[nvel,vel_array[0],dvel,'velocity [km/s]']]

		############################## START SAMPLING PROCESS ############################
		
		write_log_file(main_log_file,"############ STARTING TO SAMPLE THE OBJECTIVE FUNCTIONS (FWI AND VPTFWI) ##########\n")		

		# LOOP OVER MODEL POINTS
		for ivel in range(nvel):

			# UPDATE THE VELOCITY VALUE AND WRITE TO LOGFILE
			vel = vel_array[ivel]
	
			write_log_file(main_log_file,"############ vel %s: %s [km/s]" %(ivel,vel))			

			# CREATE NAME OF CURRENT VELOCITY MODEL WHERE WE WANT TO TEST THE OBJECTIVE FUNCTION
			model_nb = str(ivel)
			model = cwd+main_wrk_dir+"/"+process_prefix+"_model_"+model_nb+".H"
			model = model.replace("/","\/")

			# CREATE 2 NEW PARAMATER FILES NAMES FOR THE LINEAR INVERSIONS FOR THE SPECIFIC VELOCITY VALUES
			lin_fwd_parfile = "Par/"+process_prefix+"sampling_lin_fwd_model"+model_nb+".p"
			lin_fwd_parfile = lin_fwd_parfile.replace("/","\/")
			lin_adj_parfile = "Par/"+process_prefix+"sampling_lin_adj_model"+model_nb+".p"			
			lin_adj_parfile = lin_adj_parfile.replace("/","\/")

			# MAKE SURE THE FILES ARE DELETED BEFORE APPENDING THEM
			cmd = "rm -f %s" %(lin_fwd_parfile)
			sep.RunShellCmd(cmd)							
			cmd = "rm -f %s" %(lin_adj_parfile)
			sep.RunShellCmd(cmd)	

			# CREATE THE PARFILES FOR THE LINEARIZED INVERSION
			# UPDATE THE VELOCITY MODEL FILE NAME IN BOTH PARAMETER FILES
			cmd = "sed 's/vel=/vel=%s/g' < %s > %s" %(model,lin_fwd_parfile_template,lin_fwd_parfile)
			sep.RunShellCmd(cmd)					
			cmd = "sed 's/vel=/vel=%s/g' < %s > %s" %(model,lin_adj_parfile_template,lin_adj_parfile)
			sep.RunShellCmd(cmd)					

			# CREATE THE HEADER AND BINARY FOR THE CURRENT VELOCITY MODEL
			cmd = "Math file1=%s exp='file1*0.0+%s' > %s" %(m0,vel,model)
			cmd = cmd.replace("\/","/")
			sep.RunShellCmd(cmd)	

			# CREATE THE BACKGROUND DATA FILE NAME
			background_data_file = cwd+main_wrk_dir+"/"+process_prefix+"_dat0_sampling_model"+model_nb+".H"
			
			# CREATE THE DATA RESIDUAL FILE NAME
			data_residual_file = cwd+main_wrk_dir+"/"+process_prefix+"_dat_sampling_diff0_model"+model_nb+".H"

			# GENERATE BACKGROUND DATA
			cmd1 = "rm -f wrk/*sampletempdata*;" 
			cmd2 = "%s pbs_template=%s vel=%s par=%s queues=%s os=%s nshots=%s nshotjob=%s ds=%s queues_cap=%s path_out=wrk prefix=sampletempdata total_jobs_cap=%s;" %(nl_modeling,pbs_template,model,main_parfile,queues,os,nshots,nshotjob,ds,queues_cap,total_jobs_cap)
			cmd2 = cmd2.replace("\/","/")
			cmd3 = "Mv sampletempdata-final.H %s;" %(background_data_file)
			cmd4 = "rm -f wrk/*sampletempdata*;"
			cmd = cmd1+cmd2+cmd3+cmd4
			sep.RunShellCmd(cmd,True,True) # first one prints the command and the second one prints the output
		
			# COMPUTE DATA RESIDUALS
			cmd = "Add %s %s scale=1,-1 > %s" %(data_file,background_data_file,data_residual_file)
			sep.RunShellCmd(cmd)

			# GETTING FWI OBJECTIVE FUNCTION VALUES
			cmd = "Attr < %s param=1 want=norm | colrm 1 5" %(data_residual_file) # extract the norm of the data residual file
			fwi_objective_function_array[ivel] = float(sep.RunShellCmd_output(cmd)) # run the command and convert the result to float
			fwi_objective_function_array[ivel] = 0.5*fwi_objective_function_array[ivel]*fwi_objective_function_array[ivel] # square the norm and multiply by 0.5

			# WRITE FWI OBJECTIVE FUNCTION VALUE FOR CURRENT VELOCITY MODEL 
			write_log_file(main_log_file,"FWI objective function value: %s" %(fwi_objective_function_array[ivel]))			

			# DEFINE NAME OF HEADER WHERE WE ARE GOING TO WRITE THE VALUE OF THE FWI OBJECTIVE FUNCTION
			fwi_obj_function_file = cwd+main_wrk_dir+"/main_fwi_objective_function"+process_prefix+".H"
			sep.sep_write_file(fwi_obj_function_file,header_fwi,fwi_objective_function_array)

			# DEFINE THE NAME OF VPTFWI OBJECTIVE FUNCTIONS
			data_obj_final_file = cwd+main_wrk_dir+"/main_data_objective_function"+process_prefix+".H"
			model_obj_final_file = cwd+main_wrk_dir+"/main_model_objective_function"+process_prefix+".H"
			total_obj_final_file = cwd+main_wrk_dir+"/main_total_objective_function"+process_prefix+".H"			

			# LOOP OVER EPSILON VALUES
			for ieps in range(neps):

				# UPDATE EPSILON VALUE
				eps = eps_array[ieps]
				eps_nb = str(ieps)
				write_log_file(main_log_file,"epsilon %s: %s" %(ieps,eps))	
				
				# UPDATE TEH NUMBER OF ITERATIONS FOR THIS EPSILON
				niter = int(niter_array[ieps])
				niter = str(niter)
				write_log_file(main_log_file,"number of linear iterations: %s" %(niter))

				# CREATE THE NAME FOR THE SUB WORK DIRECTORY
				sub_wrk_dir = cwd+main_wrk_dir+"/LinInv_Model"+model_nb+"_Eps"+eps_nb

				# CREATE THE NAME OF THE INVERTED MODEL
				inverted_model = process_prefix+"_inverted_reflectivity_Mod"+model_nb+"_Eps"+eps_nb+".H"

				# CREATE THE SUFFIX 
				suffix = "lin_inv_Mod"+model_nb+"_Eps"+eps_nb	

				# CREATE LOG FILE NAME FOR THE LINEARIZED INVERSION
				log_file = "log_file"			

				# LAUNCH THE VP INVERSION
				if (reg_parfiles):

					# case where user provides regularization parfiles
 					write_log_file(main_log_file,"launching the inversion")
 					write_log_file(main_log_file,"regularization parfiles provided") 					
					cmd = "%s fwd_cmd_file=%s adj_cmd_file=%s fwd_cmd_reg_file=%s adj_cmd_reg_file=%s data=%s init_model=%s inv_model=%s suffix=%s iteration_movies=obj,model,gradient,residual niter=%s log_file=%s debug=%s wrk_dir=%s epsilon=%s" %(l_solver,lin_fwd_parfile,lin_adj_parfile,reg_fwd_parfile,reg_adj_parfile,data_residual_file,reflectivity_init_file,inverted_model,suffix,niter,log_file,debug,sub_wrk_dir,eps) 
					cmd = cmd.replace("\/","/")		
					print 'cmd to launch the inversion:',cmd
					sep.RunShellCmd(cmd)					
 					write_log_file(main_log_file,"finished the inversion")
 				
 				else:
 
 					# case where user provides regularization parfiles
 					write_log_file(main_log_file,"launching the inversion")
 					write_log_file(main_log_file,"regularization parfiles NOT provided, using default")
					cmd = "%s fwd_cmd_file=%s adj_cmd_file=%s data=%s init_model=%s inv_model=%s suffix=%s iteration_movies=obj,model,gradient,residual niter=%s log_file=%s debug=%s wrk_dir=%s epsilon=%s" %(l_solver,lin_fwd_parfile,lin_adj_parfile,data_residual_file,reflectivity_init_file,inverted_model,suffix,niter,log_file,debug,sub_wrk_dir,eps) 
					cmd = cmd.replace("\/","/")		
					print 'cmd to launch the inversion:',cmd
					sep.RunShellCmd(cmd)					
 					write_log_file(main_log_file,"finished the inversion")				

				# DEFINE THE NAME OF THE OBJECTIVE FUNCTIONS FILES THAT WILL AUTOMATICALLY BE CREATED FROM THE INVERSION
				data_obj_file = sub_wrk_dir+'/obj'+suffix+"1.H"
				model_obj_file = sub_wrk_dir+'/obj'+suffix+"2.H"
				total_obj_file = sub_wrk_dir+'/obj'+suffix+".H"

				# GET THE NUMBER OF ITERATIONS AND THE F INDEX FOR WINDOW3D (THE SAME FOR ALL OBJECTIVE FUNCTIONS)
				cmd = "Get < %s n1 parform=n" %(total_obj_file)
				f_obj = int(sep.RunShellCmd_output(cmd))-1

				# STORE OBJECTIVE FUNCTION VALUES
				
				# case where epsilon = 0 => only the total objective function is creates
				# in that case, set the model objective function to zero
				# set the data objective function to the total objective function
				cmd = "Window3d < %s n1=1 f1=%s | Attr want=max param=1 | colrm 1 7" %(total_obj_file,f_obj)			
				total_objective_function_array[ivel,ieps] = float(sep.RunShellCmd_output(cmd))

				if (eps == 0):
					data_objective_function_array[ivel,ieps] = total_objective_function_array[ivel,ieps]
					model_objective_function_array[ivel,ieps] = 0.0
				
				else:

					# data
					cmd = "Window3d < %s n1=1 f1=%s | Attr want=max param=1 | colrm 1 7" %(data_obj_file,f_obj)			
					data_objective_function_array[ivel,ieps] = float(sep.RunShellCmd_output(cmd))

					# model
					cmd = "Window3d < %s n1=1 f1=%s | Attr want=max param=1 | colrm 1 7" %(model_obj_file,f_obj)			
					model_objective_function_array[ivel,ieps] = float(sep.RunShellCmd_output(cmd))

				# WRITING RESULTS TO SEP FILES
				sep.sep_write_file(data_obj_final_file,header_vptfwi,data_objective_function_array)
				sep.sep_write_file(model_obj_final_file,header_vptfwi,model_objective_function_array)
				sep.sep_write_file(total_obj_final_file,header_vptfwi,total_objective_function_array)

				# WRITE FWI OBJECTIVE FUNCTION VALUE FOR CURRENT VELOCITY MODEL 
				write_log_file(main_log_file,"VPTFWI total objective function value: %s" %(total_objective_function_array[ivel,ieps]))
				write_log_file(main_log_file,"VPTFWI data objective function value: %s" %(data_objective_function_array[ivel,ieps]))				
				write_log_file(main_log_file,"VPTFWI model objective function value: %s" %(model_objective_function_array[ivel,ieps]))	
				write_log_file(main_log_file,"finished computing VPTFWI objective function for vel=%s, eps=%s \n" %(vel,eps))


		write_log_file(main_log_file,"#####################################################################################")
		write_log_file(main_log_file,"######################## FINISHED SAMPLING OBJECTIVE FUNCTION #######################")
		write_log_file(main_log_file,"#####################################################################################\n")



