program dso_inv_main_var_proj

	use sep
	use dso_inv

  	implicit none

  	integer              					  		:: nz2,nx2,n3,niter,iter
  	integer									  		:: hzmax,hxmax,htmax,nhz,nhx,nht
  	integer*8										:: nbytes
  	real									  		:: dot1,dot2,error,alpha
  	real									  		:: dz,dx,dhz,dhx,dht
  	real									  		:: oz,ox,ohz,ohx,oht  	
	real,dimension(:,:,:,:,:),allocatable         	:: reflectivity_in,reflectivity_out
	logical									  		:: adj
	integer											:: sreed,srite,check

  	call sep_init()
  	
	call from_history("n1",nz2)
	call from_history("d1",dz)	
	call from_history("o1",oz)	

	call from_history("n2",nx2)
	call from_history("d2",dx)
	call from_history("o2",ox)

	call from_param("hzmax",hzmax)
	call from_param("hxmax",hxmax)
	call from_param("htmax",htmax)	
	call from_param("axis",niter,2) ! if the number of iteration is one -> you need to specify it

	if (niter .gt. 1) then
		write(0,*) "number of iteration is greater than 1"
	end if
	
	nhz = 2 * hzmax + 1 
	nhx = 2 * hxmax + 1 
	nht = 2 * htmax + 1 	

	if (hzmax .gt. 0) then

		if (niter .gt. 1) then
			call from_history("n4",niter)
		end if
		
		call to_history("n3",nhz)
		call to_history("d3",dz)	
		call to_history("o3",dz*(-hzmax))		
		call to_history("n4",niter)

	end if

	if (hxmax .gt. 0) then
		
		if (niter .gt. 1) then		
			call from_history("n5",niter)
		end if

		call to_history("n3",nhz)
		call to_history("d3",dz)	
		call to_history("o3",dz*(-hzmax))		
		call to_history("n4",nhx)
		call to_history("d4",dx)	
		call to_history("o4",dx*(-hxmax))				
		call to_history("n5",niter)
		
	end if

	if (htmax .gt. 0) then 

		if (niter .gt. 1) then		
			call from_history("n6",niter)
		end if

		call to_history("n3",nhz)
		call to_history("d3",dz)	
		call to_history("o3",dz*(-hzmax))		
		call to_history("n4",nhx)
		call to_history("d4",dx)	
		call to_history("o4",dx*(-hxmax))				
		call to_history("n5",nht)
		call to_history("n6",niter)	
	
	end if
	
	nbytes = 4 * nz2 * nx2 * nhz * nhx * nht
	
	call from_param("adj",adj)
	call from_param("alpha",alpha)
	write(0,*) "alpha=",alpha		
	
	allocate(reflectivity_in(nz2,nx2,-hzmax:hzmax,-hxmax:hxmax,-htmax:htmax))
	allocate(reflectivity_out(nz2,nx2,-hzmax:hzmax,-hxmax:hxmax,-htmax:htmax))	
	
	! initialize operator
	call dso_inv_init(nz2,nx2,hzmax,hxmax,htmax,alpha)
	
	! apply inverse DSO operator
	if (.not. adj) then

		do iter = 1,niter
			! read the reflectivity
			check = sreed("in",reflectivity_in,nbytes)

			! apply DSO forward to reflectivity_in
			call dso_inv_op2(.false.,.false.,reflectivity_in,reflectivity_out) 
			check = srite("out",reflectivity_out,nbytes)	
		end do
				
	else

		do iter = 1,niter
			! read the reflectivity
			check = sreed("in",reflectivity_in,nbytes)

			! apply DSO adjoint to reflectivity_in
			call dso_inv_op2(.true.,.false.,reflectivity_out,reflectivity_in) 
			check = srite("out",reflectivity_out,nbytes)	
		end do

	end if


	call to_history("n1",nz2)
	call to_history("d1",dz)	
	call to_history("o1",oz)	

	call to_history("n2",nx2)
	call to_history("d2",dx)
	call to_history("o2",ox)



	call sep_close()					
	
end program