module im_cond_wemva

	implicit none

	integer,private                                     :: nx,nz,nt,hxmax,hzmax,htmax
	real*8,dimension(:,:,:),allocatable,private     	:: recv_wfld

	contains

	subroutine im_cond_wemva_init(nz_in,nx_in,hzmax_in,hxmax_in,htmax_in,nt_in,recv_wfld_in)
		integer                           	  :: nx_in,nz_in,nt_in,hxmax_in,hzmax_in,htmax_in
		real*8                                :: dts_in
		real*8,dimension(:,:,:)           	  :: recv_wfld_in

		nx  = nx_in
		nz  = nz_in
		nt   = nt_in
		hxmax = hxmax_in
		hzmax = hzmax_in		
		htmax = htmax_in

		if (.not. allocated(recv_wfld)) allocate(recv_wfld(nz,nx,nt))
		recv_wfld = recv_wfld_in

	end subroutine

	function im_cond_wemva_op(adj,add,model,data) result(stat)
		logical,intent(in)   	:: adj, add
		real*8,dimension(:)     :: model,data
		integer              	:: stat
		call im_cond_wemva_op2(adj,add,model,data)
		stat=0
	end function

	subroutine im_cond_wemva_op2(adj,add,model,data)
		logical                                    							 :: adj,add
		real*8,dimension(nz,nx,nt)              							 :: data
		real*8,dimension(nz,nx,-hzmax:hzmax,-hxmax:hxmax,-htmax:htmax)		 :: model											
		integer                                    							 :: it,iz,ix,ihz,ihx,iht,ihtmin,ihtmax
		integer																 :: ixx1,ixx2,ixx
		integer																 :: izz1,izz2,izz		
		integer																 :: ihxmin,ihxmax,ihzmin,ihzmax
		
		if ((adj .eqv. .true.) .and. (add .eqv. .false.)) model = 0.
		if ((adj .eqv. .false.) .and. (add .eqv. .false.)) data = 0.
		
		if (adj .eqv. .true.) then
			
			! ADJOINT: 
			! in: wavefiled
			! out: reflectivity
			do it = 1,nt
	

				ihtmax = (nt-it) / 2
				ihtmax = min(htmax,ihtmax)
				ihtmin = (1-it) / 2
				ihtmin = max(-htmax,ihtmin)		


				do iht = ihtmin,ihtmax	

					do ix = 1,nx

						ixx1 = ceiling((1.0+ix) / 2.0)
						ixx1 = max(ix-hxmax,ixx1)
						ixx2 = (nx+ix) / 2
						ixx2 = min((hxmax+ix),ixx2)	

						do ixx = ixx1,ixx2

							ihx = ixx - ix

							do iz = 1,nz
								
								izz1 = ceiling((1.0+iz) / 2.0)
								izz1 = max(iz-hzmax,izz1)
								izz2 = (nz+iz) / 2
								izz2 = min((hzmax+iz),izz2)	
								
								do izz = izz1,izz2

									ihz = izz - iz			
	
									model(izz,ixx,ihz,ihx,iht) = model(izz,ixx,ihz,ihx,iht) - data(iz,ix,it) * recv_wfld(iz+2*ihz,ix+2*ihx,it+2*iht)										
														
								end do
							end do
						end do
					end do
				end do
			end do		
		
		else
			
			! FORWARD: 
			! in: reflectivity
			! out: wavefield
			do it = 1,nt
	
				ihtmax = (nt-it) / 2
				ihtmax = min(htmax,ihtmax)
				ihtmin = (1-it) / 2
				ihtmin = max(-htmax,ihtmin)		


				do iht = ihtmin,ihtmax	

					do ix = 1,nx

						ixx1 = ceiling((1.0+ix) / 2.0)
						ixx1 = max(ix-hxmax,ixx1)
						ixx2 = (nx+ix) / 2
						ixx2 = min((hxmax+ix),ixx2)	

						do ixx = ixx1,ixx2

							ihx = ixx - ix

							do iz = 1,nz
								
								izz1 = ceiling((1.0+iz) / 2.0)
								izz1 = max(iz-hzmax,izz1)
								izz2 = (nz+iz) / 2
								izz2 = min((hzmax+iz),izz2)	
								
								do izz = izz1,izz2

									ihz = izz - iz
									data(iz,ix,it) = data(iz,ix,it) - model(izz,ixx,ihz,ihx,iht) * recv_wfld(iz+2*ihz,ix+2*ihx,it+2*iht)
					
								end do
							end do
						end do
					end do

				end do
	
			end do		
		
		end if		

	end subroutine

end module		
		
		