program imaging_main

	use sep

	implicit none	
	
	integer									:: iz,ix,its,ihz,ihx,itau,left_lag,right_lag,lag
	integer									:: nz2,nx2,nt,offset
	integer									:: extension
	real									:: oz,dz,ox,dx,ots,dts
	real,dimension(:,:,:),allocatable		:: swfld,rwfld
	real,dimension(:,:,:,:,:),allocatable	:: image
		
	call sep_init(SOURCE)	

	! READ PARAMETERS 
	call from_history("n1",nz2)
 	call from_history("d1",dz)	
 	call from_history("o1",oz)
 	
 	call from_history("n2",nx2)
 	call from_history("d2",dx)	
 	call from_history("o2",ox) 

 	call from_history("n3",nts)
 	call from_history("d3",dts)	
 	call from_history("o3",ots)
 	
	! TYPE OF EXTENSION 
	! 0 = hz
	! 1 = hx
	! 2 = tau		
	call from_param("extension",extension)		
	call from_param("offset",offset)			
 	
	! ALLOCATE
	allocate(swfld(nz2,nx2,nts))
	allocate(rwfld(nz2,nx2,nts))	
	allocate(image(nz2,nx2)
	allocate(cumu_image(nz2,nx2,nts))
	allocate(instant_image(nz2,nx2,nts))
	
	! INITIALIZE IMAGES
	image = 0.0
	cumu_image = 0.0		
	instant_image = 0.0
	
	! READ SOURCE AND RECEIVER WAVEFIELDS
  	call sep_read(swfld)
   	call sep_read(rwfld,"rwfld")
   	
   	! START COMPUTING IMAGES
   	select case (extension)
   	
   		case (1) ! hz extension
		
			do its = 1,nts
				!$OMP PARALLEL DO private(ix,iz)				
				do ix = 1,nx2
					do iz = 1,nz2
						image(iz,ix) = image(iz,ix) - swfld(iz-offset,ix,its)*rwfld(iz+offset,ix,its)  
						cumu_image(iz,ix,its) = cumu_image(iz,ix,its) - swfld(iz-offset,ix,its)*rwfld(iz+offset,ix,its)  
						instant_image(iz,ix,its) = instant_image(iz,ix,its) - swfld(iz-offset,ix,its)*rwfld(iz+offset,ix,its)  						
					end do
				end do
          		!$OMP END PARALLEL DO
			end do												
			


	end select	

  	call sep_close()

end program


	