! THIS CODE MUTES: 
!	(1) direct arrivals + earlier energy on shot gather with a taper in the time direction
!	(2) large offset energy for each shot gather
program mute_shot

	use sep

  	implicit none
  	
  	integer     							:: nt,nr,ns,it,ir,is
	real									:: dt,dr,ds,ot,or,os
	real									:: vel_mute,argument,exp
  	integer									:: it_cutoff,buffer_time,it_min,it_max
	integer									:: ir_min_cutoff,ir_max_cutoff,ir_min_cutoff_taper,ir_max_cutoff_taper
	real									:: r_min,r_max,max_offset,r_min_mute,r_max_mute
	real									:: s,r,dsr,t,t0
	real									:: buffer_offset
	real,dimension(:,:,:),allocatable 		:: model,data,datac
	real,dimension(:,:,:),allocatable 		:: time_taper,offset_taper
	logical									:: time_mute,offset_mute,inv
  	
	call sep_init  	
  	
  	! import dimensions
  	call from_history("n1",nt)
  	call from_history("n2",nr)
  	call from_history("n3",ns)
  	call from_history("o1",ot)
  	call from_history("o2",or)
  	call from_history("o3",os) 
  	call from_history("d1",dt)
  	call from_history("d2",dr)
  	call from_history("d3",ds)  	 	

	! decide which muting/tapering to apply
	! 1. time_mute = mutes the data along the time axis (trace by trace)
	call from_param("time_mute",time_mute)

	! 2. offset_mute = mutes the data the offset axis
	call from_param("offset_mute",offset_mute)	

	! 3. inv = output model is the complement of the muted data
	call from_param("inv",inv,.false.)
	
	! allocate model/data	
	allocate(model(nt,nr,ns))
	allocate(data(nt,nr,ns))

	! read shot gather(s)
	call sep_read(model)
	data = model

	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! TIME MUTE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	if (time_mute) then

		call from_param("vel_mute",vel_mute)
		call from_param("t0",t0)
		call from_param("buffer_time",buffer_time) ! number of time samples to be tapered 
		call from_param("exp",exp)

		! allocate and initialize the time_taper
		allocate(time_taper(nt,nr,ns))	
		time_taper = 1.0
	
		! loop over shot
		!$OMP PARALLEL DO private(is,s,ir,r,dsr,t,it,it_cutoff,it_min,argument) 
		do is = 1,ns
			
			! compute shot position [km]
			s = os + (is-1) * ds

			! loop over receiver
			do ir = 1,nr
			
				! compute receiver position			
				r = or + (ir-1) * dr 

				! compute absolute value of offset
				dsr = abs(r-s) 

				! compute arrive time 
				t = dsr / vel_mute + t0 

				! convert time to sample number (nearest neighbor)
				it_cutoff = (t-ot) / dt + 1 

				! compute lower time bound [sample]
				it_min = it_cutoff-buffer_time 

				! set all energy for early arrival times to zero
				do it = 1,it_min
					time_taper(it,ir,is) = 0.0
					data(it,ir,is) = data(it,ir,is) * time_taper(it,ir,is)
				end do
				
				! apply taper in the intermediate zone
				do it = it_min,it_cutoff
					argument = 1.0 * (it - it_min) / (it_cutoff-it_min)
					time_taper(it,ir,is) = sin(3.14159/2.0*(argument))**exp			
					data(it,ir,is) = data(it,ir,is) * time_taper(it,ir,is)
			
				end do
									
			end do
		end do	
		!$OMP END PARALLEL DO 
	end if	
	
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! OFFSET MUTE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!	
	if (offset_mute) then

		! compute minimum receiver position
		r_min = or 
		r_max = or + (nr-1) * dr

		call from_param("max_offset",max_offset) ! max offset to keep [km] after that, start tapering down
		call from_param("buffer_offset",buffer_offset) ! taper width [samples]	
		call from_param("exp",exp)
		
		! make sure the offset value is positive
		max_offset = abs(max_offset)

		! allocate and initialize the offset_taper
		allocate(offset_taper(nt,nr,ns))	
		offset_taper = 1.0
	
		! loop over shot
		!$OMP PARALLEL DO private(is,s,r_min_mute,r_max_mute,ir_min_cutoff,ir_max_cutoff,ir_min_cutoff_taper,ir_max_cutoff_taper,argument)
		do is = 1,ns

			! compute shot position [km]
			s = os + (is-1) * ds

			! compute cutoff position of the receivers from each side of the source 
			r_max_mute = min(s + max_offset,r_max)
			r_min_mute = max(s - max_offset,r_min)

			! convert receiver position into sample number
			ir_min_cutoff = (r_min_mute-or) / dr + 1
			ir_max_cutoff = (r_max_mute-or) / dr + 1

			! compute bounds of receivers (after buffer)
			ir_min_cutoff_taper = ir_min_cutoff - buffer_offset
			ir_max_cutoff_taper = ir_max_cutoff + buffer_offset

			do ir = 1,nr
			
				if ((ir .le. ir_min_cutoff_taper) .or. (ir .ge. ir_max_cutoff_taper)) then
					
					offset_taper(:,ir,is) = 0.0
				
				else if (ir .le. ir_min_cutoff) then
				
					argument = 1.0 * (ir - ir_min_cutoff_taper) / (ir_min_cutoff-ir_min_cutoff_taper)
					offset_taper(:,ir,is) = sin(3.14159/2.0*(argument))**exp			

				else if (ir .ge. ir_max_cutoff) then
					argument = 1.0 * (ir - ir_max_cutoff) / (ir_max_cutoff_taper-ir_max_cutoff)
					offset_taper(:,ir,is) = cos(3.14159/2.0*(argument))**exp				
				end if
				
			end do

			! apply offset taper to shot gather
			data(:,:,is) = data(:,:,is) * offset_taper(:,:,is)

		end do
		!$OMP END PARALLEL DO 	
		
	end if
	
	if (inv) data = model - data
	
	call sep_write(data)

  	call to_history("n1",nt)
  	call to_history("n2",nr)
  	call to_history("n3",ns)

  	call to_history("o1",ot)
  	call to_history("o2",or)
  	call to_history("o3",os) 

  	call to_history("d1",dt)
  	call to_history("d2",dr)
  	call to_history("d3",ds)				

	if (time_mute) then
		
		call sep_write(time_taper,"time_taper")	
		
		call to_history("n1",nt,"time_taper")	
		call to_history("n2",nr,"time_taper")	
		call to_history("n3",ns,"time_taper")	

		call to_history("o1",ot,"time_taper")	
		call to_history("o2",or,"time_taper")	
		call to_history("o3",os,"time_taper")	

		call to_history("d1",dt,"time_taper")	
		call to_history("d2",dr,"time_taper")	
		call to_history("d3",ds,"time_taper")	  	
		
	end if
	
	if (offset_mute) then
		
		call sep_write(offset_taper,"offset_taper")	
		
		call to_history("n1",nt,"offset_taper")	
		call to_history("n2",nr,"offset_taper")	
		call to_history("n3",ns,"offset_taper")	

		call to_history("o1",ot,"offset_taper")	
		call to_history("o2",or,"offset_taper")	
		call to_history("o3",os,"offset_taper")	

		call to_history("d1",dt,"offset_taper")	
		call to_history("d2",dr,"offset_taper")	
		call to_history("d3",ds,"offset_taper")	  	
		
	end if	

	call sep_close
  	
end program
