program radial_kfilt

	use sep
    use fourier
	
	implicit none	
	
  	integer									  :: nz,nx,nz2,nx2,n3,n_left,n_ext,ne_index
  	integer									  :: iz,ix,fft_pad
	integer									  :: ikz,ikx,ntaper,index_okx,index_okz
	real								      :: oz,ox,dz,dx
	real									  :: dkz,dkx,okz,okx,dk_taper
	real									  :: k,kz,kx,k_taper,k_cut,k_big,k_small
	real									  :: sign1,sign2
  	real,dimension(:,:),allocatable  		  :: model,filter,filterc,model_low,model_high
  	complex,dimension(:,:),allocatable		  :: model_fft,model_fft_filter,model_fft_filterc,cmodel_low,cmodel_high
	logical									  :: debug
	integer									  :: rc,sreed,sk,sseek_block
		
	call sep_init(SOURCE)

	! READ FROM HISTORY
	call from_history("n1",nz)
 	call from_history("d1",dz)	
 	call from_history("o1",oz)
 	
 	call from_history("n2",nx)
 	call from_history("d2",dx)	
 	call from_history("o2",ox) 
 	
	call from_param("fft_pad",fft_pad,500)
	call from_param("debug",debug,.false.)

	nz2 = nz + 2 * fft_pad
	nx2 = nx + 2 * fft_pad	

	! ALLOCATE
	allocate(model(nz2,nx2))
	allocate(model_fft(nz2,nx2))
	allocate(filter(nz2,nx2))
	allocate(filterc(nz2,nx2))			
	allocate(model_fft_filter(nz2,nx2))
	allocate(model_fft_filterc(nz2,nx2))	
	allocate(cmodel_low(nz2,nx2))
	allocate(cmodel_high(nz2,nx2))
	allocate(model_low(nz2,nx2))	
	allocate(model_high(nz2,nx2))		
				
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! READ MODEL !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	! READ MODEL
  	rc = sreed("in",model(fft_pad+1:fft_pad+nz,fft_pad+1:fft_pad+nx),4*nz*nx)

	! PAD THE MODEL
    do ix = 1,nx
        do iz = 1,fft_pad
            model(iz,ix+fft_pad)      		 = model(fft_pad+1,ix+fft_pad)
            model(fft_pad+nz+iz,ix+fft_pad)  = model(fft_pad+nz,ix+fft_pad)
        end do
    end do

    do ix = 1,fft_pad
	    do iz = 1,nz2
            model(iz,ix)             = model(iz,fft_pad+1)
            model(iz,fft_pad+nx+ix)  = model(iz,fft_pad+nx)
        end do
    end do

	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! INITIALIZE FFT !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

 	! FFT PARAMETERS FOR Z
	sign1 = 1.0 
  	dkz = 1./((nz2-1)*dz)
	n_left = (nz2-(nz2+1)/2)
	okz = -1.0 * n_left * dkz 	
	write(0,*) "dkz",dkz,"[1/km]"
	
 	! FFT PARAMETERS FOR X
	sign2 = 1.0 
  	dkx = 1./((nx2-1)*dx)
	n_left = (nx2-(nx2+1)/2)
	okx = -1.0 * n_left * dkx 	
	write(0,*) "dkx",dkx,"[1/km]"	
 	
	! APPLY FFT TO MODEL 
	model_fft = model
 	call ft_init(nz2,nx2)
 	call ft1axis(.false.,sign1,model_fft)
 	call ft2axis(.false.,sign2,model_fft) 

	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! INITIALIZE FILTER !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 	
 	
	! WAVENUMBER CUTOFF
	! every point in the kz/kx domain that has a |k| > k_cutoff should be multiplied by 0
	call from_param("k_cut",k_cut)
	call from_param("ntaper",ntaper) 

	! COMPUTE THE WIDTH (IN WAVENUMBER UNITS) OF THE TAPER ZONE
	dk_taper = ntaper * max(dkz,dkx)

	write(0,*) "cutoff wavenumber:",k_cut,"[1/km]"
	write(0,*) "taper length:",ntaper,"[samples]"	
	write(0,*) "taper length:",dk_taper,"[1/km]"	

	! CUTOFFS
	k_big = k_cut
	k_small = k_cut - dk_taper

	! CHECK
	if (dk_taper .ge. k_cut) then 
		write(0,*)"WARNING: dk_taper greater than k_cut !!!"
		write(0,*)"WARNING: will terminate program !!!"		
		stop
	end if

	! INITIALIZE FILTER TO ZERO
	filter = 0.0

	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!! COMPUTE FFT LOW-PASS FILTER !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	! INDEX FOR DC COMPONENT
	index_okz = nz2-(nz2+1)/2+1
	index_okx = nx2-(nx2+1)/2+1	

	! COMPUTE LOW-PASS FILTER
	do ikz = 1,nz2

 		kz = (ikz - index_okz) * dkz			
		do ikx = 1,nx2

 			kx = (ikx - index_okx) * dkx
 			k = sqrt(kz*kz+kx*kx) 

			! compute simple radial filter
			if (k .lt. k_small) then  
 				filter(ikz,ikx) = 1.0 
 			else if ((k .ge.  k_small) .and. (k .lt.  k_big)) then
 				filter(ikz,ikx) = cos(3.14159265359/2.0 * (k-k_small)/(k_big-k_small))**2
 			end if

		end do

  	end do 	
	
	! COMPUTE COMPLEMENT OF FILTER ONLY FOR DEBUG PURPOSES 
	filterc = 1.0 - filter
	
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! APPLY FFT FILTERS !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	model_fft_filter = model_fft * filter
	model_fft_filterc = model_fft * filterc	

	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! APPLY IFFT !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!	

	! APPLY IFFT
 	cmodel_low = model_fft_filter
 	cmodel_high = model_fft_filterc

 	call ft1axis(.true.,sign1,cmodel_low)
 	call ft2axis(.true.,sign2,cmodel_low) 	
	model_low = real(cmodel_low)

 	call ft1axis(.true.,sign1,cmodel_high)
 	call ft2axis(.true.,sign2,cmodel_high) 	
	model_high = real(cmodel_high)

	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! WRITE OUTPUT !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	! WRITE LOW-PASSED MODEL
	call sep_write(model_low(fft_pad+1:fft_pad+nz,fft_pad+1:fft_pad+nx))

 	call to_history("n1",nz)
 	call to_history("d1",dz)
 	call to_history("o1",oz)
 	call to_history("label1","z [km]")

 	call to_history("n2",nx)
 	call to_history("d2",dx)
 	call to_history("o2",ox)
 	call to_history("label2","x [km]") 	
 
 	call to_history("n3",1)
 	call to_history("d3",0.0)
 	call to_history("o3",0.0)
 	
	! WRITE HIGH-PASSED MODEL
	call sep_write(model_high(fft_pad+1:fft_pad+nz,fft_pad+1:fft_pad+nx),"high")

 	call to_history("n1",nz,"high")
 	call to_history("d1",dz,"high")
 	call to_history("o1",oz,"high")
 	call to_history("label1","z [km]","high")

 	call to_history("n2",nx,"high")
 	call to_history("d2",dx,"high")
 	call to_history("o2",ox,"high")
 	call to_history("label2","x [km]","high") 	
 
 	call to_history("n3",1,"high")
 	call to_history("d3",0.0,"high")
 	call to_history("o3",0.0,"high")
 	 
	if (debug) then
	
		write(0,*) "saving intermediate steps"
	
		!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! DEBUG !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 	 

		! WRITE MODEL FFT BEFORE FILTERING
		call sep_write(cabs(model_fft),"model_fft")
	
		call to_history("n1",nz2,"model_fft")
		call to_history("d1",dkz,"model_fft")
		call to_history("o1",okz,"model_fft")
		call to_history("label1","kz [1/km]","model_fft")
	
		call to_history("n2",nx2,"model_fft")
		call to_history("d2",dkx,"model_fft")
		call to_history("o2",okx,"model_fft")
		call to_history("label2","kx [1/km]","model_fft")		

		! WRITE MODEL FFT AFTER LOW-PASS FILTERING
		call sep_write(cabs(model_fft_filter),"model_fft_low")
	
		call to_history("n1",nz2,"model_fft_low")
		call to_history("d1",dkz,"model_fft_low")
		call to_history("o1",okz,"model_fft_low")
		call to_history("label1","kz [1/km]","model_fft_low")
	
		call to_history("n2",nx2,"model_fft_low")
		call to_history("d2",dkx,"model_fft_low")
		call to_history("o2",okx,"model_fft_low")
		call to_history("label2","kx [1/km]","model_fft_low")		

		! WRITE MODEL FFT AFTER HIGH-PASS FILTERING
		call sep_write(cabs(model_fft_filterc),"model_fft_high")
	
		call to_history("n1",nz2,"model_fft_high")
		call to_history("d1",dkz,"model_fft_high")
		call to_history("o1",okz,"model_fft_high")
		call to_history("label1","kz [1/km]","model_fft_high")
	
		call to_history("n2",nx2,"model_fft_high")
		call to_history("d2",dkx,"model_fft_high")
		call to_history("o2",okx,"model_fft_high")
		call to_history("label2","kx [1/km]","model_fft_high")		

		! WRITE FILTER
		call sep_write(filter,"filter")
	
		call to_history("n1",nz2,"filter")
		call to_history("d1",dkz,"filter")
		call to_history("o1",okz,"filter")
	
		call to_history("n2",nx2,"filter")
		call to_history("d2",dkx,"filter")
		call to_history("o2",okx,"filter")

		! WRITE FILTER COMPLEMENT
		call sep_write(filterc,"filterc")
	
		call to_history("n1",nz2,"filterc")
		call to_history("d1",dkz,"filterc")
		call to_history("o1",okz,"filterc")
	
		call to_history("n2",nx2,"filterc")
		call to_history("d2",dkx,"filterc")
		call to_history("o2",okx,"filterc")	
	 
		
	end if			

 	call sep_close()

end program 

