program scale_mixing

	use sep
    use fourier
	
	implicit none	
	
  	integer									  :: nz2,nx2,n3,n_left,n_ext,ne_index
	integer									  :: ikz,ikx,ntaper,index_okx,index_okz
	real								      :: oz,ox,dz,dx
	real									  :: dkz,dkx,okz,okx,dk_taper
	real									  :: k,kz,kx,k_taper,k_cut,k_big,k_small
	real									  :: sign1,sign2
  	real,dimension(:,:),allocatable  		  :: filter,grad_bg,grad_pert,grad_sum,grad_low,grad_high
  	complex,dimension(:,:),allocatable		  :: grad_sum_fft,grad_sum_fft_filter,cgrad_low
	real,dimension(:,:,:),allocatable		  :: gradient_in
	logical									  :: debug
		
	call sep_init(SOURCE)

	! READ FROM HISTORY
	call from_history("n1",nz2)
 	call from_history("d1",dz)	
 	call from_history("o1",oz)
 	
 	call from_history("n2",nx2)
 	call from_history("d2",dx)	
 	call from_history("o2",ox) 
 	
	call from_history("n3",n3)
	n_ext = n3 - 1 		

	call from_param("debug",debug,.false.)
	if (debug) write(0,*) "saving intermediate steps"	

	! ALLOCATE
 	allocate(gradient_in(nz2,nx2,n3))
	allocate(grad_bg(nz2,nx2))
	allocate(grad_pert(nz2,nx2))	
	allocate(grad_sum(nz2,nx2))
	allocate(filter(nz2,nx2))
	allocate(grad_sum_fft(nz2,nx2))
	allocate(grad_sum_fft_filter(nz2,nx2))
	allocate(cgrad_low(nz2,nx2))
	allocate(grad_low(nz2,nx2))			
	allocate(grad_high(nz2,nx2))			

	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! READ AND SUM GRADIENTS !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	! READ TOTAL GRADIENT
	call sep_read(gradient_in)
	
	! EXTRACT BACKGROUND GRADIENT
	grad_bg = gradient_in(:,:,1)

	! EXTRACT NON-EXTENDED PERTURBATION GRADIENT
	ne_index = (n_ext - 1) / 2 + 2
	grad_pert = gradient_in(:,:,ne_index)	
	
	! SUM THE TWO GRADIENTS
	grad_sum = grad_pert + grad_bg 

	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! INITIALIZE FFT / FILTER PARAMETERS !!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

 	! FFT PARAMETERS FOR Z
	sign1 = 1.0 
  	dkz = 1./((nz2-1)*dz)
	n_left = (nz2-(nz2+1)/2)
	okz = -1.0 * n_left * dkz 	
	write(0,*) "dkz",dkz,"[1/km]"
	
 	! FFT PARAMETERS FOR X
	sign2 = 1.0 
  	dkx = 1./((nx2-1)*dx)
	n_left = (nx2-(nx2+1)/2)
	okx = -1.0 * n_left * dkx 	
	write(0,*) "dkx",dkx,"[1/km]"	
 	
	! WAVENUMBER CUTOFF
	! every point in the kz/kx domain that has a |k| > k_cutoff should be multiplied by 0
	call from_param("k_cut",k_cut)
	call from_param("ntaper",ntaper) 

	! COMPUTE THE WIDTH (IN WAVENUMBER UNITS) OF THE TAPER ZONE
	dk_taper = ntaper * max(dkz,dkx)

	write(0,*) "cutoff wavenumber:",k_cut,"[1/km]"
	write(0,*) "taper length:",ntaper,"[samples]"	
	write(0,*) "taper length:",dk_taper,"[1/km]"	

	! CUTOFFS
	k_big = k_cut
	k_small = k_cut - dk_taper

	! CHECK
	if (dk_taper .ge. k_cut) then 
		write(0,*)"WARNING: dk_taper greater than k_cut !!!"
		write(0,*)"WARNING: will terminate program !!!"		
		stop
	end if

	! INITIALIZE FILTER TO ZERO
	filter = 0.0

	! APPLY FFT TO GRADIENT 
	grad_sum_fft = grad_sum
 	call ft_init(nz2,nx2)
 	call ft1axis(.false.,sign1,grad_sum_fft)
 	call ft2axis(.false.,sign2,grad_sum_fft) 

	write(0,*) "max grad_sum_fft",maxval(cabs(grad_sum_fft))
 	
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!! COMPUTE AND APPLY FFT LOW-PASS FILTER !!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	! INDEX FOR DC COMPONENT
	index_okz = nz2-(nz2+1)/2+1
	index_okx = nx2-(nx2+1)/2+1	

	! COMPUTE LOW-PASS FILTER
	do ikz = 1,nz2

 		kz = (ikz - index_okz) * dkz			
		do ikx = 1,nx2

 			kx = (ikx - index_okx) * dkx
 			k = sqrt(kz*kz+kx*kx) 

  			if (k .lt. k_small) then  
 				filter(ikz,ikx) = 1.0 
 			else if ((k .ge.  k_small) .and. (k .lt.  k_big)) then
 				filter(ikz,ikx) = cos(3.14159265359/2.0 * (k-k_small)/(k_big-k_small))**2
 			end if

		end do

  	end do 	
	
	! APPLY FFT LOW-PASS FILTER
	grad_sum_fft_filter = grad_sum_fft * filter
	
	write(0,*) "max grad_sum_fft_filter",maxval(cabs(grad_sum_fft_filter))	
	
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! APPLY IFFT !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!	

	! APPLY IFFT
 	cgrad_low = grad_sum_fft_filter

 	call ft1axis(.true.,sign1,cgrad_low)
 	call ft2axis(.true.,sign2,cgrad_low) 	
	grad_low = real(cgrad_low)

 	write(0,*) "max cgrad_low",maxval(cabs(cgrad_low))

	! COMPUTE COMPLEMENT MODEL
  	grad_high = grad_sum - grad_low

	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! WRITE OUTPUT !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	! WRITE TOTAL GRADIENT AFTER FILTERING
 	gradient_in(:,:,1) = grad_low
 	gradient_in(:,:,ne_index) = grad_high

	call sep_write(gradient_in)

 	call to_history("n1",nz2)
 	call to_history("d1",dz)
 	call to_history("o1",oz)
 	call to_history("label1","z [km]")

 	call to_history("n2",nx2)
 	call to_history("d2",dx)
 	call to_history("o2",ox)
 	call to_history("label2","x [km]") 	
 
 	call to_history("n3",n3)
 	call to_history("d3",1.0)
 	call to_history("o3",0.0)
 	 
	if (debug) then
	
		write(0,*) "saving intermediate steps"
	
		!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! DEBUG !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 	 
	 
		! WRITE FFT BEFORE FILTERING
		call sep_write(cabs(grad_sum_fft),"grad_fft")
	
		call to_history("n1",nz2,"grad_fft")
		call to_history("d1",dkz,"grad_fft")
		call to_history("o1",okz,"grad_fft")
		call to_history("label1","kz [1/km]","grad_fft")
	
		call to_history("n2",nx2,"grad_fft")
		call to_history("d2",dkx,"grad_fft")
		call to_history("o2",okx,"grad_fft")
		call to_history("label2","kx [1/km]","grad_fft")

		! WRITE FFT AFTER LOW-PASS FILTER
		call sep_write(cabs(grad_sum_fft_filter),"grad_fft_filter")
	
		call to_history("n1",nz2,"grad_fft_filter")
		call to_history("d1",dkz,"grad_fft_filter")
		call to_history("o1",okz,"grad_fft_filter")
		call to_history("label1","kz [1/km]","grad_fft_filter")
	 
		call to_history("n2",nx2,"grad_fft_filter")
		call to_history("d2",dkx,"grad_fft_filter")
		call to_history("o2",okx,"grad_fft_filter")
		call to_history("label2","kx [1/km]","grad_fft_filter")
	 
		! WRITE FILTER
		call sep_write(filter,"filter")
	
		call to_history("n1",nz2,"filter")
		call to_history("d1",1.0,"filter")
		call to_history("o1",0.0,"filter")
	
		call to_history("n2",nx2,"filter")
		call to_history("d2",1.0,"filter")
		call to_history("o2",0.0,"filter")
	
		! WRITE LOW-PASSED MODEL
		call sep_write(grad_low,"grad_low")
	
		call to_history("n1",nz2,"grad_low")
		call to_history("d1",dz,"grad_low")
		call to_history("o1",oz,"grad_low")
		call to_history("label1","z [km]","grad_low")

		call to_history("n2",nx2,"grad_low")
		call to_history("d2",dx,"grad_low")
		call to_history("o2",ox,"grad_low")
		call to_history("label2","x [km]","grad_low")
	 
		! WRITE HIGH-PASSED MODEL
		call sep_write(grad_high,"grad_high")
	
		call to_history("n1",nz2,"grad_high")
		call to_history("d1",dz,"grad_high")
		call to_history("o1",oz,"grad_high")
		call to_history("label1","z [km]","grad_high")

		call to_history("n2",nx2,"grad_high")
		call to_history("d2",dx,"grad_high")
		call to_history("o2",ox,"grad_high")
		call to_history("label2","x [km]","grad_high") 
		
	end if			

 	call sep_close()

end program 

