program scale_mixing_gradient

	use sep
    use fourier
	
	implicit none	
	
  	integer									  :: nz,nx,nz2,nx2,n3,n_left,n_ext,ne_index
	integer									  :: iz,ix,fft_pad
	integer									  :: ikz,ikx,ntaper,index_okx,index_okz,buttord
	real								      :: oz,ox,dz,dx
	real									  :: dkz,dkx,okz,okx,dk_taper
	real									  :: k,kz,kx,k_taper,k_cut,k_big,k_small
	real									  :: sign1,sign2
  	real,dimension(:,:),allocatable  		  :: filter,grad_bg,grad_pert,grad_sum,grad_low,grad_high
  	complex,dimension(:,:),allocatable		  :: grad_sum_fft,grad_sum_fft_filter,cgrad_low
	real,dimension(:,:,:),allocatable		  :: gradient
	logical									  :: debug
		
	call sep_init(SOURCE)

	! READ FROM HISTORY
	call from_history("n1",nz)
 	call from_history("d1",dz)	
 	call from_history("o1",oz)
 	
 	call from_history("n2",nx)
 	call from_history("d2",dx)	
 	call from_history("o2",ox) 
 	
	call from_history("n3",n3)
	
	call from_param("fft_pad",fft_pad,500)
	call from_param("debug",debug,.false.)
	
	call from_param("buttord",buttord,1)
	write(0,*) "Butterworth wavenumber low-pass filter"
	write(0,*) "filter order:",buttord		

	nz2 = nz + 2 * fft_pad
	nx2 = nx + 2 * fft_pad	
	n_ext = n3 - 1 

	! ALLOCATE
 	allocate(gradient(nz,nx,n3))
	allocate(grad_bg(nz2,nx2))
	allocate(grad_pert(nz2,nx2))	
	allocate(grad_sum(nz2,nx2))
	allocate(filter(nz2,nx2))
	allocate(grad_sum_fft(nz2,nx2))
	allocate(grad_sum_fft_filter(nz2,nx2))
	allocate(cgrad_low(nz2,nx2))
	allocate(grad_low(nz2,nx2))			
	allocate(grad_high(nz2,nx2))			

	! READ TOTAL GRADIENT
	call sep_read(gradient)

	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! EXTRACT BACKGROUND GRADIENT !!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	! EXTRACT BACKGROUND GRADIENT
	grad_bg(fft_pad+1:fft_pad+nz,fft_pad+1:fft_pad+nx) = gradient(:,:,1)
	
	! PAD THE BACKGROUND
    do ix = 1,nx
        do iz = 1,fft_pad
            grad_bg(iz,ix+fft_pad)      	   = grad_bg(fft_pad+1,ix+fft_pad)
            grad_bg(fft_pad+nz+iz,ix+fft_pad)  = grad_bg(fft_pad+nz,ix+fft_pad)
        end do
    end do

    do ix = 1,fft_pad
	    do iz = 1,nz2
            grad_bg(iz,ix)             = grad_bg(iz,fft_pad+1)
            grad_bg(iz,fft_pad+nx+ix)  = grad_bg(iz,fft_pad+nx)
        end do
    end do

	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! EXTRACT BACKGROUND GRADIENT !!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	! EXTRACT NON-EXTENDED PERTURBATION GRADIENT
	ne_index = (n_ext - 1) / 2 + 2
	grad_pert(fft_pad+1:fft_pad+nz,fft_pad+1:fft_pad+nx) = gradient(:,:,ne_index)	
	
	! PAD THE PERTURBATION
    do ix = 1,nx
        do iz = 1,fft_pad
            grad_pert(iz,ix+fft_pad)      		 = grad_pert(fft_pad+1,ix+fft_pad)
            grad_pert(fft_pad+nz+iz,ix+fft_pad)  = grad_pert(fft_pad+nz,ix+fft_pad)
        end do
    end do

    do ix = 1,fft_pad
	    do iz = 1,nz2
            grad_pert(iz,ix)             = grad_pert(iz,fft_pad+1)
            grad_pert(iz,fft_pad+nx+ix)  = grad_pert(iz,fft_pad+nx)
        end do
    end do	
	
	! SUM THE TWO GRADIENTS
	grad_sum = grad_pert + grad_bg 

	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! INITIALIZE FFT !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

 	! FFT PARAMETERS FOR Z
	sign1 = 1.0 
  	dkz = 1./((nz2-1)*dz)
	n_left = (nz2-(nz2+1)/2)
	okz = -1.0 * n_left * dkz 	
	write(0,*) "dkz",dkz,"[1/km]"
	
 	! FFT PARAMETERS FOR X
	sign2 = 1.0 
  	dkx = 1./((nx2-1)*dx)
	n_left = (nx2-(nx2+1)/2)
	okx = -1.0 * n_left * dkx 	
	write(0,*) "dkx",dkx,"[1/km]"	
	
	! APPLY FFT TO GRADIENT 
	grad_sum_fft = grad_sum
 	call ft_init(nz2,nx2)
 	call ft1axis(.false.,sign1,grad_sum_fft)
 	call ft2axis(.false.,sign2,grad_sum_fft) 

	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! INITIALIZE FILTER !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 	 	
 	
	! WAVENUMBER CUTOFF
	call from_param("k_cut",k_cut)
	write(0,*) "cutoff wavenumber:",k_cut,"[1/km]"

	! INITIALIZE FILTER TO ZERO
	filter = 0.0

	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!! COMPUTE FFT LOW-PASS FILTER !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	! INDEX FOR DC COMPONENT
	index_okz = nz2-(nz2+1)/2+1
	index_okx = nx2-(nx2+1)/2+1	

	! COMPUTE LOW-PASS FILTER
	do ikz = 1,nz2

 		kz = (ikz - index_okz) * dkz			
		do ikx = 1,nx2

 			kx = (ikx - index_okx) * dkx
 			k = sqrt(kz*kz+kx*kx) 

			! compute Butterworth filter
			filter(ikz,ikx) = 1 / sqrt(1 + (k/k_cut)**(2*buttord))

		end do

  	end do 	
	
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!! APPLY FFT LOW-PASS FILTER !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	
	grad_sum_fft_filter = grad_sum_fft * filter
	
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! APPLY IFFT !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!	

	! APPLY IFFT
 	cgrad_low = grad_sum_fft_filter

 	call ft1axis(.true.,sign1,cgrad_low)
 	call ft2axis(.true.,sign2,cgrad_low) 	
	grad_low = real(cgrad_low)

	! COMPUTE COMPLEMENT GRADIENT
  	grad_high = grad_sum - grad_low

	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! WRITE OUTPUT !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	! WRITE TOTAL GRADIENT AFTER FILTERING
 	gradient(:,:,1) = grad_low(1+fft_pad:nz+fft_pad,1+fft_pad:nx+fft_pad)
 	gradient(:,:,ne_index) = grad_high(1+fft_pad:nz+fft_pad,1+fft_pad:nx+fft_pad)

	call sep_write(gradient)

 	call to_history("n1",nz)
 	call to_history("d1",dz)
 	call to_history("o1",oz)
 	call to_history("label1","z [km]")

 	call to_history("n2",nx)
 	call to_history("d2",dx)
 	call to_history("o2",ox)
 	call to_history("label2","x [km]") 	
 
 	call to_history("n3",n3)
 	call to_history("d3",1.0)
 	call to_history("o3",0.0)
 	 
	if (debug) then
	
		write(0,*) "saving intermediate steps"
	
		!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! DEBUG !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 	 
	 
		! WRITE SUM OF TWO GRADIENTS BEFORE FILTERING
		call sep_write(grad_sum(fft_pad+1:fft_pad+nz,fft_pad+1:fft_pad+nx),"grad_sum")

		call to_history("n1",nz,"grad_sum")
		call to_history("d1",dz,"grad_sum")
		call to_history("o1",oz,"grad_sum")
		call to_history("label1","z [km]","grad_sum")

		call to_history("n2",nx,"grad_sum")
		call to_history("d2",dx,"grad_sum")
		call to_history("o2",ox,"grad_sum")
		call to_history("label2","x [km]","grad_sum")
 
		call to_history("n3",1,"grad_sum")
		call to_history("d3",0.0,"grad_sum")
		call to_history("o3",0.0,"grad_sum")
		
		! WRITE LOW-PASSED SUM
		call sep_write(grad_low(fft_pad+1:fft_pad+nz,fft_pad+1:fft_pad+nx),"grad_low")

		call to_history("n1",nz,"grad_low")
		call to_history("d1",dz,"grad_low")
		call to_history("o1",oz,"grad_low")
		call to_history("label1","z [km]","grad_low")

		call to_history("n2",nx,"grad_low")
		call to_history("d2",dx,"grad_low")
		call to_history("o2",ox,"grad_low")
		call to_history("label2","x [km]","grad_low")
 
		call to_history("n3",1,"grad_low")
		call to_history("d3",0.0,"grad_low")
		call to_history("o3",0.0,"grad_low")		

		! WRITE HIGH-PASSED SUM
		call sep_write(grad_high(fft_pad+1:fft_pad+nz,fft_pad+1:fft_pad+nx),"grad_high")

		call to_history("n1",nz,"grad_high")
		call to_history("d1",dz,"grad_high")
		call to_history("o1",oz,"grad_high")
		call to_history("label1","z [km]","grad_high")

		call to_history("n2",nx,"grad_high")
		call to_history("d2",dx,"grad_high")
		call to_history("o2",ox,"grad_high")
		call to_history("label2","x [km]","grad_high")
 
		call to_history("n3",1,"grad_high")
		call to_history("d3",0.0,"grad_high")
		call to_history("o3",0.0,"grad_high")		

		! WRITE FFT BEFORE FILTERING
		call sep_write(cabs(grad_sum_fft),"grad_fft")
	
		call to_history("n1",nz2,"grad_fft")
		call to_history("d1",dkz,"grad_fft")
		call to_history("o1",okz,"grad_fft")
		call to_history("label1","kz [1/km]","grad_fft")
	
		call to_history("n2",nx2,"grad_fft")
		call to_history("d2",dkx,"grad_fft")
		call to_history("o2",okx,"grad_fft")
		call to_history("label2","kx [1/km]","grad_fft")

		! WRITE FFT AFTER LOW-PASS FILTER
		call sep_write(cabs(grad_sum_fft_filter),"grad_fft_filter")
	
		call to_history("n1",nz2,"grad_fft_filter")
		call to_history("d1",dkz,"grad_fft_filter")
		call to_history("o1",okz,"grad_fft_filter")
		call to_history("label1","kz [1/km]","grad_fft_filter")
	 
		call to_history("n2",nx2,"grad_fft_filter")
		call to_history("d2",dkx,"grad_fft_filter")
		call to_history("o2",okx,"grad_fft_filter")
		call to_history("label2","kx [1/km]","grad_fft_filter")
	 
		! WRITE FILTER
		call sep_write(filter,"filter")
	
		call to_history("n1",nz2,"filter")
		call to_history("d1",dkz,"filter")
		call to_history("o1",okz,"filter")
	
		call to_history("n2",nx2,"filter")
		call to_history("d2",dkx,"filter")
		call to_history("o2",okx,"filter")
	
	end if			

 	call sep_close()

end program 

