program stack_batch

	use sep
	use omp_lib
	
  	implicit none

	integer					  				  :: nz,nx,nhz,nhx,ntau,ns,next
	integer					 				  :: is,iext,ix,iz
	real					  				  :: oz,dz,ox,dx,ohz,dhz,ohx,dhx,otau,dtau,os,ds
	real,dimension(:,:,:),allocatable         :: data,model
	integer					  				  :: nthreads
	integer*8				  				  :: srite,sreed,rc,wc,nbytes,bytes

  	call sep_init()
  	
	call from_history("n1",nz)
	call from_history("d1",dz)	
	call from_history("o1",oz)	

	call from_history("n2",nx)
	call from_history("d2",dx)
	call from_history("o2",ox)

	call from_history("n3",nhz)
	call from_history("d3",dhz)
	call from_history("o3",ohz)

	call from_history("n4",nhx)
	call from_history("d4",dhx)
	call from_history("o4",ohx)

	call from_history("n5",ntau)
	call from_history("d5",dtau)
	call from_history("o5",otau)

	call from_history("n6",ns)
	call from_history("d6",ds)
	call from_history("o6",os)

	call from_param("nthreads",nthreads,24)	  

 	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	write(0,'(a)') "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"!
	write(0,'(a)') "!!!!!!!!!!!!!!!!! PARALLEL COMPUTING PARAMETERS !!!!!!!!!!!!!!!!!!!!"!
	write(0,'(a)') "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	write(0,'(a,i4,a)') "NUMBER OF CORES: ",omp_get_num_procs()," [cores]"
	nthreads = omp_get_num_procs()
	write(0,'(a,i4,a)') "REQUIRED NUMBER OF CORES: ",nthreads," [cores]"
	CALL OMP_SET_NUM_THREADS(nthreads)
	!!$OMP PARALLEL
	!write(0,*) omp_get_num_threads()
	!!$OMP END PARALLEL
	write(0,'(a)') "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"!
	write(0,'(a)') "                                                                    "!

	! DEFINE EXTENSION SIZE
	next = nhz * nhx * ntau

	! ALLOCATE MODEL AND DATA
	allocate(model(nz,nx,next))
	allocate(data(nz,nx,next))

	model = 0.0
	data = 0.0

	if (next .gt. 1) then

		! STACK MODEL --> DATA
		do is = 1,ns

			! READ MODEL
			bytes = 4
			nbytes = bytes * nx * nz * next  
			rc = sreed("in",model,nbytes)

			! CHECK IF MODEL WAS READ CORRECTLY
			if(rc .ne. nbytes) then
				write(0,*) "error reading model"
				stop
			end if	

			!$OMP PARALLEL DO private(iext,ix,iz)
			do iext = 1,next
				do ix = 1,nx
					do iz = 1,nz
						data(iz,ix,iext) = data(iz,ix,iext) + model(iz,ix,iext) 
					end do
				end do
			end do
       		!$OMP END PARALLEL DO
		end do
	else
		! STACK MODEL --> DATA
		do is = 1,ns
		
			! READ MODEL
			bytes = 4
			nbytes = bytes * nx * nz * next  
			rc = sreed("in",model,nbytes)

			! CHECK IF MODEL WAS READ CORRECTLY
			if(rc .ne. nbytes) then
				write(0,*) "error reading model"
				stop
			end if	
		
			do iext = 1,next
				!$OMP PARALLEL DO private(ix,iz)			
				do ix = 1,nx
					do iz = 1,nz
						data(iz,ix,iext) = data(iz,ix,iext) + model(iz,ix,iext)
					end do
				end do
       				!$OMP END PARALLEL DO
			end do
		end do	
	end if

	! WRITE DATA
	nbytes = bytes * nz * nx * next
	wc = srite("out",data,nbytes)

	call to_history("n1",nz)
	call to_history("d1",dz)	
	call to_history("o1",oz)	

	call to_history("n2",nx)
	call to_history("d2",dx)
	call to_history("o2",ox)

	call to_history("n3",nhz)
	call to_history("d3",dhz)
	call to_history("o3",ohz)

	call to_history("n4",nhx)
	call to_history("d4",dhx)
	call to_history("o4",ohx)

	call to_history("n5",ntau)
	call to_history("d5",dtau)
	call to_history("o5",otau)
	
	call to_history("n6",1)
	call to_history("d6",0.0)
	call to_history("o6",0.0)

	call sep_close()					
	
end program
